/**
 * @class Sb.Editor
 * <p>This singleton object, which is represent core of the editor.</p>
 * @singleton
 */
Ext.define('Sb.Editor', function () {
	var controllers = new Ext.util.MixedCollection();

	Ext.onReady(function () {
		Sb.Editor.init();
	});

	return {
		singleton: true,

		/**
		 * Initialize the Editor and its components
		 */
		init: function () {
			Ext.getBody().clean();

			if ("addEventListener" in window) {
				window.addEventListener("hashchange", Sb.EcwidWidgetsManager.onHashChange, false);
			} else if ("attachEvent" in window) {
			// for IE
				window.attachEvent("onhashchange", Sb.EcwidWidgetsManager.onHashChange);
			}

			Ext.BLANK_IMAGE_URL = Sb.Config.blankImageUrl;
			Ext.QuickTips.init();

			Ext.state.Manager.setProvider(
				new Ext.state.CookieProvider({
					path: "/",
					expires: new Date(new Date().getTime() + 311040000000) //10 years
				})
			);

			// :TODO: Necessary to rework our DD for correct work with default value of notifyOccluded options
			Ext.dd.DragDropMgr.notifyOccluded = true;

			// Init Editor controllers
			Ext.each(['Pages', 'Blogs', 'DesignPresets'], function (controllerName) {
				var controller = this.getController(controllerName);
				controller.init();
			}, this);

			this.windowManager = new Sb.WindowManager();
			/**
			 * The site model
			 * @type Sb.model.Site
			 * @property model
			 */
			this.model = new Sb.model.Site();
			this.model.on('beforeload', function () {
				Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgInitialize'));
			});
			this.model.on('load', function () {
				Sb.Editor.Ui.init();
				Sb.Loading.hide();

				controllers.each(function (controller) {
					controller.onLaunch();
				});

				if (Ext.state.Manager.get('isShowDashboard') && Sb.Editor.model.get('isPublished')) {
					Sb.Editor.Ui.mainWindow.showDashboard();
				}

				if (Sb.Config.gettingStartedDemoEnabled && Ext.state.Manager.get('showGettingStarted') !== false) {
					this.showGettingStarted();
				}
			}, this, {single: true});

			this.model.on('load', function () {
				var model = Sb.Editor.model,
					checkPages = Sb.Limits.checkPages(model),
					checkWidgets = Sb.Limits.checkWidgets(model),
					checkLocales = Sb.Limits.checkLocales(model, model.get('locales')),
					listWidgetsExceedsLimits = [];

				if (checkPages.isExceeded) {
					Sb.Editor.Ui.mainWindow.statusBar.addMessageWarning(
						Sb.Locale.get(
							'maxPagesNumberLimitExceeded',
							checkPages.count,
							checkPages.limit
						)
					);
				}

				if (checkLocales.isExceeded) {
					Sb.Editor.Ui.mainWindow.statusBar.addMessageWarning(
						Sb.Locale.get(
							'maxAdditionalLocalesLimitExceeded',
							model.get('locales').length - 1,
							checkLocales.limit
						)
					);
				}

				if (checkWidgets.isExceeded) {
					Ext.iterate(checkWidgets.widgets, function(widgetTypeName, widgetData) {
						listWidgetsExceedsLimits.push(Sb.WidgetTypeManager.get(widgetTypeName).getTranslation('title'));
					});

					Sb.Editor.Ui.mainWindow.statusBar.addMessageWarning(
						Sb.Locale.get(
							'modulesLimitExceeded',
							listWidgetsExceedsLimits.join(', ')
						)
					);
				}
			}, this);

			this.siteView = new Sb.SiteView({
				renderTo: Ext.getBody()
			});

			if (Sb.isNeedUpgradeSite) {
				this.upgradeSite();
			} else {
				Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgSiteStructure'));
				this.model.load();
			}

			this.documents = new Sb.model.Document();

			var el = Ext.get('sb-top-message');
			if (el) {
				Sb.TopMessageBox.show(el.dom.innerHTML);
			}
		},

		upgradeSite: function () {
			Sb.SiteService.upgrade(function (result) {
				if (!result.success) {
					var win = new Sb.Window({
						title: Sb.Locale.get('upgradeErrorTitle'),
						html: result.error,
						width: 450,
						closable: false,
						draggable: false,
						modal: true
					});
					win.show();
					return;
				}

				Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgSiteStructure'));
				this.model.load();
			}, this);
		},

		/**
		 * Start over
		 * @param {Boolean} doImport
		 */
		startOver: function (doImport) {
			Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgStartingOver'));
			Sb.Loading.show();
			Sb.SiteService.backToStub(function () {
				window.location = Sb.Config.baseUrl + '/Start' + (doImport ? '?import' : '');
			});
		},

		/**
		 * Open help window
		 * @param {String} [articleUrl]
		 */
		openHelp: function (articleUrl) {
			if (!window.open(this.getHelpUrl(articleUrl), '_blank')) {
				Sb.Editor.showLocalizedWarning('helpPopupBlocked');
			}
		},

		getHelpUrl: function (articleUrl) {
			if (undefined === articleUrl) {
				articleUrl = '';
			} else if ('/' != articleUrl[0]) {
				articleUrl = '/' + articleUrl;
			}
			return Sb.Config.helpUrl.replace('%%CONTEXT%%', encodeURIComponent('SiteBuilder' + articleUrl));
		},

		showLocalizedWarning: function (msgLocaleKey) {
			Ext.Msg.show({
				title	: Sb.Locale.get('warningMessageBoxTitle'),
				msg		: Sb.Locale.get(msgLocaleKey),
				closable: false,
				buttons	: Ext.Msg.OK
			});
		},

		showPublishErrorDetails: function () {
			var mainWindow = Sb.Editor.Ui.mainWindow,
				publishResult = mainWindow.progressBar.publishResult,
				msg = publishResult._statusText + '\n',
				buttons = [],
				win;

			Ext.each(publishResult._messagesDetailed, function (item) {
				msg += item.message + '\n';
			});
			if (publishResult._isSendToAdmin) {
				buttons.push({
					text: Sb.Locale.get('sendToAdmin'),
					handler: function () {
						this.disable();
						Sb.SiteService.msgToAdmin(Ext.getCmp('sb-publish-details-text').getValue(), function (result) {
							win.close();
							if (result === false) {
								mainWindow.statusBar.setMessageError(Sb.Locale.get('emailNotSentToAdmin', Sb.Config.general.product));
							} else {
								mainWindow.statusBar.setMessageInfo(Sb.Locale.get('emailSentToAdmin'));
							}
						});
					}
				});
			}

			win = new Sb.Window({
				layout	: 'fit',
				width	: 500,
				height	: 300,
				plain	: true,
				title	: Sb.Locale.get('publishDetailsTitle'),
				modal	: true,

				items: {
					xtype: 'textarea',
					id: 'sb-publish-details-text',
					value: msg,
					readOnly: true
				},

				buttons: buttons
			});
			win.show(this);
		},

		showGettingStarted: function () {
			var protocol = Ext.isSecure ? 'https' : 'http',
				url = Sb.Config.gettingStartedDemoUrl;

			Ext.each(Sb.WidgetTypeManager.get('Video').providersSource, function (provider) {
				if (url.indexOf(provider.host + '/') == -1) {
					return true;
				}

				var videoMatch = url.match(provider.videoRe);
				if (null == videoMatch) {
					return false;
				}

				url = provider.movie.replace('{videoId}', videoMatch[1]).replace('{lang}', Sb.Config.locale);
				return false;
			});

			var win = new Sb.Window({
				modal: true,
				resizable: false,
				cls: 'sb-getting-started',
				width: 599,
				autoHeight: true,
				defaults: {
					border: false
				},
				items: [{
					xtype: 'box',
					cls: 'sb-getting-started-greeting-block',
					html: Sb.Locale.get('gettingStartedMessage', Sb.Config.general.product)
				}, {
					xtype: 'box',
					cls: 'sb-getting-started-player-block',
					height: 288,
					html: '<object id="id' + (++Ext.Component.AUTO_ID) + '" width="480" height="274" classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" ' +
							'codebase="' + protocol + '://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,115,0">' +
						'<param name="movie" value="' + url + '" />' +
						'<param name="allowFullScreen" value="true" />' +
						'<param name="allowscriptaccess" value="always" />' +
						'<embed src="' + url + '" type="application/x-shockwave-flash"' +
							' width="480" height="274" allowscriptaccess="always" allowfullscreen="true"></embed>' +
						'</object>'
				}, {
					cls: 'sb-getting-started-tools-block',
					defaults: {
						border: false
					},
					items: [{
						cls: 'sb-getting-started-tools-checkbox-block',
						items: [{
							xtype: 'checkbox',
							id: 'showGettingStarted',
							boxLabel: Sb.Locale.get('showGettingStarted'),
							checked: (Ext.state.Manager.get('showGettingStarted')) ? true : false
						}]
					}, {
						cls: 'sb-getting-started-tools-button-block',
						items: [{
							xtype: 'button',
							cls: 'sb-getting-started-close',
							width: 75,
							text: Sb.Locale.get('closeGettingStarted'),
							handler: function () {
								win.close();
							}
						}]
					}]
				}],
				listeners: {
					scope: this,
					close: function () {
						Ext.state.Manager.set('showGettingStarted', Ext.getCmp('showGettingStarted').checked);
					}
				}
			});
			win.show();
		},

		startFacebookIntegration: function () {
			var url = Sb.Config.general.facebookApplicationUrl;
			url += (url.split('?').length == 1 ? '?' : '&') +
				'siteUrl=' + encodeURIComponent(Sb.Editor.model.get('publishWebSiteUrl'));

			if (!window.open(url, '_blank')) {
				Sb.Editor.showLocalizedWarning('helpPopupBlocked');
			}
		},

		isDemoMode: function () {
			return Sb.Editor.model.get('isTrial') || Sb.Config.isDemo;
		},

		/**
		 * Returns a reference to a {@link Sb.Controller controller} with the given name
		 * @param name {String}
		 */
		getController: function (name) {
			var controller = controllers.get(name);

			if (!controller) {
				controller = new Sb.controller[name]({
					id: name
				});
				controllers.add(controller);
			}

			return controller;
		}
	};
});