/**
 * @class Sb.ResourceManager
 * @singleton
 */
Ext.define('Sb.ResourceManager', function () {
	var readyStates		= {},
		UNINITIALIZED	= 0,
		LOADING			= 1,
		LOADED			= 2,
		ERROR			= 3,
		queue			= [];

	/**
	 * Resource success loading callback
	 * @private
	 * @param {Object} resource Resource config
	 */
	function onResourceLoaded(resource) {
		readyStates[resource.url] = LOADED;
		if (resource.onLoad) {
			resource.onLoad.call(resource.scope);
		}

		processQueue();
	}

	/**
	 * Processes current resources load state in queue and
	 */
	function processQueue() {
		var i, item, j, resources;

		if (queue.length === 0) {
			return;
		}

		i = 0;
		do {
			item = queue[i];
			resources = item.resources;
			j = 0;

			do {
				if (readyStates[resources[j].url] === LOADED) {
					resources.splice(j, 1);
				} else {
					j++;
				}
			} while (j < resources.length);

			if (item.resources.length === 0) {
				queue.splice(i, 1);
				Ext.callback(item.callback, item.scope);
			} else {
				i++;
			}
		} while (i < queue.length);
	}

	/**
	 * Resource error loading callback
	 * @private
	 * @param {Object} resource Resource config
	 */
	function onResourceLoadError(resource) {
		readyStates[resource.url] = ERROR;
		if (resource.onError) {
			resource.onError.call(resource.scope);
		}

		if (Sb.Config.isDebug && console && console.error) {
			console.error("Failed loading '" + resource.url + "', please verify that the file exists");
		}
	}

	/**
	 * Inject a script/link element to document's head, call onLoad and onError accordingly
	 * @private
	 * @param {Object} resource Resource config
	 * @param {Function} onLoad The callback function to be executed on success load
	 * @param {Function} onError The callback function to be executed on error load
	 * @param {Object} [scope] The execution scope (<code>this</code>) of the onLoad and onError functions
	 */
	function loadResource(resource, onLoad, onError, scope) {
		if (readyStates[resource.url] == LOADING) {
			return;
		}
		/* :TODO: smikhaleyv :2011/05/26: actions before reload a resource's
		if (readyStates[resource.url] == LOADED) {
		}
		*/
		var el,
			head		= document.head || document.getElementsByTagName('head')[0],
			resourceUrl	= resource.url,
			isLoaded	= false,
			onLoadFn	= function () {
				if (!isLoaded) {
					isLoaded = true;
					onLoad.call(scope);
				}
			};

		// protect against cache in debug mode
		if (Sb.Config.isDebug && resourceUrl.split('?').length == 1) {
			resourceUrl += '?' + (new Date()).getTime();
		}

		switch (resource.type) {
			case 'js':
				el		= document.createElement('script');
				el.type	= 'text/javascript';
				el.src	= resourceUrl;
				el.onreadystatechange = function () {
					if (this.readyState === 'loaded' || this.readyState === 'complete') {
						onLoadFn();
					}
				};
				el.onload	= onLoadFn;
				el.onerror	= onError;
				break;
			case 'css':
				el		= document.createElement('link');
				el.type	= 'text/css';
				el.rel	= 'stylesheet';
				el.href	= resourceUrl;
				break;
		}

		readyStates[resource.url] = LOADING;
		head.appendChild(el);

		if (resource.type === 'css') {
			onLoad.call(scope);
		}
	}

	return {
		singleton: true,

		/**
		 * Loads resources by the given urls and types.
		 * @param {Array} resources Array of required resources configurations.
		 * Example: object {url: ..., type: 'js'|'css', onLoad: function () {...}, onError: function () {...}, force: false}
		 * or string with resource url
		 * @param {Function} [callback] The callback function to be executed after all resources loaded
		 * @param {Object} [scope] The execution scope (<code>this</code>) of the callback function
		 */
		require: function (resources, callback, scope) {
			if (resources.length === 0) {
				Ext.callback(callback, scope);
				return;
			}

			Ext.each(resources, function (resource, index) {
				if (Ext.isString(resource)) {
					resource = {
						url: resource,
						force: false
					};
				}

				var urlParts = resource.url.split('?');

				// detect resource type if not specified
				if (!resource.type) {
					if (urlParts[0].match(/([^\/\\]+)\.(\w+)$/) ) {
						resource.type = RegExp.$2;
					}
				}

				resources[index] = resource;
			});

			queue.push({
				resources: resources,
				callback: callback,
				scope: scope
			});

			resources = resources.slice();
			Ext.each(resources, function (resource, index) {
				readyStates[resource.url] = readyStates[resource.url] || UNINITIALIZED;
				if (readyStates[resource.url] < LOADING || resource.force) {
					loadResource(
						resource,
						onResourceLoaded.createDelegate(this, [resource]),
						onResourceLoadError.createDelegate(this, [resource]),
						this
					);
				} else if (readyStates[resource.url] == ERROR) {
					onResourceLoadError(resource);
				}
			});

			processQueue();
		}
	};
});
