/**
 * @class Sb.SiteView
 * Listens the model for changes and renders site
 */
Ext.define('Sb.SiteView', {
	extend: 'Ext.Component',

	cls: 'sb-frame',

	onRender: function () {
		Sb.SiteView.superclass.onRender.apply(this, arguments);

		this.el.ddScrollConfig = {
			vthresh: 50,
			frequency: 10,
			increment: 100
		};
		Ext.dd.ScrollManager.register(this.el);

		this.el.on('scroll', this.reanchorEditPanel);
		this.el.on('contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
			type: 'site',
			region: 'site'
		});
		Ext.EventManager.onWindowResize(this.reanchorEditPanel, this);
		Sb.Editor.model.on('beforeload', this.destroyPage, this);
		Sb.Editor.model.on('load', this.onModelLoad, this);
		Sb.TopMessageBox.on('resize', function (mb, w, h) {
			this.el.setStyle('height', 'calc(100% - ' + h + 'px)');
		}, this);

		this.deferIds = {};
	},

	/**
	 * Make edit panel always in one place, where it should be,
	 * for example under widget.
	 *
	 * We need this as our current implementation of superpanels emulates
	 * body as div with scroll.
	 */
	reanchorEditPanel: function () {
		if (!Sb.Editor.Ui.editComponent) {
			return;
		}
		var editPanel = Sb.Editor.Ui.editComponent.getEditPanel(),
			stylePanel = Sb.Editor.Ui.stylePanel;

		if (editPanel && editPanel.component) {
			editPanel.reanchor(false);
		}

		if (stylePanel && stylePanel.component) {
			stylePanel.reanchor(false);
		}
	},

	/**
	 * Fires when widget drag start
	 */
	onWidgetStartDrag: function () {
		this.maskNavigations();
		Sb.Highlighting.show();
	},

	/**
	 * Add mask for all navigation widgets
	 */
	maskNavigations: function () {
		Sb.WidgetTypeManager.get('navigation').cache.each(function (model) {
			var widget = Ext.getCmp('sb-widget-' + model.data.uuid);
			if (widget) {
				widget.contentElement.mask();
			}
		});
	},

	/**
	 * Fires when widget drag end
	 * @param {Sb.editor.component.Base} [widget] or component
	 */
	onWidgetEndDrag: function (widget) {
		if (widget) {
			widget.controlsEl.hide();
		}

		this.unmaskNavigations();
		Sb.Highlighting.hide();
	},

	/**
	 * Remove mask from all navigation widgets
	 */
	unmaskNavigations: function () {
		Sb.WidgetTypeManager.get('navigation').cache.each(function (model) {
			var navigationWidget = Ext.getCmp('sb-widget-' + model.data.uuid);
			if (navigationWidget) {
				navigationWidget.contentElement.unmask();
			}
		});
	},

	/**
	 * Initial actions after model loaded
	 */
	onModelLoad: function () {
		var templateUrl = Sb.Config.templatesUrl + '/' + encodeURIComponent(Sb.Editor.model.data.template.code);
		Sb.ResourceManager.require([Sb.Utils.antiCache(templateUrl + '/css/style.css')], function () {
			this.el.removeClass('rtl');
			if (Sb.Editor.model.isRtl()) {
				Sb.ResourceManager.require([Sb.Utils.antiCache(templateUrl + '/css/style-rtl.css')], function () {
					this.el.addClass('rtl');
				}, this);
			}

			this.onColorSchemeChange();
			this.onFontChange();
			this.onFontColorChange();

			this.setCurrentPage(Sb.Editor.model.rootPage.getFirstVisiblePage());
			Sb.Loading.hide();
		}, this);

		Sb.Editor.model.design.on('colorschemechange', this.onColorSchemeChange, this);
		Sb.Editor.model.design.on('fontchange', this.onFontChange, this);
		Sb.Editor.model.design.on('fontcolorchange', this.onFontColorChange, this);
	},

	/**
	 * Change current site page to another specified
	 * @param {Sb.model.SitePage} page
	 * @param {Function} [cb] A callback function to call after the current page is displayed.
	 * @param {Object} [scope] The scope (<code>this</code> reference) in which the callback is executed. Defaults to a new current page.
	 */
	setCurrentPage: function (page, cb, scope) {
		if (page.data.identity == Sb.Editor.model.rootPage.data.identity) {
			page = Sb.Editor.model.rootPage.getFirstVisiblePage();
		}
		if (page != this.currentPage && page.get('sitePageTypeId') != Sb.model.SitePage.LINK_TYPE) {
			this.compilePage(page);
		}
		Ext.callback(cb, scope || page);
	},

	/**
	 * Compiles whole page
	 * @param {Sb.model.SitePage} [page] Site page model node
	 */
	compilePage: function (page) {
		page = page || this.currentPage;

		if (Sb.Editor.Ui.editComponent) {
			Sb.Editor.Ui.editComponent.completeEdit();
		}

		this.destroyPage();

		this.currentPage = page;
		this.pageWidget = Ext.create({
			xtype : Sb.Editor.model.rootPage.widget.data.type + '_widget',
			model : Sb.Editor.model.rootPage.widget,
			renderTo: this.el
		});
		Sb.EcwidWidgetsManager.refresh();

		Sb.WidgetTypeManager.toggleButtons();
	},

	/**
	 * Destroy page if it compiled
	 */
	destroyPage: function () {
		if (this.pageWidget) {
			Sb.EcwidWidgetsManager.clear();
			this.currentPage = null;
			this.pageWidget.destroy();
		}
	},

	/**
	 * Update color scheme css
	 */
	onColorSchemeChange: function () {
		var design				= Sb.Editor.model.design,
			colorScheme			= design.data.colorScheme,
			css					= '',
			i, j, color;

		css += Sb.WidgetTypeManager.get('search').getColorSchemeCssTemplate().apply({
			globalItemBorderColor	: colorScheme.color3,
			resultsUrlColor			: colorScheme.color3
		});

		for (i = 1; i <= 4; i++) {
			for (j = 0; j <= 4; j++) {
				color = 'color' + i + (j > 0 ? '-' + j : '');
				css += '.palette-' + color + ' {color: ' + design.getColor(color) + ';}';
				css += '.palette-background-' + color + ' {background-color: ' + design.getColor(color) + ';}';
			}
		}
		Sb.StyleManager.addStyleSheet(css, 'colorscheme');

		this.onFontColorChange();
	},

	/**
	 * Update css for fonts
	 */
	onFontChange: function () {
		var fontData = Ext.apply({}, Sb.Editor.model.design.getSettings().font);
		if (!Sb.SiteView.fontTemplate) {
			Sb.SiteView.fontTemplate = new Ext.XTemplate(
				'.pageContentText{',
					'font-family:{textFont};',
					'font-size:{textFontSize}px;',
					'font-weight:{[values.textFontBold == "1" ? "bold" : "normal"]};',
					'font-style:{[values.textFontItalic == "1" ? "italic" : "normal"]};',
				'}',
				'.pageContentText p {',
					'text-decoration:{[values.textFontUnderline == "1" ? "underline" : "none"]};',
				'}',
				'.pageContentText h1{',
					'font-family:{h1Font};',
					'font-size:{h1FontSize}px;',
					'font-weight:{[values.h1FontBold == "1" ? "bold" : "normal"]};',
					'font-style:{[values.h1FontItalic == "1" ? "italic" : "normal"]};',
					'text-decoration:{[values.h1FontUnderline == "1" ? "underline" : "none"]};',
				'}',
				'.pageContentText h2 {',
					'font-family:{h2Font};',
					'font-size:{h2FontSize}px;',
					'font-weight:{[values.h2FontBold == "1" ? "bold" : "normal"]};',
					'font-style:{[values.h2FontItalic == "1" ? "italic" : "normal"]};',
					'text-decoration:{[values.h2FontUnderline == "1" ? "underline" : "none"]};',
				'}'
			);
		}

		var css = '';
		css += Sb.SiteView.fontTemplate.apply(fontData);

		var widgetType = Sb.WidgetTypeManager.get('eshopCatalog');
		css += widgetType.getFontSchemeCssTemplate().apply(widgetType.getCssData());

		Sb.StyleManager.addStyleSheet(css, 'font-set');
		Sb.FontManager.loadFont(fontData.h1Font);
		Sb.FontManager.loadFont(fontData.h2Font);
		Sb.FontManager.loadFont(fontData.textFont);
	},

	/**
	 * Update css for fonts colors
	 */
	onFontColorChange: function () {
		if (!Sb.SiteView.fontColorCssTemplate) {
			Sb.SiteView.fontColorCssTemplate = new Ext.XTemplate(
				'<tpl if="textColor != \'inherit\'">{className} { color: {textColor}; }</tpl>',
				'<tpl if="linkColor != \'inherit\'">{className} a { color: {linkColor}; }',
				'{className} a.link { color: {linkColor} !important; }</tpl>',
				'<tpl if="h1Color != \'inherit\'">{className} h1, {className} h1 a { color: {h1Color}; }</tpl>',
				'<tpl if="h2Color != \'inherit\'">{className} h2, {className} h2 a { color: {h2Color}; }</tpl>'
			);
		}

		var design			= Sb.Editor.model.design,
			fontColor		= design.data.fontColor,
			regions			= ['content', 'header', 'footer', 'column1', 'column2'],
			css				= '';

		Ext.each(regions, function (region) {
			css += Sb.SiteView.fontColorCssTemplate.apply({
				className: '.' + region + '-color',
				textColor: design.getColor(fontColor[region + 'TextColor']),
				linkColor: design.getColor(fontColor[region + 'LinkColor']),
				h1Color: design.getColor(fontColor[region + 'H1Color']),
				h2Color: design.getColor(fontColor[region + 'H2Color'])
			});
		});
		css += Sb.WidgetTypeManager.get('search').getFontColorCssTemplate().apply({
			resultsHeaderLinkColor: design.getColor(fontColor.contentLinkColor)
		});

		var widgetType = Sb.WidgetTypeManager.get('eshopCatalog');
		css += widgetType.getColorSchemeCssTemplate().apply(widgetType.getCssData());

		Sb.StyleManager.addStyleSheet(css, 'text-color');
	},

	getCurrentSiteLocale: function() {
		var locale = Ext.util.Cookies.get('currentSiteLocale');
		if (locale && Sb.Editor.model.get('locales').indexOf(locale) != -1) {
			return locale;
		}
		return Sb.Editor.model.get('locale');
	},

	applyContainerBackground: function (el, region, skipElastic) {
		var design = Sb.Editor.model.design,
			background = design.getBackground(region),
			cfg = {
				'background-color': 'transparent',
				'background-size': '',
				'background-repeat': '',
				'background-image': ''
			},
			stretchKeyToCss = {
				'tile': {'background-repeat': 'repeat'},
				'crop': {'background-size': 'cover'},
				'resize': {'background-size': 'contain'},
				'actual size': {}
			},
			isCustom = false,
			backgroundImagePath,
			format,
			headersStore,
			header,
			gradientParams,
			backgroundRgb,
			gradientRgb,
			gradientStart,
			gradientEnd,
			backgroundColor = design.getColor(background.color),
			backgroundOpacity = background.opacity;

		if (['header', 'content', 'footer'].indexOf(region) > -1 &&
			((design.getSettings().layout[region + 'Layout'] === 'elastic-background') === skipElastic)
		) {
			return;
		}

		if (background.type === 'gradient') {
			backgroundOpacity /= 100;

			gradientParams = Sb.Editor.model.data.template.decorations.gradients[background.gradient];
			gradientRgb = Ext.ux.util.Color.hex2rgba(gradientParams.startColor, gradientParams.transparency);
			if (backgroundColor != 'transparent') {
				backgroundRgb = Ext.ux.util.Color.hex2rgba(backgroundColor);
				gradientStart = Ext.ux.util.Color.apply(backgroundRgb, gradientRgb);
				gradientStart[3] = backgroundOpacity;
				gradientEnd = backgroundRgb;
				gradientEnd[3] = backgroundOpacity;
			} else {
				gradientStart = gradientRgb;
				gradientStart[3] *= backgroundOpacity;
				gradientEnd = [0,0,0,0];
			}
			gradientStart = gradientStart.join(',');
			gradientEnd = gradientEnd.join(',');

			cfg['background-image'] = 'linear-gradient(' +
				'to ' + gradientParams.direction + ',rgba(' + gradientStart + '),rgba(' + gradientEnd + ')' +
			')';
			cfg['background-size'] = '100% 100%';
		}

		if (['pattern', 'image'].indexOf(background.type) !== -1) {
			if (background.type === 'pattern') {
				isCustom = background.pattern.indexOf('.') !== -1;
				backgroundImagePath = isCustom
					? design.getBackgroundUrl(region) + '/' + background.pattern
					: Sb.Config.templatesUrl + '/' + Sb.Editor.model.data.template.code +
						'/decorations/background/' + background.pattern + '/images/' + background.pattern + '.png';
			} else {
				if (background.image.indexOf('.') === -1) {
					headersStore = Ext.StoreMgr.get('headers');
					format = headersStore.getAt(headersStore.find('name', background.image)).get('format');
					backgroundImagePath = Sb.Config.headersUrl + '/' + background.image + '/header.' + format;
				} else {
					backgroundImagePath = design.getResourcesUrl() + '/' + Sb.Config.path.attachmentsFolder +
						'/' + background.image;
				}
				backgroundColor = 'transparent';
			}

			cfg = Ext.apply({
				'background-color': region == 'site' ? design.getColor(background.color) : 'transparent',
				'background-size': 'auto',
				'background-repeat': 'no-repeat',
				'background-position': background.position
			}, stretchKeyToCss[background.stretch]);

			this.applyBackgroundImage(
				el,
				Sb.Utils.antiCache(backgroundImagePath, true),
				backgroundOpacity,
				backgroundColor
			);
		}

		el.setStyle(cfg);
	},

	applyBackgroundImage: function (el, imageUrl, opacity, color) {
		window.clearTimeout(this.deferIds[el.id]);
		if (opacity == 100 && color == 'transparent') {
			el.setStyle('backgroundImage', 'url("' + imageUrl + '")');
			return;
		}

		this.deferIds[el.id] = Ext.defer(function () {
			el = Ext.get(el);
			if (!el.dom) {
				return;
			}

			Sb.Utils.setBackgroundImage(
				el,
				imageUrl,
				opacity / 100,
				color
			);
		}, 100, this);
	}
});