/**
 * @class Sb.editor.DocumentsPanel
 * @extends Ext.form.FormPanel
 * Site files panel
 * @constructor
 * @param {Object} config The config object
 * @xtype documentspanel
 */
Ext.define('Sb.editor.DocumentsPanel', {
	extend: 'Sb.editor.BaseFormPanel',
	xtype: 'documentspanel',

	hasOkCancelButtons: false,

	// private
	initComponent: function () {
		this.documentManager = new Sb.editor.DocumentManager({
			model: Sb.Editor.documents,
			listeners: {
				scope: this,
				selectionchange: function (dataView) {
					var records = dataView.getSelectedRecords();
					if (records.length == 1) {
						this.documentProperties.show();
						this.documentNoProperties.hide();

						var record = records[0];
						this.documentName.setValue(record.get('name'));
						this.documentLink.setValue(record.get('file'));
						if (Sb.Config.isTrialMode) {
							this.documentLink.hide();
						}
						this.documentType.setValue(Sb.Utils.getDocumentAppByExtension(record.get('extension')));
						this.documentSize.setValue(record.get('size'));
						this.documentDate.setValue(record.get('date'));
					} else {
						this.documentProperties.hide();
						this.documentNoProperties.show();
					}
					if (records.length) {
						this.removeButton.enable();
					} else {
						this.removeButton.disable();
					}
				}
			}
		});
		this.documentManager.model.on({
			scope: this,
			'load': function (result) {
				if (result.success) {
					this.documentManager.selectRange(0, 0); // First document must be selected always to fill document properties column
				}
				Sb.Loading.hide();
			},
			'remove': function (result) {
				Ext.Msg.hide();
				if (result.files.length) {
					Sb.Editor.Ui.mainWindow.statusBar.clear();
					if (result.failedDocuments.length) {
						var errorMessage = '',
							separator = (errorMessage === '') ? '' : ',';

						Ext.each(result.failedDocuments, function (fileInfo) {
							errorMessage += separator + ' ' + fileInfo.name;
						});

						Sb.Editor.Ui.mainWindow.statusBar.addMessageError(
							Sb.Locale.get('documentDeleteFailed', errorMessage)
						);
					}
				}
			}
		});

		var uploadButton = new Ext.ux.form.FileUploadField({
			xtype		: 'fileuploadfield',
			buttonOnly	: true,
			multiple	: true,
			name		: 'document',
			msgTarget	: 'under',
			buttonCfg	: {
				text	: Sb.Locale.get('upload')
			},
			listeners: {
				scope: this,
				render: function (cmp) {
					cmp.button.setIconClass('sb-upload-icon');
				},
				click: function (e) { e.stopPropagation(); },
				fileselected: this.upload
			}
		});

		Ext.apply(this, {
			cls: 'sb-editor-documents-panel',
			title: Sb.Locale.get('siteDocumentsTitle'),
			helperMessage: Sb.Locale.get('siteDocumentsDescription'),
			tabTip: Sb.Locale.get('siteDocumentsDescriptionOnHover'),
			fileUpload: true,
			labelAlign: 'top',
			layout: 'column',
			autoHeight: true,
			defaults: {
				border: false,
				buttonAlign: 'left'
			},
			items: [{
				columnWidth: 0.6,
				items: [this.documentManager, {
					layout		: 'form',
					baseCls		: 'sb-editor-documents-panel-buttons',
					border		: false,
					hideLabels	: true,
					buttonAlign: 'left',
					buttons: [uploadButton, {
						ref		: '../../../removeButton',
						xtype	: 'button',
						text	: Sb.Locale.get('Remove'),
						iconCls	: 'sb-delete-icon',
						scope	: this,
						disabled: true,
						handler: this.onRemoveDocuments
					}]
				}]
			}, {
				columnWidth: 0.4,
				cls: 'sb-editor-documents-panel-properties-area',
				defaults: {
					border: false
				},
				items: [{
					layout: 'form',
					ref: '../documentProperties',
					hidden: true,
					cls: 'sb-editor-documents-panel-properties',
					items: [{
						ref: '../../documentName',
						xtype: 'displayfield',
						fieldLabel: Sb.Locale.get('siteDocumentsDocumentName')
					}, {
						ref: '../../documentLink',
						xtype: 'displayfield',
						fieldLabel: Sb.Locale.get('siteDocumentsDocumentLink')
					}, {
						ref: '../../documentType',
						xtype: 'displayfield',
						fieldLabel: Sb.Locale.get('siteDocumentsDocumentType')
					}, {
						ref: '../../documentSize',
						xtype: 'displayfield',
						fieldLabel: Sb.Locale.get('siteDocumentsDocumentSize')
					}, {
						ref: '../../documentDate',
						xtype: 'displayfield',
						fieldLabel: Sb.Locale.get('siteDocumentsDocumentDate')
					}]
				}, {
					cls: 'sb-editor-documents-panel-noproperties',
					ref: '../documentNoProperties',
					html: Sb.Locale.get('siteDocumentsNoProperties')
				}]
			}]
		});
		Sb.editor.DocumentsPanel.superclass.initComponent.apply(this, arguments);
	},

	onActivate: function () {
		Sb.editor.DocumentsPanel.superclass.onActivate.apply(this, arguments);

		if (this.documentManager.model.isLoaded) {
			this.documentManager.refresh();
		} else {
			Sb.Loading.updateMessage(Sb.Locale.get('getDocumentsList'));
			Sb.Loading.show();
			this.documentManager.loadDocuments();
		}
	},

	onRemoveDocuments: function () {
		Ext.Msg.show({
			title	: Sb.Locale.get('documentDeleteTitle'),
			msg		: Sb.Locale.get('documentDeleteConfirm'),
			closable: false,
			buttons	: Ext.Msg.YESNO,
			scope	: this,
			fn		: function (btn) {
				if (btn == 'yes') {
					Ext.Msg.wait(Sb.Locale.get('removingDocument'), Sb.Locale.get('pleaseWait'));
					this.documentManager.removeDocuments();
				}
			}
		});
	},

	upload: function (field, fieldValue) {
		if ('' === fieldValue) {
			return;
		}
		var form = this.getForm(),
			url = 'Upload/Document',
			params = {'X-CSRF-Token': Sb.Config.csrfToken},
			waitMsg = Sb.Locale.get('uploadingDocument');

		if (Ext.isIE) {
			form.submit({
				url: url,
				params: params,
				clientValidation: false,
				waitMsg: waitMsg,
				success: function (fp, action) {
					this.afterUploadFiles(action.result);
					field.reset();
				}.createDelegate(this),
				failure: function (fp, action) {
					Sb.Editor.Ui.mainWindow.statusBar.addMessageError(Sb.Locale.get('uploadingError'));
					field.reset();
				}
			});
		} else {
			var result = {
					errors: [],
					files: []
				},
				uploader = new Ext.ux.uploader.Manager({
					url: url,
					params: params,
					waitMsg: waitMsg,
					items: field.getFiles(),
					listeners: {
						scope: this,
						itemuploadfailure: function (item, info) {
							if (info.result.errors) {
								result.errors = result.errors.concat(info.result.errors);
							} else {
								result.errors.push(Sb.Locale.get('uploadingError'));
							}
						},
						itemuploadsuccess: function (item, info) {
							this.afterUploadFiles(info.result);
						},
						completeupload: function () {
							this.afterUploadFiles(result);
							field.reset();
						}
					}
				});
			uploader.startUpload();
		}
	},

	afterUploadFiles: function (result) {
		Ext.each(result.errors, function (item) {
			Sb.Editor.Ui.mainWindow.statusBar.addMessageError(item);
		});

		this.documentManager.addDocuments(result.files);
	}
});