/**
 * @class Sb.editor.SnapshotMenu
 * @extends Ext.menu.Menu
 * Class for snapshot menu
 */
Ext.define('Sb.editor.SnapshotMenu', {
	extend: 'Ext.menu.Menu',

	/**
	 * Collapse popup menu when clicked outside
	 */
	isCollapseMenu: true,

	/**
	 * Type of menu: save or load
	 */
	type: 'save',

	initComponent: function () {
		this.addEvents(
			/**
			 * @event saveslot
			 * Fires when the slot save
			 * @param {string} id Id of snapshot
			 */
			'saveslot',

			/**
			 * @event loadslot
			 * Fires when the slot load
			 * @param {string} id Id of snapshot
			 */
			'loadslot',

			/**
			 * @event uploadslot
			 * Fires when the slot upload
			 * @param {Ext.ux.form.FileUploadField} field Field with uploaded snapshot
			 * @param {string} id Id of snapshot
			 */
			'uploadslot',

			/**
			 * @event downloadslot
			 * Fires when the slot download
			 * @param {string} id Id of snapshot
			 */
			'downloadslot',

			/**
			 * @event deleteslot
			 * Fires when the slot delete
			 * @param {string} id Id of snapshot
			 */
			'deleteslot'
		);

		var columnActions = [{
			getClass: function (v, meta, rec, index) {
				return this.isEmptySlot(index) ? 'disabled' : 'sb-slot-delete';
			},
			handler: this.onDeleteSlot,
			tooltip: Sb.Locale.get('deleteSnapshotTooltip'),
			scope: this
		}];
		if (Sb.Config.uploadSnapshots) {
			columnActions.unshift({
				id: 'upload',
				getClass: function (v, meta, rec, index) {
					return 'sb-slot-upload';
				},
				scope: this
			});
			columnActions.push({
				getClass: function (v, meta, rec, index) {
					return this.isEmptySlot(index) ? 'disabled' : 'sb-slot-download';
				},
				handler: this.onDownloadSlot,
				tooltip: Sb.Locale.get('downloadSnapshotTooltip'),
				scope: this
			});
		}
		this.grid = new Ext.grid.EditorGridPanel({
			store			: this.store,
			columns			: [{
				id: 'title',
				header: Sb.Locale.get('snapshotTitle'),
				dataIndex: 'title',
				renderer: Ext.util.Format.htmlEncode,
				editable: this.isTitleEditable,
				editor: new Ext.form.TextField({
					allowBlank: false,
					enableKeyEvents: true,
					fieldClass: 'sb-grid-edit-cell',
					listeners: {
						scope: this,
						keydown: function (field, e) {
							if (this.isApplyOnEnter && e.getKey() == e.ENTER) {
								this.onApply();
							}
						}
					}
				})
			}, {
				id: 'date',
				header: Sb.Locale.get('snapshotDate'),
				dataIndex: 'date',
				renderer: Ext.util.Format.dateRenderer('G:i, j M Y'),
				width: 115
			}, {
				id: 'actions',
				xtype: 'actioncolumn',
				width: 66,
				items: columnActions,
				renderer: function (v, meta) {
					return Sb.Config.uploadSnapshots ? '<form class="sb-form-snapshot-upload">' +
						'<input class="sb-btn-snapshot-upload" name="snapshot" type="file" ' +
						'ext:qtip="' + Sb.Locale.get('uploadSnapshotTooltip') + '" />' +
					'</form>' : '';
				}
			}],
			enableHdMenu		: false,
			enableColumnResize	: false,
			autoHeight			: true,
			border				: false,
			viewConfig			: {
				scrollOffset	: 0,
				markDirty		: false,
				listeners: {
					scope: this,
					refresh: this.onRefresh,
					rowupdated: this.onRowUpdated
				}
			},
			selModel		: new Ext.grid.RowSelectionModel({
				singleSelect		: true,
				moveEditorOnEnter	: false,
				listeners: {
					scope: this,
					rowdeselect: function (selectionModel, index, record) {
						selectionModel.grid.stopEditing();
						record.reject();
					}
				}
			}),
			enableColumnMove: false,
			autoExpandColumn: 'title',
			clicksToEdit	: 1,
			listeners: {
				scope		: this,
				beforeedit	: function (e) {
					if (this.isEmptySlotTitle(e.value)) {
						e.record.set('title', Sb.Locale.get('userSnapshotName'));
					}
				},
				keydown		: function (e) {
					if (this.isApplyOnEnter && e.getKey() == e.ENTER) {
						this.onApply();
					}
				},
				celldblclick: function (grid, rowIndex, columnIndex) {
					if (this.type == 'load') {
						var columnId = grid.getColumnModel().getColumnId(columnIndex);
						if (columnId != 'actions') {
							this.onApply();
						}
					}
				}
			}
		});

		Ext.apply(this, {
			cls: 'sb-slot-menu',
			width: 360,
			zIndex: 3000,
			items: {
				xtype: 'panel',
				border: false,
				height: this.height,
				width: this.width,
				items: this.grid,
				buttons: [{
					text: Sb.Locale.get(this.type),
					handler: this.onApply,
					scope: this
				}, {
					text: Sb.Locale.get('Close'),
					handler: this.onCancel,
					scope: this
				}]
			},
			listeners: {
				scope: this,
				beforehide: function (cmp) {
					return this.isCollapseMenu;
				},
				hide: function () {
					this.grid.stopEditing();
					this.store.rejectChanges();
					this.store.clearFilter();
				}
			}
		});

		Sb.editor.SnapshotMenu.superclass.initComponent.call(this);
	},

	onApply: function () {
		var selectionModel = this.grid.getSelectionModel();
		if (selectionModel.getCount() != 1) {
			return;
		}
		var	record = selectionModel.getSelected(),
			index = this.store.indexOf(record);

		record.commit();
		this.type == 'save' ? this.onSaveSlot(index) : this.onLoadSlot(index);
	},

	onSaveSlot: function (index) {
		var encode	= Ext.util.Format.htmlEncode;
		if (!this.isEmptySlot(index)) {
			Ext.Msg.show({
				width		: 400,
				closable	: false,
				title		: encode(Sb.Locale.get('overwriteSnapshotTitle')),
				msg			: encode(Sb.Locale.get('overwriteSnapshotMessage')),
				buttons		: Ext.Msg.YESNO,
				scope		: this,
				fn			: function (btn) {
					if ('yes' == btn) {
						this.fireEvent('saveslot', this.getSlotByIndex(index));
					}
				}
			});
		} else {
			this.fireEvent('saveslot', this.getSlotByIndex(index));
		}
	},

	onLoadSlot: function (index) {
		Sb.editor.UiContext.tryToSwitch(function (){
			if (this.isAllSlotsEmpty()) {
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get('loadSnapshotAllEmptyError'));
				return;
			}
			if (this.isEmptySlot(index)) {
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get('loadSnapshotEmptyError'));
				return;
			}

			Ext.Msg.show({
				width: 400,
				closable : false,
				title: Ext.util.Format.htmlEncode(Sb.Locale.get('loadSnapshotTitle')),
				msg: Ext.util.Format.htmlEncode(Sb.Locale.get('loadSnapshotMessage')),
				buttons: Ext.Msg.YESNO,
				scope: this,
				fn: function (btn) {
					if (btn == 'yes') {
						this.fireEvent('loadslot', this.getSlotByIndex(index));
					}
					this.isCollapseMenu = true;
				}
			});
			this.isCollapseMenu = false;
		}.createDelegate(this));
	},

	onRefresh: function () {
		Ext.select('input.sb-btn-snapshot-upload').each(function (input, elLite, index) {
			this.addHandlerForUploadSnapshot(input, index);
		}, this);
	},

	onRowUpdated: function (gridView, index) {
		var row = new Ext.Element(gridView.getRow(index)),
			input = row.child('input.sb-btn-snapshot-upload');
		this.addHandlerForUploadSnapshot(input, index);
	},

	addHandlerForUploadSnapshot: function (el, slotIndex) {
		if (!Sb.Config.uploadSnapshots) {
			return;
		}
		el.on('change', function (e, btn) {
			btn = new Ext.Element(btn);

			// validate
			var error = false,
				files = btn.dom.files,
				regex = new RegExp('\\.ssb$', 'i');

			// for IE
			if (!files) {
				files = [{name: btn.getValue().replace(/^.*\\/, '')}];
			}

			Ext.each(files, function (file) {
				if (file.name = '' || !regex.test(file.name)) {
					error = true;
					Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
						Sb.Locale.get('snapshotUploadErrorTypeInvalid')
					);
					return false;
				}
			});

			var form = new Ext.form.BasicForm(btn.up('form'), {
				fileUpload: true
			});

			if (error) {
				form.reset();
				return;
			}

			var buttonElement = Ext.get(this.type + 'SnapshotButton');
			this.fireEvent('uploadslot', form, this.getSlotByIndex(slotIndex),
				function () {
					Sb.Editor.Ui.mainWindow.statusBar.setMessage(Sb.Locale.get('siteSnapshotUploaded'));
					this.showAt(buttonElement.getAlignToXY(buttonElement));
				}.createDelegate(this),
				function (message) {
					Sb.Editor.Ui.mainWindow.statusBar.setMessageError(message);
					this.showAt(buttonElement.getAlignToXY(buttonElement));
				}.createDelegate(this)
			);
		}, this);
	},

	onDownloadSlot: function (grid, rowIndex) {
		this.fireEvent('downloadslot', this.getSlotByIndex(rowIndex));
	},

	onDeleteSlot: function (grid, rowIndex) {
		this.isCollapseMenu = false;
		Ext.Msg.show({
			width		: 400,
			closable	: false,
			title		: Sb.Locale.get('deleteSnapshotTitle'),
			msg			: Sb.Locale.get('deleteSnapshotMessage'),
			buttons		: Ext.Msg.YESNO,
			scope		: this,
			fn			: function (btn) {
				if (btn == 'yes') {
					this.fireEvent('deleteslot', this.getSlotByIndex(rowIndex));
				}
				this.isCollapseMenu = true;
			}
		});
	},

	onCancel: function () {
		this.hide();
	},

	isEmptySlot: function (index) {
		return this.store.getAt(index).get('date') == null;
	},

	isEmptySlotTitle: function (title) {
		return title == Sb.Locale.get('emptySlot');
	},

	isAllSlotsEmpty: function () {
		return -1 == this.store.findBy(function (r) {
			return r.get('date') != null;
		});
	},

	getSlotByIndex: function (index) {
		return this.store.getAt(index).get('type') == 'quick' ? 'quick' : index;
	}
});