/**
 * @class Sb.editor.StylePanel
 * @extends Sb.editor.BaseFormPanel
 * Style editor panel
 * @constructor
 * @param {Object} config The config object
 * @xtype stylepanel
 */
Ext.define('Sb.editor.StylePanel', {
	extend: 'Sb.editor.BaseFormPanel',
	xtype: 'stylepanel',

	// private
	initComponent: function () {
		var schemes, schemesStore, schemeTab, fontSizeComboCfg, fontComboCfg, fontsTab;

		schemes = [];
		Ext.iterate(Sb.Editor.model.data.template.colorschemes, function (key, value) {
			schemes.push({
				id: key,
				title: Sb.Locale.get('Styleset' + key),
				colors: value
			});
		});
		schemesStore = new Ext.data.JsonStore({
			fields: ['id', 'title', 'colors'],
			data: schemes
		});

		schemeTab = {
			itemId: 'scheme',
			title: Sb.Locale.get('styleSchemeTab'),
			helperMessage: Sb.Locale.get('styleSchemeTabDescription'),
			cls: 'sb-editor-style-scheme-panel',
			layout: 'form',
			defaults: {border: false},
			items: [{
				layout: 'form',
				items: [{
					xtype: 'combo',
					id: 'colorschemes',
					hideLabel: true,
					triggerAction: 'all',
					mode: 'local',
					store: schemesStore,
					anchor: '100%',
					editable: false,
					displayField: 'title',
					valueField: 'id',
					tpl: new Ext.XTemplate(
						'<tpl for="."><div class="x-combo-list-item sb-styleset-list-item">' +
							'<div style="float: left;">{title}</div>' +
							'<ul>' +
								'<li class="first" style="background-color: {[values.colors.color1]};"></li>' +
								'<li style="background-color: {[values.colors.color2]};"></li>' +
								'<li style="background-color: {[values.colors.color3]};"></li>' +
								'<li class="last" style="background-color: {[values.colors.color4]};"></li>' +
							'</ul>' +
						'</div></tpl>'
					),
					listeners: {
						scope: this,
						select: function (combo) {
							var record = combo.findRecord(combo.valueField, combo.getValue()),
								scheme = record.get('colors');

							this.form.findField('baseColor1').setRawValue(scheme.color1);
							this.form.findField('baseColor2').setRawValue(scheme.color2);
							this.form.findField('baseColor3').setRawValue(scheme.color3);
							this.form.findField('baseColor4').setRawValue(scheme.color4);
							this.selectedColorField.updateCurrentPicker();

							this.onColorSchemeChange();
						}
					}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-style-description',
					html		: Sb.Locale.get('styleSchemeTabLongDescription')
				}]
			}, {
				layout: 'column',
				items: [{ // Column 1
					width: 195,
					cls: 'sb-editor-panel-column-margin',
					id: 'schemeColorPicker',
					xtype: 'colorpicker',
					listeners: {
						scope: this,
						change: function (colorpicker, value) {
							this.selectedColorField.setRawValue(value, true);
							this.lastChangeTime = this.lastChangeTime || (new Date()).getTime();
							var delay = 0, taskDelay = 0;
							if (!Ext.isIE) {
								delay = 100;
								taskDelay = 20;
							} else {
								delay = 250;
							}
							if (delay > 0) {
								var now = (new Date()).getTime();
								var difference = now - this.lastChangeTime;
								if (difference <= delay) {
									return;
								}
								this.lastChangeTime = now;
							}
							if (taskDelay) {
								var task = new Ext.util.DelayedTask(this.onColorFieldChange, this, [this.selectedColorField]);
								task.delay(taskDelay);
							} else {
								this.onColorFieldChange(this.selectedColorField);
							}
						},
						mouseup: function (colorpicker, value) {
							this.onColorFieldChange(this.selectedColorField);
						}
					}
				}, { // Column 2
					columnWidth: 1,
					layout: 'form',
					border: false,
					labelAlign: 'right',
					style: {paddingTop: '25px'},
					labelWidth: 120,
					defaults: {
						xtype			: 'colorfield',
						currentPicker	: 'schemeColorPicker',
						leaveSelection: true,
						listeners: {
							scope		: this,
							focus		: this.selectColorField,
							valid		: this.onColorFieldChange,
							invalid		: Sb.editor.UiContext.registerChanges
						}
					},
					items: [{
						name: 'baseColor1',
						fieldLabel: Sb.Locale.get('color') + ' 1'
					}, {
						name: 'baseColor2',
						fieldLabel: Sb.Locale.get('color') + ' 2'
					}, {
						name: 'baseColor3',
						fieldLabel: Sb.Locale.get('color') + ' 3'
					}, {
						name: 'baseColor4',
						fieldLabel: Sb.Locale.get('color') + ' 4'
					}]
				}]
			}],
			listeners: {
				scope		: this,
				activate	: {
					fn		: this.updateSchemeTab
				}
			}
		};

		fontSizeComboCfg = {
			xtype: 'sbfontsizecombo',
			anchor: '90%',
			listeners: {
				scope: this,
				select: this.onFontFieldChange,
				change: this.onFontFieldChange,
				specialkey: function (combo, e) {
					if (e.getKey() === e.ENTER) {
						this.onFontFieldChange(combo);
					}
				}
			}
		};

		fontComboCfg = {
			xtype: 'sbfontfamilycombo',
			anchor: '95%',
			listeners: {
				select: this.onFontFieldChange.createDelegate(this)
			}
		};

		fontsTab = {
			itemId: 'fonts',
			title: Sb.Locale.get('styleFontsTab'),
			cls: 'sb-editor-style-fonts-panel',
			layout: 'column',
			defaults: {border: false, layout: 'form'},
			helperMessage: Sb.Locale.get('styleFontsTabDescription'),
			items: [{ // Column 1
				columnWidth: 0.6,
				cls: 'sb-editor-panel-column-margin',
				defaults: {border: false},
				items: [{
					xtype: 'label',
					text: Sb.Locale.get('h1ColorLabel'),
					cls: 'sb-editor-panel-heading'
				}, {
					layout: 'column',
					defaults: {
						border: false,
						layout: 'form',
						labelAlign: 'top'
					},
					items: [{
						columnWidth: 0.5,
						items: Ext.apply({name: 'h1Font'}, fontComboCfg)
					}, {
						columnWidth: 0.2,
						items: Ext.apply({name: 'h1FontSize'}, fontSizeComboCfg)
					}, {
						columnWidth: 0.3,
						items: this.generateFontTabDecorationsField('h1')
					}]
				}, {
					xtype: 'label',
					text: Sb.Locale.get('h2ColorLabel'),
					cls: 'sb-editor-panel-heading'
				}, {
					layout: 'column',
					defaults: {
						border: false,
						layout: 'form',
						labelAlign: 'top'
					},
					items: [{
						columnWidth: 0.5,
						items: Ext.apply({name: 'h2Font'}, fontComboCfg)
					}, {
						columnWidth: 0.2,
						items: Ext.apply({name: 'h2FontSize'}, fontSizeComboCfg)
					}, {
						columnWidth: 0.3,
						items: this.generateFontTabDecorationsField('h2')
					}]
				}, {
					xtype: 'label',
					text: Sb.Locale.get('textColorLabel'),
					cls: 'sb-editor-panel-heading'
				}, {
					layout: 'column',
					defaults: {
						border: false,
						layout: 'form',
						labelAlign: 'top'
					},
					items: [{
						columnWidth: 0.5,
						items: Ext.apply({name: 'textFont'}, fontComboCfg)
					}, {
						columnWidth: 0.2,
						items: Ext.apply({name: 'textFontSize'}, fontSizeComboCfg)
					}]
				}]
			}, { // Column 2
				columnWidth: 0.4,
				labelAlign: 'top',
				items: [{
					xtype: 'displayfield',
					name: 'fontPreview',
					cls: 'x-form-text',
					style: {
						'overflow': 'hidden'
					},
					anchor: '0',
					height: 174,
					value:	'<div class="pageContentText">' +
								'<h1>' + Sb.Locale.get('previewHeading') + ' 1</h1>' +
								'<h2>' + Sb.Locale.get('previewHeading') + ' 2</h2>' +
								'<p>' + Sb.Locale.get('previewText') + '</p>' +
							'</div>',
					fieldLabel: Sb.Locale.get('preview')
				}]
			}],
			listeners: {
				scope		: this,
				activate	: {
					fn		: this.updateFontsTab
				}
			}
		};

		Ext.apply(this, {
			cls: 'sb-editor-style-panel',
			title: Sb.Locale.get('styleTitle'),
			helperMessage: Sb.Locale.get('styleDescription'),
			tabTip: Sb.Locale.get('styleDescriptionOnHover'),
			autoHeight: true,
			items: {
				xtype: 'sbtabpanel',
				tabWidth: 90,
				tabPosition: 'left',
				activeTab: 0,
				border: false,
				cls: 'sb-editor-style-panel-tabs',
				items: [{
					itemId: 'presets',
					xtype: 'designpresetspanel'
				}, {
					itemId: 'layout',
					xtype: 'layoutpanel'
				}, schemeTab, {
					itemId: 'styleColor',
					xtype: 'stylecolorpanel',
					stylePanel: this
				}, fontsTab, {
					itemId: 'styleCorner',
					xtype: 'cornerspanel'
				}, {
					itemId: 'styleBorder',
					xtype: 'styleborderpanel'
				}],
				defaults: {autoHeight: true},
				listeners: {
					scope: this,
					tabchange: this.onTabChange,
					beforetabchange: this.onBeforeTabChange
				}
			},
			buttonAlign: 'left',
			buttons: [{
				text: Sb.Locale.get('designPresetsExportButtonTitle'),
				tooltip: Sb.Editor.isDemoMode()
					? Sb.Locale.get('designPresetsExportButtonDisabled')
					: Sb.Locale.get('designPresetsExportButtonDescription'),
				disabled: Sb.Editor.isDemoMode(),
				handler: this.exportDesignHandler,
				scope: this
			}, {
				xtype: 'container',
				autoEl: {
					tag: 'form'
				},
				items: {
					xtype: 'fileuploadfield',
					buttonOnly: true,
					buttonCfg: {
						text: Sb.Locale.get('designPresetsImportButtonTitle')
					},
					regex: /\.zip$/i,
					listeners: {
						scope: this,
						fileselected: this.importDesignHandler,
						afterrender: function(field) {
							field.fileInput.on('mouseover', function() {
								Sb.Editor.Ui.mainWindow.setHelperMessage(
									Sb.Locale.get('designPresetsImportButtonDescription')
								);
							});
							field.fileInput.on('mouseout', function() {
								Sb.Editor.Ui.mainWindow.restoreHelperMessage();
							});
						}
					}
				}
			}, '->']
		});

		Sb.editor.StylePanel.superclass.initComponent.apply(this, arguments);
		this.bodyCfg.tag = 'div';

		/**
		 * Inner tab panel
		 * @type Ext.TabPanel
		 * @property tabs
		 */
		this.tabs = this.items.item(0);

		this.colorSchemesCombo = Ext.getCmp('colorschemes');
	},

	selectColorField: function (fieldToSelect) {
		var i;
		for (i = 1; i <= 4; i++) {
			var field = this.form.findField('baseColor' + i);
			if (field === fieldToSelect) {
				continue;
			}
			field.isSelected = false;
			field.updateWrapperClasses();
		}
		this.selectedColorField = fieldToSelect;
	},

	onColorFieldChange: function (field) {
		if (field.selectedStartValue === field.getValue()) {
			return;
		}
		this.colorSchemesCombo.setValue(Sb.Locale.get('Stylesetcustom'));
		this.onColorSchemeChange();
	},

	onColorSchemeChange: function () {
		var colors = {colorScheme: {}};
		for (var i = 1; i <= 4; i++) {
			var field = this.form.findField('baseColor' + i);
			if (!field.isValidatedOk) {
				continue;
			}
			colors.colorScheme['color' + i] = field.getValue();
		}
		if (!Sb.Editor.model.design.unserialize(colors)) {
			return;
		}
		Sb.editor.UiContext.registerChanges();
	},

	updateSchemeTab: function () {
		var designSettings = Sb.Editor.model.design.getSettings(),
			colorScheme = designSettings.colorScheme;

		this.form.findField('baseColor1').setRawValue(colorScheme.color1, true);
		this.form.findField('baseColor2').setRawValue(colorScheme.color2, true);
		this.form.findField('baseColor3').setRawValue(colorScheme.color3, true);
		this.form.findField('baseColor4').setRawValue(colorScheme.color4, true);

		// search match current color scheme to one in all colors schemes
		this.colorSchemesCombo.setValue(Sb.Locale.get('Stylesetcustom'));
		// iterate style sets
		this.colorSchemesCombo.store.each(function (record) {
			// set color set name if match
			if (colorScheme.color1 === record.data.colors.color1
				&& colorScheme.color2 === record.data.colors.color2
				&& colorScheme.color3 === record.data.colors.color3
				&& colorScheme.color4 === record.data.colors.color4
			) {
				this.colorSchemesCombo.setValue(record.data.id);
				return false;
			}
		}, this);
		if (!this.selectedColorField) {
			this.selectedColorField = this.form.findField('baseColor1');
		}
		this.selectedColorField.focus();
	},

	updateFontsTab: function () {
		var designSettings = Sb.Editor.model.design.getSettings();

		this.form.setValues({
			textFont: designSettings.font.textFont,
			textFontSize: designSettings.font.textFontSize,
			h1Font: designSettings.font.h1Font,
			h1FontSize: designSettings.font.h1FontSize,
			h2Font: designSettings.font.h2Font,
			h2FontSize: designSettings.font.h2FontSize
		});
	},

	/**
	 * Generates common config for font decorations
	 *
	 * @param {String} prefix
	 */
	generateFontTabDecorationsField: function (prefix) {
		var panel = new Sb.form.ColorDecorationPanel({
			itemId			: prefix + 'Decoration',
			cls				: 'sb-edit-control clearfix',
			widgetControlCls	: 'sb-edit-control-',
			children		: 'decoration',
			decorations		: ['Bold', 'Italic']
		});
		var decorationPanel	= panel.getComponent('Decoration');
		var model			= Sb.Editor.model.design;
		Ext.each(panel.decorations, function (decoration) {
			var button		= decorationPanel.getComponent(decoration);
			var modelKey	= prefix + 'Font' + decoration.toUpperCaseFirst();
			this.on('render', function () {
				this.tabs.getComponent('fonts').on('activate', function () {
					button.toggle(1 == model.data.font[modelKey], true);
				});
			}, this);
			button.on('toggle', function () {
				var newProperties				= {font: {}};
				newProperties.font[modelKey]	= this.pressed;
				model.unserialize(newProperties);
				Sb.editor.UiContext.registerChanges();
			});
		}, this);

		return panel;
	},

	/**
	 * Fires after fonts form field has changed.
	 * Sets new design settings values
	 *
	 * @param {Object} field Text field or combobox
	 */
	onFontFieldChange: function (field) {
		var newProperties = {font: {}};
		newProperties.font[field.getName()] = field.getValue();

		Sb.Editor.model.design.unserialize(newProperties);
		Sb.editor.UiContext.registerChanges();
	},

	/**
	 * Activate handler
	 */
	onActivate: function () {
		this.memento = Sb.Editor.model.design.saveToMemento();
		Ext.StoreMgr.get('backgrounds').on('remove', this.onRemoveBackground, this);
		this.tabs.activeTab.fireEvent('activate', this.tabs.activeTab);
		Sb.editor.StylePanel.superclass.onActivate.apply(this, arguments);
	},

	onRemoveBackground: function (store, record) {
		var name = record.get('name');
		Ext.iterate(this.memento.background, function (key, value) {
			if (value === name) {
				delete this.memento.background[key];
			}
		}, this);
	},

	/**
	 * Fires before the active tab has changed.
	 * @param {Ext.TabPanel} tabPanel
	 * @param {Ext.Panel} newTab
	 * @param {Ext.Panel} currentTab
	 */
	onBeforeTabChange: function (tabPanel, newTab, currentTab) {
		if (this.showActiveErrors()) {
			return false;
		}
	},

	/**
	 * Fires after the active tab has changed.
	 * @param {Ext.TabPanel} tabPanel
	 * @param {Ext.Panel} tab
	 */
	onTabChange: function (tabPanel, tab) {
		Sb.Editor.Ui.mainWindow.setHelperMessageHeight(0, true);

		this.helperMessage = tab.helperMessage;
		Sb.Editor.Ui.mainWindow.setHelperMessage(tab.helperMessage, true);
	},

	/**
	 * Allows us to safely switch to subtab of this tab.
	 *
	 * @param {String} subTabItemId The subtab`s itemId being activated
	 */
	setActiveTab: function (subTabItemId) {
		var subTab = this.tabs.getComponent(subTabItemId),
			parentTabPanel = this.ownerCt.ownerCt.tabs;
		if (this.tabs.activeTab && !subTab.rendered) {
			this.tabs.activeTab.hide();
			parentTabPanel.setActiveTab(this);
			this.tabs.setActiveTab(subTab);
		} else {
			this.tabs.setActiveTab(subTab);
			parentTabPanel.setActiveTab(this);
		}
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function (afterSaveCallbacks) {
		Ext.StoreMgr.get('backgrounds').un('remove', this.onRemoveBackground, this);
		if (this.showActiveErrors()) {
			afterSaveCallbacks.failure();
			return;
		}
		if (!Sb.Editor.model.design.saveIfChanged(this.memento, {
			success: function () {
				Ext.callback(afterSaveCallbacks.success);
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('messageChangesSaved')
				);
			},
			failure: function () {
				Ext.callback(afterSaveCallbacks.failure);
			}
		})) {
			Ext.callback(afterSaveCallbacks.success);
		}
	},

	/**
	 * Discard handler
	 */
	discardHandler: function () {
		Ext.StoreMgr.get('backgrounds').un('remove', this.onRemoveBackground, this);
		Sb.Editor.model.design.restoreFromMemento(this.memento);
		Sb.Editor.model.design.saveIfChanged(this.memento);
	},

	/**
	 * Handler for export design button
	 */
	exportDesignHandler: function () {
		if (!Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			Sb.editor.UiContext.tryToSwitch(this.exportDesignHandler.createDelegate(this));
			return;
		}

		Sb.model.apiProvider.runAfterLastCall(function() {
			Ext.DomHelper.append(document.body, {
				tag: 'iframe',
				frameBorder: 0,
				width: 0,
				height: 0,
				'class': 'sb-hidden',
				src: Sb.Config.baseUrl + '/download/designpreset?X-CSRF-Token=' + Sb.Config.csrfToken
			});
		});
	},

	/**
	 * Handler for import design button
	 */
	importDesignHandler: function (field, fieldValue) {
		if (!field.validate()) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get('designPresetsImportErrorTypeInvalid'));
			return;
		}

		if (!this.importDesignForm) {
			this.importDesignForm = new Ext.form.BasicForm(field.ownerCt.el, {
				fileUpload: true
			});
			this.importDesignForm.add(field);
		}

		this.importDesignForm.submit({
			url: Sb.Config.baseUrl + '/upload/designpreset',
			waitMsg: Sb.Locale.get('designPresetsImportWaitMessage'),
			clientValidation: false,
			success: function (fp, action) {
				field.reset();
				var preset = action.result.preset;
				preset.layout.reInitUuid = true;
				if (preset.banner) {
					preset.banner.reInitUuid = true;
				}
				preset.design.presetType = 'generic';
				preset.design.presetName = '';
				Sb.Editor.getController('DesignPresets').unserializePreset(preset);
				Ext.StoreMgr.get('backgrounds').loadCustomBackgrounds();
				Sb.Editor.Ui.mainWindow.statusBar.clear();
			},
			failure: function (fp, action) {
				field.reset();
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
					Sb.Locale.get(action.result.message || 'designPresetsImportError')
				);
			}
		});
	}
});