/**
 * @class Sb.Editor.Ui
 * <p>This singleton object, which is represent user interface of the editor.</p>
 * @singleton
 */
Ext.define('Sb.Editor.Ui', {
	singleton: true,

	/**
	 * Checks that any object is dragged at this time
	 */
	isDragActive: function () {
		return Ext.dd.DragDropMgr.dragCurrent != null;
	},

	/**
	 * Current edit component reference
	 * @type {Sb.editor.component}
	 * @property editComponent
	 */
	editComponent: null,

	/**
	 * History of components that are edited before current
	 * @type {Array}
	 * @property editComponentHistory
	 */
	editComponentHistory: [],

	/**
	 * Current over component reference
	 * @type {Sb.editor.component}
	 * @property editComponent
	 */
	overComponent: null,

	/**
	 * History of components that are overed before current
	 * @type {Array}
	 * @property overComponentHistory
	 */
	overComponentHistory: [],

	/**
	 * Main window reference
	 * @type {Sb.editor.MainWindow}
	 * @property mainWindow
	 */
	mainWindow: null,

	/**
	 * Widget style panel reference
	 * @type {Sb.editor.component.BaseStylePanel}
	 * @property stylePanel
	 */
	stylePanel: null,

	/**
	 * Visibility of widget style panel
	 * @type {Boolean}
	 * @property isShowStylePanel
	 */
	isShowStylePanel: false,

	init: function () {
		this.mainWindow = new Sb.editor.MainWindow({
			widgets: Sb.Config.widgets
		});
		this.mainWindow.show();

		this.mobileSimulator = new Sb.editor.MobileSimulator({x: 2,	y: 3});

		var statusBar	= Sb.Editor.Ui.mainWindow.statusBar;

		if (Ext.isIE) {
			var isResized = 96 != screen.deviceXDPI;
			if (isResized) {
				this.mainWindow.setPosition(0, 0);
				statusBar.addMessageWarning(Sb.Locale.get('ieResizedWarning', Sb.Config.general.product));
			}
		}

		if (window.console && window.console.firebug) {
			statusBar.addMessageWarning(Sb.Locale.get('firebugWarning', Sb.Config.general.product));
		}

		if (Ext.isIE7) {
			statusBar.addMessageWarning(Sb.Locale.get('updateYouIE7toNewVersion', Sb.Config.general.product));
		}

		if (Ext.isOpera) {
			statusBar.addMessageWarning(Sb.Locale.get('unsupportedBrowserOpera', Sb.Config.general.product));
		}

	},

	/**
	 * Determines, whether we can show context menu
	 *
	 * @param event
	 * @return {Boolean}
	 */
	isShowContextMenu: function (event) {
		if (event.getTarget('.' + Sb.WidgetTypeManager.get('search').activeResultsClassName)) {
			return false;
		}
		return true;
	},

	onContextMenu: function (event, element, options) {
		if (!this.isShowContextMenu(event)) {
			return;
		}

		event.stopEvent();

		var items = [],
			verticalContainer,
			containerForAdd;

		if (options.type == 'container') {
			verticalContainer = containerForAdd = Sb.Editor.model.rootPage.widget.widgets.get(options.region);
		} else if (options.type == 'widget') {
			if (options.widget.model.get('type') == 'pagecontent') {
				containerForAdd = Sb.Editor.siteView.currentPage.widget.widgets.get(options.widget.model.get('placeholder'));
			}
			verticalContainer = options.widget.model;
			while (verticalContainer && verticalContainer.get('type') != 'container_vertical') {
				if (!containerForAdd && ['columns_column', 'blog_post'].indexOf(verticalContainer.get('type')) != -1) {
					containerForAdd = verticalContainer;
				}
				verticalContainer = verticalContainer.ownerWidget;
			}

			if (!containerForAdd) {
				containerForAdd = verticalContainer;
			}
			if (!options.widget.model.isInSiteWideArea()) {
				verticalContainer = Sb.Editor.model.rootPage.widget.widgets.get(verticalContainer.get('region'));
			}
		}

		if (options.type == 'container' || options.type == 'widget') {
			var widgetItems = [];
			Ext.each(Sb.Config.widgets, function (widget) {
				var menuItemCls = '',
					type,
					limit;

				if (widget == '-') {
					if (widgetItems[widgetItems.length - 1] != '-') {
						widgetItems.push('-');
					}
					return;
				}

				type = Sb.WidgetTypeManager.get(widget);
				if (type.isDroppableTo(containerForAdd)) {
					limit = Sb.Editor.model.get('limits')[type.getPhpType()];
					if (limit > -1 && type.cache.length >= limit) {
						menuItemCls = 'sb-upsell-menu-item';
					}

					widgetItems.push({
						text: type.getTranslation('title'),
						cls: menuItemCls,
						iconCls: 'sb-' + widget.toLowerCase() + '-icon',
						disabled: !type.isAvailable(),
						handler: function () {
							containerForAdd.createChild(type, null, function (widget) {
								if (Sb.Limits.checkWidgets(Sb.Editor.model, widget.get('type')).isExceeded) {
									Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
										Sb.Locale.get(
											'moduleLimitExceeded',
											widget.getType().getTranslation('title')
										)
									);
								}
							});
						}
					});
				}
			});
			items.push({
				text: Sb.Locale.get('cmAddWidget'),
				iconCls: 'sb-add-widget-icon',
				hideOnClick: false,
				menu: {
					cls: 'sb-add-widget-menu',
					items: widgetItems
				}
			});
		}

		var mainTabs = Sb.Editor.Ui.mainWindow.tabs;

		if (options.type == 'watermark') {
			items.push({
				text: Sb.Locale.get('cmLayoutMargins'),
				iconCls: 'sb-layout-margins-icon',
				handler: function () {
					var styleTab = mainTabs.getComponent('style');
					styleTab.setActiveTab('layout');
				}
			});
		}

		if (verticalContainer && ['header', 'footer'].indexOf(verticalContainer.get('region')) == -1) {
			items.push({
				text: Sb.Locale.get('cmLayoutWidth'),
				iconCls: 'sb-layout-width-icon',
				handler: function () {
					var styleTab = mainTabs.getComponent('style');
					styleTab.setActiveTab('layout');
				}
			});
		}

		if (verticalContainer) {
			items.push({
				text: Sb.Locale.get('cmStyleBackground'),
				iconCls: 'sb-style-colors-icon',
				handler: function () {
					var styleTab = mainTabs.getComponent('style');
					styleTab.setActiveTab('colors');
					styleTab.form.findField('bgPlace').setValue(verticalContainer.get('region'));
					styleTab.tabs.activeTab.onBgPlaceSelect(verticalContainer.get('region'));
				}
			}, {
				text: Sb.Locale.get('cmStyleCorners'),
				iconCls: 'sb-style-corners-icon',
				handler: function () {
					var styleTab = mainTabs.getComponent('style');
					styleTab.setActiveTab('styleCorner');
					var containerComboBox = styleTab.tabs.activeTab.items.first().items.find(function (object) {
						return (object.refName == 'selectedContainer');
					});
					containerComboBox.setValue(verticalContainer.get('region'));
					styleTab.tabs.activeTab.onContainerSelect(containerComboBox);
				}
			});
		}

		var pageContent = Ext.get(element).up('.sb-widget-pagecontent');
		if (!pageContent && options.region && Ext.get(options.region)) {
			pageContent = Ext.get(options.region).child('.sb-widget-pagecontent');
		}

		if (pageContent) {
			items.push({
				text: Sb.Locale.get('cmPagecontent'),
				iconCls: 'sb-pagecontent-icon',
				handler: function () {
					Ext.getCmp(pageContent.getAttribute('id')).startEdit();
				}
			});
		}

		// Common menu
		if (items.length) {
			items.push('-');
		}
		items.push({
			text: Sb.Locale.get('cmStyleFonts'),
			iconCls: 'sb-style-fonts-icon',
			handler: function () {
				mainTabs.getComponent('style').setActiveTab('fonts');
			}
		}, {
			text: Sb.Locale.get('cmLayout'),
			iconCls: 'sb-layout-icon',
			handler: function () {
				mainTabs.getComponent('style').setActiveTab('layout');
			}
		}, {
			text: Sb.Locale.get('cmStyleScheme'),
			iconCls: 'sb-style-scheme-icon',
			handler: function () {
				mainTabs.getComponent('style').setActiveTab('scheme');
			}
		}, {
			text: Sb.Locale.get('cmStyleBorders'),
			iconCls: 'sb-style-borders-icon',
			handler: function () {
				mainTabs.getComponent('style').setActiveTab('styleBorder');
			}
		}, {
			text: Sb.Locale.get('cmPages'),
			iconCls: 'sb-pages-icon',
			handler: function () {
				mainTabs.setActiveTab('pages');
			}
		}, {
			text: Sb.Locale.get('cmSettings'),
			iconCls: 'sb-settings-icon',
			handler: function () {
				mainTabs.setActiveTab('settings');
			}
		}, '-', {
			text: Sb.Locale.get('cmHelp'),
			iconCls: 'sb-help-icon',
			handler: function () {
				if (null == options.type) {
					return;
				}
				var articleUrl	= '';
				articleUrl		+= options.type.toUpperCaseFirst();
				if (null != options.region) {
					articleUrl	+= '/' + options.region.toUpperCaseFirst();
				}
				Sb.Editor.openHelp(articleUrl);
			}
		});

		if (!this.menu) {
			this.menu = new Ext.menu.Menu({
				listeners: {
					scope: this,
					hide: function (m) {
						this.isHoverDisabled = false;
					}
				}
			});
		} else {
			this.menu.removeAll();
		}
		this.menu.add(items);
		this.isHoverDisabled = true;
		this.menu.showAt(event.getXY());
	}
});