/**
 * @class Sb.model.Node
 * @extends Ext.util.Observable
 * Base class for model nodes
 * @constructor
 * @param {Object} config The configuration options for the model element
 */
Ext.define('Sb.model.Node', {
	extend: 'Ext.util.Observable',

	isLoaded: false,
	isLoading: false,
	translatableFields: [],

	api: {
		load: undefined,
		save: undefined
	},

	/**
	 * Readonly flag - true if this Record has been modified.
	 * @type Boolean
	 */
	dirty: false,

	/**
	 * @cfg {String} persistanceProperty The property on this Persistable object that its data is saved to.
	 * Defaults to 'data' (e.g. all persistable data resides in this.data.)
	 */
	persistanceProperty: 'data',

	constructor: function (config) {
		config = config || {};

		this.addEvents(
			/**
			 * @event load
			 * Fires when the node has been successfuly loaded and unserialized.
			 * @param {Sb.model.Node} this This node.
			 */
			'load',
			/**
			 * @event save
			 * Fires when the node has been successfully saved.
			 * @param {Sb.model.Node} this This node.
			 */
			'save',
			/**
			 * @event change
			 * Fires after the model node's data has changed.
			 * @param {Sb.model.Node} this This node.
			 */
			'change',
			/**
			 * @event propertychange
			 * Fires after the model node's property has changed.
			 * @param {String} property
			 * @param {String} newValue
			 * @param {String} oldValue
			 */
			'propertychange',
			/**
			 * @event remove
			 * Fires after the model node's has removed from storage completely.
			 */
			'remove'
		);

		this[this.persistanceProperty] = {};

		/**
		 * @property {Ext.util.MixedCollection} associations
		 * The collection of associations between this model and another model via hasMany or hasOne association.
		 */
		this.associations = new Ext.util.MixedCollection(false, function (association) {
			return association.name;
		});
		if (config.associations) {
			Ext.each(config.associations, function (association) {
				switch (association.type) {
					case 'hasMany':
						this[association.name] = new Ext.util.MixedCollection();
						break;
					case 'hasOne':
						this[association.name] = new Sb.model[association.model]();
						break;
					default:
						throw new Error('Unknown Association type: "' + association.type + '"');
				}
				this.associations.add(association);
			}, this);

			delete config.associations;
		}

		Sb.model.Node.superclass.constructor.call(this);
		this.unserialize(config);
		this.dirty = false;
	},

	/**
	 * @param {String} newLocale
	 */
	changeLocale: function (newLocale) {
		var oldLocale = this.localeCode || Sb.Editor.model.get('locale'),
			data;

		if (newLocale === oldLocale) {
			return;
		}

		data = this.switchTranslations(this[this.persistanceProperty], newLocale, oldLocale, this.translatableFields);
		this.localeCode = newLocale;
		this.unserialize(data);
	},

	/**
	 * @param {Object} data
	 * @param {String} newLocale
	 * @param {String} oldLocale
	 * @param {Array} translatableFields
	 * @returns {Object}
	 * @protected
	 */
	switchTranslations: function (data, newLocale, oldLocale, translatableFields) {
		var defaultLocale = Sb.Editor.model.get('locale'),
			oldData = {};

		data = Ext.clone(data);

		if (translatableFields.length === 0) {
			return data;
		}

		data.translations = data.translations || {};
		data.translations[newLocale] = data.translations[newLocale] || {};

		Ext.each(translatableFields, function (key) {
			var value = data[key];
			if (oldLocale === defaultLocale || (value !== data.translations[defaultLocale][key] && value !== '')) {
				oldData[key] = value;
			}
		}, this);
		data.translations[oldLocale] = oldData;

		Object.keys(data.translations[newLocale]).forEach(function (key) {
			if (data.translations[newLocale][key] === '') {
				delete data.translations[newLocale][key];
			}
		});

		Ext.apply(data, data.translations[newLocale], data.translations[defaultLocale]);
		delete data.translations[newLocale];

		return data;
	},

	getTranslation: function (localeCode, key) {
		var translations = this.get('translations') || {};
		if (translations[localeCode] && translations[localeCode][key]) {
			return translations[localeCode][key];
		}
		return this.get(key);
	},

	updateTranslationList: function (locales) {
		this.clearTranslationList(this[this.persistanceProperty].translations, locales);
	},

	/**
	 * @param {Array} translations
	 * @param {Array} locales
	 * @protected
	 */
	clearTranslationList: function (translations, locales) {
		if (!translations) {
			return;
		}

		Object.keys(translations).forEach(function (key) {
			if (locales.indexOf(key) === -1) {
				delete translations[key];
			}
		});
	},

	/**
	 * Returns the value of the given field
	 * @param {String} fieldName The field to fetch the value for
	 * @return {String|Object} The value
	 */
	get: function (fieldName) {
		return this[this.persistanceProperty][fieldName];
	},

	/**
	 * Sets the given field to the given value, marks the instance as dirty
	 * @param {String|Object} fieldName The field to set, or an object containing key/value pairs
	 * @param {String|Object} value The value to set
	 * @return {Boolean} Result of setting
	 */
	set: function (fieldName, value) {
		if (arguments.length == 1 && Ext.isObject(fieldName)) {
			return this.unserialize(fieldName);
		}

		var data = {};
		data[fieldName] = value;
		return this.unserialize(data);
	},

	/**
	 * Updates the attributes and child of this Node from server.
	 */
	load: function () {
		if (this.isLoading) { // if an async load is already running, skipping new requests
			return;
		}

		if (this.api.load) {
			this.isLoading = true;
			this.api.load(this.loadCallback, this);
		}
	},

	/**
	 * Load callback
	 *
	 * @param {Object} result
	 * @param {Ext.Direct.Event} event
	 */
	loadCallback: function (result, event) {
		this.isLoading = false;
		if (!event.status) {
			return;
		}

		this.isLoaded = true;
		this.unserialize(result);
		this.dirty = false;
		this.fireEvent('load', this);
	},

	/**
	 * Fill model attributes and child models from JSON
	 *
	 * @param {Object} data May have key rootData
	 * @return {Boolean} Result of unserializing
	 */
	unserialize: function (data) {
		var changedProperties = [],
			isChanged = false;

		// translations should be hash object
		if (Ext.isArray(data.translations)) {
			delete data.translations;
		}

		Ext.iterate(data, function (key, value) {
			var association = this.associations.get(key);
			if (association) {
				switch (association.type) {
					case 'hasMany':
						Ext.each(data[association.name], function (associationData) {
							var model = new Sb.model[association.model](associationData);
							this[association.name].add(model);
						}, this);
						delete data[association.name];
						isChanged = true;
						break;
					case 'hasOne':
						isChanged = this[association.name].unserialize(data[association.name]) || isChanged;
						break;
					default:
						throw new Error('Unknown Association type: "' + association.type + '"');
				}
			} else if (this[this.persistanceProperty][key] instanceof Sb.model.Node) {
				value.rootData = data.rootData;
				isChanged = this[this.persistanceProperty][key].unserialize(value) || isChanged;
				delete this[this.persistanceProperty][key].rootData;
			} else if (Ext.isObject(value)) {
				isChanged = this.unserializeObject(key, value) || isChanged;
			} else {
				isChanged = this.unserializeProperty(key, value) || isChanged;
			}
			if (isChanged) {
				changedProperties.push(key);
			}
		}, this);

		if (isChanged) {
			this.dirty = true;
			this.fireEvent('change', this, changedProperties);
		}
		return isChanged;
	},

	/**
	 * Unserializes objects
	 * @param {String} key
	 * @param {Object} value
	 * @return {Boolean} Result of unserializing
	 */
	unserializeObject: function (key, value) {
		this[this.persistanceProperty][key] = Ext.apply({}, value);
		return true;
	},

	/**
	 * Unserializes single property
	 * @param {String} key
	 * @param {String|Number|Boolean} value
	 * @return {Boolean} Result of unserializing
	 */
	unserializeProperty: function (key, value) {
		var oldValue = this[this.persistanceProperty][key],
			isChanged;
		if (Ext.isArray(value)) {
			value = value.slice(0);
			isChanged = !value.isEqual(oldValue);
		} else {
			isChanged = oldValue !== value;
		}
		if (isChanged) {
			if (this.isDependentProperty(key)) {
				this[this.persistanceProperty][key + 'IsDependent'] = 0;
			}
			this[this.persistanceProperty][key] = value;
			this.onUnserializePropertyChange(key);
			this.fireEvent('propertychange', key, value, oldValue);
			return true;
		}
		return false;
	},

	/**
	 * Fires on changing of single property change to allow us fire up more abstract change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: Ext.emptyFn,

	/**
	 * Restore node`s state from memento
	 *
	 * @param memento
	 */
	restoreFromMemento: function (memento) {
		this.unserialize(Ext.clone(memento));
	},

	/**
	 * Save model.
	 *
	 * @param {Object} [callbacks] Object with success and failure callbacks.
	 */
	save: function (callbacks) {
		if (this.api.save) {
			this.api.save(
				this.normalizeData(this.serialize()),
				this.saveCallback.createDelegate(this, [callbacks], true)
			);
		}
	},

	/**
	 * Saves model if it actually changed
	 * @param {Object} mementoBefore Memento object before changing
	 * @param {Object} [callbacks] Object with success and failure callbacks.
	 * @return {Boolean} Tells us if model save was actually started
	 */
	saveIfChanged: function (mementoBefore, callbacks) {
		var before	= Ext.util.JSON.encode(mementoBefore),
			now		= Ext.util.JSON.encode(this.saveToMemento());
		if (before === now) {
			return false;
		}
		this.save(callbacks);
		return true;
	},

	/**
	 * Save callback
	 *
	 * @param {Object} result
	 * @param {Ext.Direct.Event} event
	 * @param {Object} [callbacks]
	 * @param {Object} [scope]
	 */
	saveCallback: function (result, event, callbacks, scope) {
		callbacks = callbacks || {};
		scope = scope || window;
		if (event.status) {
			this.dirty = false;
			this.fireEvent('save', this);
			Ext.callback(callbacks.success, scope, [result]);
		} else {
			Ext.callback(callbacks.failure, scope, [result]);
		}
	},

	/**
	 * Create JSON from model attributes and child models
	 *
	 * @param {Object} [rootData] Data of object, that intiates serialize. Should be passed in each child serilize.
	 * @return {Object}
	 */
	serialize: function (rootData) {
		var data = {};
		Ext.iterate(this[this.persistanceProperty], function (key, value) {
			if (value instanceof Sb.model.Node) {
				data[key] = value.serialize(rootData);
			} else {
				data[key] = Ext.clone(value);
			}
		}, this);

		this.associations.each(function (association) {
			switch (association.type) {
				case 'hasMany':
					data[association.name] = [];
					this[association.name].each(function (model) {
						data[association.name].push(model.serialize(rootData));
					});
					break;
				case 'hasOne':
					data[association.name] = this[association.name].serialize(rootData);
					break;
				default:
					throw new Error('Unknown Association type: "' + association.type + '"');
			}
		}, this);

		return data;
	},

	saveToMemento: function () {
		return this.normalizeData(this.serialize());
	},

	normalizeData: function (data) {
		var localeCode = this.localeCode || Sb.Editor.model.get('locale');
		return this.switchTranslations(
			data, Sb.Editor.model.get('locale'), localeCode, this.translatableFields
		);
	},

	/**
	 * Destroys a model, calling common onDestroy
	 */
	destroy: function () {
		if (!this.isDestroyed) {
			this.onDestroy();
			this.isDestroyed = true;
		}
	},

	onDestroy: Ext.emptyFn,

	/**
	 * Tells us, when property is dependent.
	 * Dependent property is a property, that have property with same name and postfix "IsDependent" with positive value.
	 * When value of dependent property changes, it becomes independent and set "IsDependent" to negative.
	 *
	 * @param {String} propertyName
	 * @returns {Boolean}
	 */
	isDependentProperty: function (propertyName) {
		return (1 == this[this.persistanceProperty][propertyName + 'IsDependent']) ? true : false;
	},

	/**
	 * Tells us whether this dependent has changed from generator
	 * @property isChangedAsDependent
	 * @type {Boolean}
	 */
	isChangedAsDependent: false,

	/**
	 * Default dependent property update mechanism, that made it independent after 1st update.
	 *
	 * @param {String} key
	 * @param {String} value
	 * @returns {Boolean} Result of setting
	 */
	updateDependentProperty: function (key, value) {
		if (this.isDependentProperty(key) && this.set(key, value)) {
			// :INFO: austimenko@ 20110203: Restore dependency, as it get destroyed by any other setting
			this[this.persistanceProperty][key + 'IsDependent'] = 1;
			this.isChangedAsDependent = true;
			return true;
		}
		return false;
	},

	/**
	 * Saves dependent, if it`s actually changed during changes in generator
	 */
	saveIfChangedAsDependent: function () {
		if (!this.isChangedAsDependent) {
			return false;
		}
		this.isChangedAsDependent = false;
		return this.save();
	}
});
