/**
 * @class Sb.model.SitePage
 * @extends Sb.model.Node
 * Class for site page model
 * @constructor
 * @param {Object} config The configuration options for the site page model
 */
Ext.define('Sb.model.SitePage', {
	extend: 'Sb.model.Node',

	statics: {
		ROOT_TYPE: 0,
		SIMPLE_TYPE: 1,
		LINK_TYPE: 2
	},

	translatableFields: [
		'title',
		'browserTitle',
		'description',
		'keywords'
	],

	constructor: function (config) {
		config = Ext.apply({
			associations: [
				/**
				 * The collection of subpages of this page
				 * @type Ext.util.MixedCollection
				 * @property pages
				 */
				{type: 'hasMany', model: 'SitePage', name: 'pages'},
				/**
				 * The model node of page widget
				 * @type Sb.model.Widget
				 * @property widget
				 */
				{type: 'hasOne', model: 'Widget', name: 'widget'}
			],

			sitePageTypeId	: Sb.model.SitePage.SIMPLE_TYPE,
			fileName		: '',
			hidden			: false,
			description		: '',
			keywords		: '',
			title			: '',
			browserTitle	: '',
			slug			: '',
			slugIsAuto		: '0',
			login			: null,
			password		: null,
			url				: '',
			target			: ''
		}, config || {});

		this.api = {
			load: Sb.PageService.read,
			save: Sb.PageService.update
		};

		/**
		 * Array of anchors, where anchor is object with properties 'name' and 'widgetId'
		 * @type Array
		 * @property anchors
		 */
		this.anchors = [];

		this.addEvents(
			/**
			 * @event titlevisibilitychange
			 * Fires after the title/visibility has changed.
			 */
			'titlevisibilitychange',
			/**
			 * @event credentialschange
			 * Fires after the login/password has changed.
			 */
			'credentialschange',
			/**
			 * @event urlchange
			 * Fires after page url has changed, for example when slug changed
			 */
			'urlchange'
		);

		Sb.model.SitePage.superclass.constructor.call(this, config);

		if (!this.get('identity')) {
			this.set('identity', Sb.Utils.getShortHash());
		}
	},

	/**
	 * Apply function to self and to children of this page.
	 *
	 * @param {Function} fn
	 */
	applySelfAndRecursive: function (fn) {
		fn.apply(this, [this, 0, 1, 2]);
		this.applyRecursive(fn);
	},

	/**
	 * Recursively apply function to children of this page.
	 *
	 * @param {Function} fn
	 */
	applyRecursive: function (fn) {
		this.pages.each(function (item, index, length) {
			fn.apply(this, [item, index, length]);
			item.applyRecursive(fn);
		}, this);
	},

	unserialize: function (data) {
		var isChanged = false;

		if (undefined !== data.widgetList) {
			data.rootData = {widgetList: data.widgetList};
			delete data.widgetList;
		}

		if (Ext.isArray(data.pages)) {
			this.pages.clear();
			Ext.each(data.pages, function (subpageData) {
				subpageData.rootData = data.rootData;
				var subpage = new Sb.model.SitePage(subpageData);
				if (Sb.Editor.siteView.currentPage && subpageData.identity == Sb.Editor.siteView.currentPage.data.identity) {
					Sb.Editor.siteView.currentPage = subpage;
				}
				this.pages.add(subpageData.identity, subpage);
				subpage.parentPage = this;
			}, this);
			delete data.pages;
			isChanged = true;
		}

		if (Ext.isString(data.widget) && data.rootData !== undefined && data.rootData.widgetList !== undefined) {
			data.widget = data.rootData.widgetList[data.widget];
		}
		if (Ext.isObject(data.widget)) {
			data.widget.rootData = data.rootData;
			this.widget.unserialize(data.widget);
			this.widget.setOwnerPage(this);

			// collect anchors
			this.widget.widgets.each(function (verticalContainer, index, length) {
				verticalContainer.widgets.each(function (widget, index2) {
					if (widget instanceof Sb.model.TextWidget) {
						var anchors = widget.collectAnchors();
						this.anchors = this.anchors.concat(anchors);
					}
				}, this);
			}, this);

			delete data.widget;
			delete this.widget.data.rootData;
			isChanged = true;
		}

		var unserializeResult = Sb.model.SitePage.superclass.unserialize.call(this, data) || isChanged;
		delete this.data.rootData;
		return unserializeResult;
	},

	removeAnchorsByWidget: function (widget) {
		var uuid = widget.get('uuid'), i;
		for (i = this.anchors.length - 1; i >= 0; i--) {
			if (uuid == this.anchors[i].widgetId) {
				this.anchors.splice(i, 1);
			}
		}
	},

	hasAnchor: function (anchorName) {
		var hasAnchor = false;
		Ext.each(this.anchors, function (anchor){
			if (anchor.name == anchorName) {
				hasAnchor = true;
				return false;
			}
		});
		return hasAnchor;
	},

	refreshAnchorsByWidget: function (widget, anchors) {
		this.removeAnchorsByWidget(widget);
		this.anchors = this.anchors.concat(anchors);
	},

	restoreFromMemento: function (memento) {
		Sb.model.SitePage.superclass.restoreFromMemento.call(this, memento);
		Sb.Editor.model.rootPage.fireEvent('change', Sb.Editor.model.rootPage);
	},

	serialize: function (rootData) {
		var isInitiator	= !rootData; rootData = rootData || {};
		var data		= Sb.model.SitePage.superclass.serialize.call(this, rootData);
		var passData	= (isInitiator) ? data : rootData;

		data.widget = this.widget.serialize(passData);

		data.pages = [];
		this.pages.each(function (page) {
			data.pages.push(page.serialize(passData));
		});

		return data;
	},

	/**
	 * Insert page.
	 * @param {Number} index The index to insert the item at.
	 * @param {Sb.model.SitePage} page Site page for inserting.
	 */
	insert: function (index, page) {
		var parentPage = page.parentPage;
		if (parentPage) {
			parentPage.pages.remove(page);
		}
		this.pages.insert(index, page);
		page.parentPage = this;
		if (this != parentPage && page.get('slug') !== '') {
			page.makeSlugUnique();
		}
		Sb.Editor.model.rootPage.fireEvent('change', Sb.Editor.model.rootPage);
	},

	/**
	 * Remove page from tree pages and fireEvent 'change' about change tree
	 * @param {Sb.model.SitePage} page
	 */
	remove: function (page) {
		this.pages.remove(page);
		page.destroy();
		Sb.Editor.model.rootPage.fireEvent('change', Sb.Editor.model.rootPage);
	},

	onDestroy: function () {
		this.widget.destroy();
	},

	/**
	 * Get path from root page to this page
	 *
	 * @return {Array}
	 */
	getPath: function () {
		var path = [this];

		while (path[0].parentPage) {
			path.unshift(path[0].parentPage);
		}

		return path;
	},

	/**
	 * Returns all child pages in plain array.
	 *
	 * @param {String} format Format of result: array|mixedcollection
	 * @return {Array|Ext.util.MixedCollection}
	 */
	getChildren: function (format) {
		var result = [];
		format = (format || 'array').toLowerCase();

		this.applyRecursive(function (page) {
			result.push(page);
		});
		if ('mixedcollection' == format) {
			var resultMc = new Ext.util.MixedCollection(false,
				function (page) {
					return page.data.identity;
				}
			);
			resultMc.addAll(result);
			return resultMc;
		} else {
			return result;
		}
	},

	/**
	 * Gets first non-hidden page
	 *
	 * @return {Sb.model.SitePage} page
	 */
	getFirstVisiblePage: function () {
		var firstPage = null;

		this.pages.each(function (page) {
			if (page.get('sitePageTypeId') != Sb.model.SitePage.LINK_TYPE && !page.data.hidden) {
				firstPage = page;
				return false;
			}
		});

		return firstPage;
	},

	/**
	 * Tells us whether we have our page protected
	 * @returns {Boolean}
	 */
	isProtected: function () {
		return (this.data.login && this.data.password) ? true : false;
	},

	/**
	 * Fires on changing of single property change to allow us fire up navigation widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		if (key == 'title' || key == 'browserTitle' || key == 'hidden') {
			this.fireEvent('titlevisibilitychange');
		} else if (key == 'login' || key == 'password') {
			this.fireEvent('credentialschange');
		} else if (key == 'slug') {
			this.applySelfAndRecursive(function (page) {
				page.fireEvent('urlchange');
			});
		}
	},

	/**
	 * Make a slug of this page unique by adding a suffix
	 */
	makeSlugUnique: function () {
		var slug = this.get('slug');

		var suffix = 0;
		if (Sb.Config.invalidSlugs.indexOf(slug.toLowerCase()) != -1
			|| this.parentPage == Sb.Editor.model.rootPage
			&& (
				Sb.Config.reservedSlugs.indexOf(slug.toLowerCase()) != -1
				|| (/^[a-z]{2}-[a-z]{2}$/).test(slug.toLowerCase())
			)
		) {
			suffix = 1;
		}
		var re = new RegExp('^' + Ext.escapeRe(slug) + '(?:-(\\d+))?$', 'i');
		this.parentPage.pages.each(function (curPage) {
			if (curPage != this && re.test(curPage.get('slug'))) {
				var curSuffix = parseInt(RegExp.$1, 10) || 0;
				if (curSuffix >= suffix) {
					suffix = curSuffix + 1;
				}
			}
		}, this);
		if (suffix > 0) {
			slug += '-' + suffix;
			this.set('slug', slug);
		}
	},

	/**
	 * Generates current page URL so as it will be on published side.
	 *
	 * @return {String} URL as of http://host/path/to/page/
	 */
	getFullUrl: function () {
		var path = this.getPath();

		path.shift();

		return path.reduce(function (url, page) {
			return url + page.get('slug') + '/';
		}, Sb.Editor.model.getPublishUrl());
	},

	/**
	 * Check if this page has commenting module
	 *
	 * @return {Boolean}
	 */
	isPageHasCommenting: function () {
		var result = false;
		this.widget.widgets.each(function (verticalContainer) {
			if (result) {
				return false;
			}
			verticalContainer.widgets.each(function (widget) {
				if (widget instanceof Sb.model.CommentingWidget) {
					result = true;
					return false;
				}
			}, this);
		}, this);
		return result;
	},

	/**
	 * Gets master widget (Eshop Catalog | Blog | Blog Post)
	 *
	 * @return {Sb.model.Widget}
	 */
	getMasterWidget: function () {
		return this.widget.queryBy(function (widget) {
			return widget instanceof Sb.model.BlogWidget
				|| widget instanceof Sb.model.BlogPostWidget
				|| widget instanceof Sb.model.EshopCatalogWidget;
		})[0];
	}
});
