/**
 * @class Sb.model.Widget
 * @extends Sb.model.Node
 * Class for widget model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Ext.define('Sb.model.Widget', {
	extend: 'Sb.model.Node',

	/**
	 * @cfg {String} uuid
	 * The widget uuid (defaults to new generated uuid)
	 */

	/**
	 * @cfg {String} type
	 * text, navigation etc. (defaults to <tt>text</tt>)
	 */

	isShowEditPanel: true,

	isOnExclude: true,

	isConfigured: true,

	constructor: function (config) {
		this.api = {
			save: Sb.WidgetService.save
		};

		config = Ext.apply({
			type: 'text'
		}, config || {});

		this.addEvents(
			/**
			 * @event styleschange
			 * Fires after the model node's styles property has changed.
			 * @param {Object} value
			 */
			'styleschange'
		);

		/**
		 * The collection of child widgets as a {@link Ext.util.MixedCollection}
		 * @type Ext.util.MixedCollection
		 * @property widgets
		 */
		this.widgets = new Ext.util.MixedCollection(false, function (childWidget) {
			if (childWidget instanceof Sb.model.ContainerVerticalWidget) {
				return childWidget.data.region;
			}
			if (childWidget instanceof Sb.model.Widget) {
				return childWidget.data.type + childWidget.data.uuid;
			}
		});
		this.widgets.on('add', this.onWidgetAdd, this);
		this.widgets.on('remove', this.onWidgetRemove, this);

		Sb.model.Widget.superclass.constructor.call(this, config);

		if (!this.get('uuid')) {
			this.set('uuid', Sb.Utils.generateUuid());
		}
	},

	onWidgetAdd: function (index, widget, key) {
		widget.ownerWidget = this;
		widget.setOwnerPage(this.ownerPage);
	},

	onWidgetRemove: function (widget) {
		delete widget.ownerWidget;
		widget.setOwnerPage(undefined);
	},

	/**
	 * Set the page that has this widget
	 *
	 * @param {Sb.model.SitePage} page
	 */
	setOwnerPage: function (page) {
		this.ownerPage = page;
		this.widgets.each(function (widget) {
			widget.setOwnerPage(this.ownerPage);
		}, this);
		this.getType().cache[page ? 'add' : 'remove'](this);
	},

	onDestroy: function () {
		this.widgets.un('add', this.onWidgetAdd, this);
		this.widgets.un('remove', this.onWidgetRemove, this);
		this.widgets.each(function (widget) {
			widget.destroy();
		});
		this.getType().cache.remove(this);
		this.fireEvent('remove');
		Sb.model.Widget.superclass.onDestroy.call(this);
	},

	unserialize: function (data) {
		var isChanged = false;

		// :TODO: 20120319: Temporary solution for list of child widgets. Need global reworking
		if (undefined !== data.widget_list) {
			data.widgetList = {};
			Ext.each(data.widget_list, function (widgetData) {
				data.widgetList[widgetData.uuid] = widgetData;
			});
			delete data.widget_list;
		}

		if (undefined !== data.widgetList) {
			data.rootData = {widgetList: data.widgetList};
			delete data.widgetList;
		}

		if (Ext.isArray(data.widgets)) {
			if (this.widgets.getCount() > 0) {
				this.widgets.each(function (widget) {
					widget.destroy();
				});
				this.widgets.clear();
			}
			Ext.each(data.widgets, function (widgetData) {
				if (Ext.isString(widgetData) && data.rootData !== undefined && data.rootData.widgetList !== undefined) {
					widgetData = data.rootData.widgetList[widgetData];
					if (widgetData) {
						widgetData.rootData = data.rootData;
					}
				}
				// reInitUuid - need for save widgets from special design presets
				widgetData.reInitUuid = data.reInitUuid;
				var widget = Sb.WidgetTypeManager.create(widgetData);
				delete widget.data.rootData;
				this.widgets.add(widget);
			}, this);
			delete data.widgets;
			isChanged = true;
		}

		if (data.reInitUuid) {
			data.uuid = Sb.Utils.generateUuid();
		}
		delete data.reInitUuid;

		if (data.styles) {
			isChanged = this.unserializeStyles(data.styles) || isChanged;
			delete data.styles;
		}
		if (data.hidden) {
			this.hidden = data.hidden;
			delete data.hidden;
		}
		isChanged = Sb.model.Widget.superclass.unserialize.call(this, data) || isChanged;
		delete this.data.rootData;
		return isChanged;
	},

	/**
	 * Unserializes styles
	 * @param {Object} newStyles
	 * @return {Boolean} Result of unserializing
	 */
	unserializeStyles: function (newStyles) {
		var changedStyles = {},
			isChanged = false,
			oldStyles = null;

		if (this[this.persistanceProperty].styles) {
			oldStyles = Ext.apply({}, this[this.persistanceProperty].styles);
		} else {
			this[this.persistanceProperty].styles = {};
		}

		Ext.iterate (newStyles, function (key, value) {
			if (!oldStyles || value !== oldStyles[key]) {
				changedStyles[key] = value;
				isChanged = true;
				this[this.persistanceProperty].styles[key] = value;
			}
		}, this);

		if (!isChanged) {
			return false;
		}

		this.fireEvent('styleschange', changedStyles);
		return true;
	},

	serialize: function (rootData) {
		var isInitiator, data, passData;

		isInitiator = !rootData;
		rootData = rootData || {};
		data = Sb.model.Widget.superclass.serialize.call(this, rootData);
		passData = (isInitiator) ? data : rootData;

		if (!rootData.widgetList) {
			rootData.widgetList = {};
		}
		if (!rootData.widgetList[this.data.uuid]) {
			data.widgets = [];
			this.widgets.each(function (widget) {
				data.widgets.push(widget.serialize(passData));
			});
			rootData.widgetList[this.data.uuid] = data;
		}

		return (isInitiator) ? data : this.data.uuid;
	},

	/**
	 * Create child widget.
	 * @param {Sb.widgetType.Base} type Type of the new widget
	 * @param {Number} index The index to insert the item at.
	 * @param {Function} callback The function to call after success response from remote Service. Arguments of call is created Widget
	 * @param {Object} scope (optional) The scope (<code>this</code> reference) in which the callback function is executed.
	 */
	createChild: function (type, index, callback, scope) {
		Sb.WidgetService.createChild(this.data.uuid, type.getPhpType(), index, function (result, response) {
			if (!response.status) {
				return;
			}
			var widget = Sb.WidgetTypeManager.create(result);
			widget.isNew = true;
			if (Ext.isNumber(index)) {
				this.widgets.insert(index, widget);
			} else {
				this.widgets.add(widget);
			}
			Ext.callback(callback, scope, [widget]);
		}, this);
	},

	/**
	 * Remove child widget.
	 * @param {Sb.model.Widget|Number} item widgetModel or Position Removes widget from model.
	 */
	removeChild: function (item) {
		var index, widgetModel;

		if (item instanceof Sb.model.Widget) {
			widgetModel = item;
			index = this.widgets.indexOf(widgetModel);
		} else {
			widgetModel = this.widgets.item(item);
			index = item;
		}

		if (index === -1) {
			return;
		}

		Sb.WidgetService.removeChild(this.data.uuid, widgetModel.get('uuid'), function (result, response) {
			if (!response.status) {
				return;
			}
			this.widgets.remove(widgetModel);
			widgetModel.destroy();
		}, this);
	},

	/**
	 * Insert widget.
	 * @param {Number} index The index to insert the item at.
	 * @param {Sb.model.Widget} widget Widget for inserting.
	 */
	insert: function (index, widget) {
		if (index === undefined) {
			return;
		}
		if (widget.ownerWidget.widgets.indexOf(widget) === -1) {
			return;
		}

		if (widget.ownerWidget.isOnExclude) {
			if (!widget.ownerWidget.onExclude(this, widget, index)) {
				return;
			}
		}

		Sb.WidgetService.insertChild(this.data.uuid, index, widget.ownerWidget.data.uuid, widget.get('uuid'), function (result, response) {
			if (!response.status) {
				return;
			}
			widget.ownerWidget.widgets.remove(widget);
			this.widgets.insert(index, widget);
		}, this);
	},

	onExclude: function (container, widget, position) {
		return true;
	},

	/**
	 * Returns an array of matched Widgets from within the this widget.
	 */
	queryBy: function (fn, scope) {
		var results = [];

		if (fn.call(scope || this, this) === true) {
			results.push(this);
		}
		this.widgets.each(function (child) {
			results = results.concat(child.queryBy(fn, scope));
		});

		return results;
	},

	/**
	 * Returns the <code>type</code> of this widget
	 *
	 * @return {Sb.widgetType.Base}
	 */
	getType: function () {
		return Sb.WidgetTypeManager.get(this.data.type);
	},

	/**
	 * Returns true if this widget placed in site wide area (shared widget), false otherwise.
	 * @return {Boolean}
	 */
	isInSiteWideArea: function () {
		return this.ownerPage == Sb.Editor.model.rootPage;
	},

	/**
	 * Returns url to folder with resources (without attachments dir).
	 *
	 * @return {String}
	 */
	getResourcesUrl: function () {
		if (this.ownerPage === Sb.Editor.model.rootPage) {
			return Sb.Editor.model.design.getResourcesUrl();
		}
		return Sb.Editor.model.getUrl();
	},

	/**
	 * Checking that all necessary widget property is configured.
	 *
	 * @returns {boolean}
	 */
	isConfiguredProperties: function () {
		return this.isConfigured;
	},

	updateIsConfiguredProperties: function () {
		var isConfigured = this.computeIsConfiguredProperties();
		if (this.isConfigured !== isConfigured) {
			this.isConfigured = isConfigured;
			this.getType().checkUpsellChange();
		}
		return isConfigured;
	},

	computeIsConfiguredProperties: function () {
		return true;
	},

	getComputedStyles: function() {
		var parentStyles, styles;

		styles = Ext.clone(this.get('styles'));
		Ext.iterate(styles, function (key, value) {
			if (value == 'inherit') {
				if (parentStyles === undefined) {
					parentStyles = this.ownerWidget ? this.ownerWidget.getComputedStyles() : {};
				}
				if (parentStyles[key] !== undefined) {
					styles[key] = parentStyles[key];
				}
			}
		}, this);

		return styles;
	}
});