/**
 * @class Sb.widget.ContainerFixed
 * @extends Ext.Container
 * @constructor
 * @param {Object} config The config object
 * @xtype container_fixed_widget
 */
Ext.define('Sb.widget.ContainerFixed', {
	extend: 'Ext.Container',
	xtype: 'container_fixed_widget',

	/**
	 * @cfg {Sb.model.Widget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	/**
	 * @type {Object}
	 */
	pressColumnsToBottomTask: null,

	/**
	 * @type {Object}
	 */
	pressWatermarkToBottomTask: null,

	initComponent : function () {
		this.items = [];
		this.model.widgets.each(function (widget) {
			this.items.push({
				xtype : widget.data.type + '_widget',
				model : widget
			});
		}, this);

		Sb.widget.ContainerFixed.superclass.initComponent.call(this);
	},

	destroy: function () {
		this.enablePressColumnsToBottom(false);
		this.enableWatermarkToBottom(false);
		if (this.promoFooter) {
			this.promoFooter.destroy();
		}
		Sb.widget.ContainerFixed.superclass.destroy.call(this);
	},

	/**
	 * Prepare layout skeleton
	 * @param ct
	 * @param position
	 */
	onRender : function (ct, position) {
		var partial, markup;

		partial = new Sb.PartialTemplate(Sb.Editor.model.data.template.partials.page);
		markup = partial.apply({}).trim();

		markup = markup.replace(/<Sitebuilder\:(\w+)((\s+(\w+)\s*=\s*"([^"]*)")*)\s*>/g, function () {
			var attrs = {};
			Ext.each(arguments[2].match(/(\w+)\s*=\s*"([^"]*)"/g), function(matches) {
				var m = matches.match(/(\w+)\s*=\s*"([^"]*)"/);
				attrs[m[1]] = m[2];
			});
			return '<div id="sb-container-' + attrs.id + '"></div>';
		});
		markup = markup.replace(/images\/s\.gif/g, Sb.Config.blankImageUrl);

		this.el = ct.insertHtml('afterBegin', markup, true);
		this.layoutEl = this.el.down('#layout');
		this.watermarkEl = this.el.down('#watermark');
		this.watermarkBackgroundEl = this.watermarkEl.child('.watermark-content');
		this.columnsEl = this.layoutEl.down('#columns');
	},

	/**
	 * Completing operations with layout
	 */
	afterRender : function () {
		this.initPromoFooter();

		this.setStylesForLayout();
		this.insertContent();
		this.setStylesForColumns();

		this.onDecorationChange();
		this.onBorderRadiusChange();

		this.mon(Sb.Editor.model.design, {
			scope: this,
			decorationchange: this.onDecorationChange,
			backgroundcolorchange: this.onDecorationChange,
			colorschemechange: this.onDecorationChange,
			layoutchange: this.onLayoutChange,
			borderradiuschange: this.onBorderRadiusChange,
			pagelayoutchange: this.onPageLayoutChange,
			save: this.doResize
		});
		this.mon(this.watermarkEl, 'contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
			type: 'watermark',
			region: 'watermark'
		});
	},

	/**
	 * Substitute layout placeholders and insert widgets content
	 */
	insertContent: function () {
		this.items.each(function (item) {
			var placeholder = Ext.getDom('sb-container-' + item.model.data.region);
			item.render(placeholder);
			item.el.insertAfter(placeholder);
			Ext.fly(placeholder).remove();
			this[item.model.data.region] = item;
		}, this);
	},

	onDecorationChange: function (name, value, type) {
		this.onBackgroundColorChange();

		Ext.each (['site', 'watermark', 'header', 'content', 'footer'], function (region) {
			if (type) {
				if (-1 != type.indexOf(region)) {
					this.setBackground(region);
					return false;
				}
			} else {
				this.setBackground(region);
			}
		}, this);
		this.setStylesForBorder();

		if (this.promoFooter) {
			if (this.getCurrentExternalBorderName() === 'external-border-none') {
				this.promoFooter.removeClass('after-external-border');
			} else {
				this.promoFooter.addClass('after-external-border');
			}
		}
	},

	/**
	 * Update styles when layout was changed
	 */
	onLayoutChange: function () {
		this.setStylesForLayout();
		this.setStylesForColumns();
		this.setStylesForBorder();
		this.onBackgroundColorChange();

		if (Ext.isWebKit) {
			this.repaintLayout();
		}
	},

	setStylesForLayout: function () {
		var data = Sb.Editor.model.design.data.layout,
			horizontalMargin = parseInt(data.horizontalMargin, 10),
			sitebarFirstMargin = data.sidebarFirst ? parseInt(data.sidebarFirstWidth, 10) + horizontalMargin : 0,
			sitebarSecondMargin = data.sidebarSecond ? parseInt(data.sidebarSecondWidth, 10) + horizontalMargin	: 0,
			width = this.getCssWidth(data.width),
			minWidth = sitebarFirstMargin + sitebarSecondMargin + 100;

		this.layoutEl.setStyle({
			minWidth: minWidth + 'px'
		});

		this.watermarkEl.setStyle({
			width: width
		});
		if (data.headerLayout == 'watermark') {
			this.layoutEl.down('#layout-header').setStyle({
				width: width
			});
		}
		if (data.contentLayout == 'watermark') {
			this.layoutEl.down('#layout-content').setStyle({
				width: width
			});
		}
		if (data.footerLayout == 'watermark') {
			this.layoutEl.down('#layout-footer').setStyle({
				width: width
			});
		}
		this.columnsEl.setStyle({
			paddingTop: data.verticalMargin + 'px',
			paddingBottom: data.verticalMargin + 'px'
		});
	},

	setStylesForColumns: function () {
		var data = Sb.Editor.model.design.data.layout,
			horizontalMargin = parseInt(data.horizontalMargin, 10),
			sitebarFirstMargin = data.sidebarFirst ? parseInt(data.sidebarFirstWidth, 10) + horizontalMargin : 0,
			sitebarSecondMargin = data.sidebarSecond ? parseInt(data.sidebarSecondWidth, 10) + horizontalMargin	: 0,
			sidebarFirstPosition = data.sidebarFirstPosition,
			sidebarSecondPosition = data.sidebarSecondPosition,
			contentLeftMargin = (sidebarFirstPosition == 'left') ? sitebarFirstMargin : sitebarSecondMargin,
			contentRightMargin = (sidebarFirstPosition == 'left') ? sitebarSecondMargin : sitebarFirstMargin;

		if (Sb.Editor.model.isRtl()) {
			sidebarFirstPosition = Sb.Utils.swapLeftAndRight(sidebarFirstPosition);
			sidebarSecondPosition = Sb.Utils.swapLeftAndRight(sidebarSecondPosition);
		}

		if (!this.column1.el.hasClass('column-' + sidebarFirstPosition)) {
			this.column1.el.addClass('column-' + sidebarFirstPosition);
			this.column1.el.removeClass('column-' + Sb.Utils.swapLeftAndRight(sidebarFirstPosition));
		}
		this.column1.el.setStyle({
			display: data.sidebarFirst ? 'block' : 'none',
			width: data.sidebarFirstWidth + 'px'
		});
		this.column1.contentElement.parent().setStyle({
			minHeight: data.sidebarFirstMinHeight + 'px'
		});

		if (!this.column2.el.hasClass('column-' + sidebarSecondPosition)) {
			this.column2.el.addClass('column-' + sidebarSecondPosition);
			this.column2.el.removeClass('column-' + Sb.Utils.swapLeftAndRight(sidebarSecondPosition));
		}
		this.column2.el.setStyle({
			display: data.sidebarSecond ? 'block' : 'none',
			width: data.sidebarSecondWidth + 'px'
		});
		this.column2.contentElement.parent().setStyle({
			minHeight: data.sidebarSecondMinHeight + 'px'
		});

		this.content.el.setStyle({
			marginLeft: contentLeftMargin + 'px',
			marginRight: contentRightMargin + 'px'
		});
		this.content.contentElement.parent().setStyle({
			minHeight: data.contentMinHeight + 'px'
		});

		Sb.ResourceManager.require(
			[Sb.Config.siteResourcesUrl + '/components/' + Sb.Config.jsFrameworkFile],
			function () {
				Sb.ResourceManager.require(
					[Sb.Utils.antiCache(
						Sb.Config.templatesUrl + '/' + Sb.Editor.model.get('template').code + '/js/helpers.js',
						Sb.Config.isDebug
					)], function () {
						this.enablePressColumnsToBottom(Sb.Editor.model.design.isPressColumnsToBottom());
						this.enableWatermarkToBottom(true);
					}, this);
			}, this);
	},

	repaintLayout: function () {
		Ext.each(['header', 'content', 'footer'], function(containerName) {
			var container = this.layoutEl.down('#layout-' + containerName),
				layout = Sb.Editor.model.design.get('layout'),
				width = '';

			if (layout[containerName + 'Layout'] == 'watermark') {
				width = this.getCssWidth(layout.width);
			}

			container.setWidth('0').setWidth.defer(10, container, [width]);
		}, this);
	},

	/**
	 * @param {Boolean} enable
	 */
	enablePressColumnsToBottom: function (enable) {
		var task = {
			run: function () {
				if (jQuery.fixHeightColumns) {
					jQuery.fixHeightColumns();
				}
			},
			scope: this,
			interval: 1000
		};

		if (enable && !this.pressColumnsToBottomTask) {
			this.pressColumnsToBottomTask = Ext.TaskMgr.start(task);
		} else if (!enable && this.pressColumnsToBottomTask) {
			Ext.TaskMgr.stop(this.pressColumnsToBottomTask);
			this.pressColumnsToBottomTask = null;
			if (jQuery.unfixHeightColumns) {
				jQuery.unfixHeightColumns();
			}
		}
	},

	/**
	 * @param {Boolean} enable
	 */
	enableWatermarkToBottom: function (enable) {
		var task = {
			run: function () {
				if (jQuery.fixWatermarkHeight) {
					jQuery.fixWatermarkHeight();
				}
			},
			scope: this,
			interval: 1000
		};

		if (enable && !this.pressWatermarkToBottomTask) {
			this.pressWatermarkToBottomTask = Ext.TaskMgr.start(task);
		} else if (!enable && this.pressWatermarkToBottomTask) {
			Ext.TaskMgr.stop(this.pressWatermarkToBottomTask);
			this.pressWatermarkToBottomTask = null;
		}
	},

	onPageLayoutChange: function () {
		Sb.Editor.siteView.compilePage();
	},

	/**
	 * Get css code of container's width depends of layout
	 * @param {String|Number} width
	 * @return {String} css width value
	 */
	getCssWidth: function (width) {
		var data = Sb.Editor.model.design.data.layout;

		return (width == '0') ? '' : width + (data.type == 'elastic' ? '%' : 'px');
	},

	/**
	 * Update border round.
	 */
	onBorderRadiusChange: function () {
		Ext.each (['watermark', 'header', 'content', 'footer'], function (region) {
			this.setBorderRadius(region);
		}, this);
	},

	setBorderRadius: function (region) {
		var designSettings = Sb.Editor.model.design.getSettings(),
			borderRadius = Sb.Editor.model.design.getDecoration('borderRadius', region + 'BorderRadius'),
			regionElement = (region == 'watermark')
				? this.watermarkBackgroundEl
				: this.layoutEl.child('#layout-' + region);

		if (
			['header', 'content', 'footer'].indexOf(region) > -1 &&
			designSettings.layout[region + 'Layout'] != 'elastic-background'
		) {
			borderRadius = '';
		}

		regionElement.setStyle({
			'-webkit-border-radius': borderRadius,
			'-moz-border-radius': borderRadius,
			'border-radius': borderRadius
		});
	},

	/**
	 * Update background color.
	 */
	onBackgroundColorChange: function () {
		Ext.each (['site', 'watermark', 'header', 'content', 'footer'], function (region) {
			this.setBackgroundColor(region);
		}, this);

		if (this.promoFooter && this.promoFooter.el) {
			var siteBackground, promoFooterColor;
			siteBackground = Sb.Editor.model.design.getDecoration('backgroundColor', 'siteColor');
			siteBackground = Sb.Editor.model.design.getColor(siteBackground);
			promoFooterColor = Ext.ux.util.Color.isLight(siteBackground) ? 'black' : 'white';
			this.promoFooter.el.setStyle('color', promoFooterColor);
		}
	},

	setBackgroundColor: function (region) {
		var designSettings = Sb.Editor.model.design.getSettings(),
			modelColor = designSettings.backgroundColor[region + 'Color'],
			backgroundColor	= modelColor || 'transparent',
			backgroundElement,
			opacity = Sb.Editor.model.design.getDecoration('background', region + 'BackgroundOpacity');

		if (Sb.Utils.getBackgroundType(designSettings.background[region + 'Background']).value !== 'none') {
			return;
		}

		if (region == 'site') {
			backgroundElement = Ext.getBody();
		} else if (region == 'watermark') {
			backgroundElement = this.watermarkBackgroundEl;
		} else {
			backgroundElement = this.layoutEl.child('#layout-' + region);
		}

		if (
			['header', 'content', 'footer'].indexOf(region) > -1 &&
			designSettings.layout[region + 'Layout'] != 'elastic-background'
		) {
			backgroundColor = 'transparent';
		}
		backgroundColor = Sb.Editor.model.design.getColor(backgroundColor);
		if (Ext.ux.util.Color.isValid(backgroundColor)) {
			backgroundColor = Ext.ux.util.Color.hex2rgba(backgroundColor, opacity);
			backgroundColor = 'rgba(' + backgroundColor.join(',') + ')';
		}

		backgroundElement.setStyle({
			backgroundColor: backgroundColor,
			backgroundImage: 'none'
		});
	},

	setBackground: function (region) {
		var backgroundElement,
			background = Sb.Editor.model.design.getDecoration('background', region + 'Background');

		if (Sb.Utils.getBackgroundType(background).value === 'none') {
			return;
		}

		if (region == 'site') {
			backgroundElement = Ext.getBody();
		} else if (region == 'watermark') {
			backgroundElement = this.watermarkBackgroundEl;
		} else {
			backgroundElement = this.layoutEl.child('#layout-' + region);
		}

		Sb.Editor.siteView.applyContainerBackground(backgroundElement, region, false);
	},

	/**
	 * Apply new styles for external-border
	 */
	setStylesForBorder: function () {
		var data = Sb.Editor.model.design.data.layout,
			name = this.getCurrentExternalBorderName(),
			path = this.getExternalBorderPath(name),
			info = this.getExternalBorderInfo(name),
			width = info.width,
			height = info.height,
			watermark = Ext.get('watermark'),
			layout = Ext.get('layout'),
			borderMargin = '0 ' + width + 'px',
			getBackground = function(imageName) {
				return {'background-image': 'url("' + path +  '/images/' + name + '-' + imageName + '.png")'};
			},
			getPadding = function(top, right, bottom, left) {
				return {padding: top + 'px ' + right + 'px ' + bottom + 'px ' + left + 'px'};
			},
			headerPadding, contentPadding, footerPadding;

		headerPadding = (data.headerLayout == 'watermark')
			? getPadding(height, width, 0, width)
			: getPadding(height, 0, 0, 0);
		contentPadding = (data.contentLayout == 'watermark')
			? getPadding(0, width, 0, width)
			: getPadding(0, 0, 0, 0);
		footerPadding = (data.footerLayout == 'watermark')
			? getPadding(0, width, 0, width)
			: getPadding(0, 0, 0, 0);
		layout.first().setStyle(headerPadding)
			.next().setStyle(contentPadding)
			.next().setStyle(footerPadding);

		watermark.child('.external-top').setHeight(height)
			.first().setStyle(getBackground('top-left')).setWidth(width)
			.next().setStyle(getBackground('top-right')).setWidth(width)
			.next().setStyle(getBackground('top')).setHeight(height).setStyle('margin', borderMargin)
				.first().setStyle(getBackground('top-left2'))
					.first().setStyle(getBackground('top-right2'));

		watermark.child('.external-bottom').setHeight(height)
			.first().setStyle(getBackground('bottom-left')).setWidth(width)
			.next().setStyle(getBackground('bottom-right')).setWidth(width)
			.next().setStyle(getBackground('bottom')).setHeight(height).setStyle('margin', borderMargin)
				.first().setStyle(getBackground('bottom-left2'))
					.first().setStyle(getBackground('bottom-right2'));

		watermark.child('.external-left').setStyle(getBackground('left')).setWidth(width)
			.first().setStyle(getBackground('left-top'))
				.first().setStyle(getBackground('left-bottom'));
		watermark.child('.external-right').setStyle(getBackground('right')).setWidth(width)
			.first().setStyle(getBackground('right-top'))
				.first().setStyle(getBackground('right-bottom'));

		watermark.child('.watermark-content').setStyle('margin', borderMargin);

		Ext.each(['header', 'content', 'footer'], function(containerName) {
			var backgroundPadding = 0,
				containerWidth = '',
				container;

			if (containerName == 'content') {
				container = this.columnsEl;
			} else {
				container = this.items.find(function (containerVertical) {
					return containerVertical.model.get('region') == containerName;
				}).el.child('.container-content');
			}

			if (data[containerName + 'Layout'] == 'elastic-background') {
				backgroundPadding = width / 2 + 'px';
				containerWidth = this.getCssWidth(data.width);
			}

			container.setStyle({
				paddingLeft	: backgroundPadding,
				paddingRight: backgroundPadding,
				width		: containerWidth
			});
		}, this);
	},

	/**
	 * Get current external border name
	 *
	 * @return {String}
	 */
	getCurrentExternalBorderName: function() {
		return Sb.Editor.model.design.getDecoration('external-border', 'watermarkExternalBorder');
	},

	/**
	 * Get external border meta info by name
	 *
	 * @return {Object}
	 */
	getExternalBorderInfo: function(name) {
		return Sb.Editor.model.data.template.decorations.externalBorders[name];
	},

	/**
	 * Get external border path by name
	 *
	 * @return {String}
	 */
	getExternalBorderPath: function (name) {
		return Sb.Config.templatesUrl + '/' +
			encodeURIComponent(Sb.Editor.model.data.template.code) +
			'/decorations/external-border/' + encodeURIComponent(name);
	},

	initPromoFooter: function () {
		if (!Sb.Editor.model.get('promoFooter')) {
			return;
		}
		this.promoFooter = new Sb.editor.component.Base({
			cls: 'sb-promofooter',
			renderTo: this.layoutEl,
			html: '<div class="promofooter">' + Sb.Editor.model.get('promoFooter') + '</div>',
			hasEditButton: false,
			hasEditPanel: false,
			movable: false,
			removable: true,
			isShowEditPanel: false,
			removeConfirmation: false,
			getStateCls: function (state) {
				return this.baseCls + '-shared-' + state;
			},
			onContentElementClick: this.showPromoFooterDialog,
			listeners: {
				remove: this.showPromoFooterDialog,
				afterrender: function () {
					this.contentElement.mask();
				}
			}
		});
	},

	showPromoFooterDialog: function () {
		Sb.Editor.Ui.mainWindow.showTBBMessage(
			Sb.Locale.get('disablePromoFooterUpsellTitle'),
			Sb.Locale.get('disablePromoFooterUpsellMsg')
		);
	},

	doResize: function () {
		this.items.each(function(item){
			item.doResize();
		}, this);
	}
});