/**
 * @class Sb.widget.Header
 * @extends Sb.widget.Widget
 * Class for header widget
 * @constructor
 * @param {Object} config The config object
 * @xtype header_widget
 */
Ext.define('Sb.widget.Header', {
	extend: 'Sb.widget.Widget',
	xtype: 'header_widget',

	cls: 'sb-widget-header',

	/**
	 * How many pixels of inner component's will be always visible in X axis
	 * @type Number
	 * @property componentVisibleOffsetX
	 */
	componentVisibleOffsetX: 16,

	/**
	 * How many pixels of inner component's will be always visible in Y axis
	 * @type Number
	 * @property componentVisibleOffsetY
	 */
	componentVisibleOffsetY: 16,

	initComponent: function () {
		this.addEvents('headerloaded');
		this.callParent(arguments);
	},

	initComponentDD: function (componentName) {
		this[componentName].dd.startDrag = function () {
			Sb.Editor.siteView.maskNavigations();
			var offsetX = this.componentVisibleOffsetX - this[componentName].el.getWidth();
			var offsetY = this.componentVisibleOffsetY - this[componentName].el.getHeight();
			this[componentName].dd.constrainTo(this.widgetContentEl, {
				left: offsetX,
				right: offsetX,
				top: offsetY,
				bottom: offsetY
			});
			if (Sb.Editor.Ui.editComponent) {
				Sb.Editor.Ui.editComponent.completeEdit.call(Sb.Editor.Ui.editComponent);
				if (!this[componentName].controlsEl.isVisible()) {
					this.disableClipping();
					Sb.Editor.Ui.overComponent = this[componentName];
					this[componentName].showControls();
				}
			}
			if (Sb.Editor.model.isRtl()) {
				this[componentName].el.setRight('auto');
			}
		}.createDelegate(this);
		this[componentName].dd.endDrag = function () {
			var properties = {},
				el = this[componentName].getEl();
			properties[componentName + 'PositionTop'] = el.getTop(true);
			if (Sb.Editor.model.isRtl()) {
				properties[componentName + 'PositionLeft'] = this.widgetContentEl.getWidth() -
					el.getRight(true) + el.getBorderWidth('lr');
			} else {
				properties[componentName + 'PositionLeft'] = el.getLeft(true);
			}
			this.model.unserialize(properties);
			this.model.saveIfChanged(this.memento);
			Sb.Editor.siteView.unmaskNavigations();
		}.createDelegate(this);
	},

	onRender: function () {
		this.callParent(arguments);

		this.createLogo();
		this.createTitle();
		this.createSubtitle();
		this.createText();

		this.refreshLogo();
		this.onTitleChange();
		this.onSubtitleChange();
	},

	show: function () {
		this.callParent();
		this.onImageChange();
	},

	createLogo: function () {
		this.logo = new Sb.editor.component.Image({
			cls: 'header-logo',
			renderTo: this.widgetContentEl,
			imagePath: this.model.getResourcesUrl() + '/' + Sb.Config.path.attachmentsFolder + '/Logo/',
			imageSize: {
				width: this.model.get('logoWidth'),
				height: this.model.get('logoHeight')
			},
			isProportional: this.model.get('logoProportional'),
			editPanelConfig: {
				uploadAction: Sb.Config.baseUrl + '/Upload/Logo/',
				validExtensions: Sb.Config.headerValidExtensions
			}
		});
		this.logo.on('remove', this.onLogoRemove, this);
		this.logo.on('startover', this.disableClipping, this);
		this.logo.on('removeover', this.enableClipping, this);
		this.logo.on('startedit', this.disableClipping, this);
		this.logo.on('completeedit', this.onLogoCompleteEdit, this);
		this.initComponentDD('logo');
	},

	createTitle: function () {
		this.title = new Sb.editor.component.Text({
			cls: 'header-title',
			emptyText: Sb.Locale.get('titleEmptyMessage'),
			renderTo: this.widgetContentEl
		});
		this.title.on('remove', this.onTitleRemove, this);
		this.title.on('startover', this.disableClipping, this);
		this.title.on('removeover', this.enableClipping, this);
		this.title.on('startedit', this.disableClipping, this);
		this.title.on('completeedit', this.onTitleCompleteEdit, this);
		this.initComponentDD('title');
	},

	createSubtitle: function () {
		this.subtitle = new Sb.editor.component.Text({
			cls: 'header-subtitle',
			emptyText: Sb.Locale.get('subtitleEmptyMessage'),
			renderTo: this.widgetContentEl
		});
		this.subtitle.on('remove', this.onSubtitleRemove, this);
		this.subtitle.on('startover', this.disableClipping, this);
		this.subtitle.on('removeover', this.enableClipping, this);
		this.subtitle.on('startedit', this.disableClipping, this);
		this.subtitle.on('completeedit', this.onSubtitleCompleteEdit, this);
		this.initComponentDD('subtitle');
	},

	createText: function () {
		var me = this;
		this.text = new Sb.widget.Text({
			cls: 'sb-widget-text header-text',
			hasVisibilityControl: false,
			model: this.model.widgets.first(),
			removeConfirmation: false,
			removeTitle: Sb.Locale.get('hideWidgetTitle'),
			removeComponentHandler: function () {
				Sb.widget.Widget.superclass.removeComponentHandler.call(this);
			},
			refreshContent: function () {
				if (this.isDestroyed) {
					return;
				}

				Sb.widget.Text.superclass.refreshContent.call(this);
				me.onTextChange();
			},
			initDD: function () {
				this.dd = new Ext.dd.DD(this);
				this.dd.setOuterHandleElId(this.ddEl);
				this.dd.component = this;
			}
		});
		this.text.render(this.widgetContentEl);
		this.text.on('remove', this.onTextRemove, this);
		this.text.on('startover', this.disableClipping, this);
		this.text.on('removeover', this.enableClipping, this);
		this.text.on('startedit', this.onTextStartEdit, this);
		this.text.on('completeedit', this.onTextCompleteEdit, this);
		this.text.on('canceledit', this.onTextCancelEdit, this);
		this.initComponentDD('text');
	},

	initEvents: function () {
		this.callParent();

		this.mon(this.model, 'heightchange', this.onHeightChange, this);
		this.mon(this.model, 'change', this.onChange, this);
		this.mon(this.model, 'imagefitchange', this.onImageFitChange, this);
		this.mon(this.model, 'logochange', this.refreshLogo, this);
		this.mon(this.model, 'titlechange', this.onTitleChange, this);
		this.mon(this.model, 'subtitlechange', this.onSubtitleChange, this);
		this.mon(this.model, 'textchange', this.onTextChange, this);
		this.mon(Sb.Editor.model, 'propertychange', function (property) {
			if (property == 'logoPath') {
				var logoPath = Sb.Editor.model.get('logoPath'),
					size;

				this.logo.updateSize({width: '', height: ''});
				this.logo.updateImage(this.model.getResourcesUrl() + '/' + logoPath);

				size = this.logo.getImageSize();

				this.model.unserialize({
					logoWidth: size.width,
					logoHeight: size.height
				});
				this.model.save();
			}
		}, this);
		this.mon(Sb.Editor.model.design, {
			scope: this,
			colorschemechange: function () {
				this.onTitleChange();
				this.onSubtitleChange();
			}
		});
	},

	getConstrainLeftTop: function (name) {
		var size = this[name].el.getSize(),
			pos = [this.model.get(name + 'PositionLeft'), this.model.get(name + 'PositionTop')];

		if (pos[0] + size.width < this.componentVisibleOffsetX) {
			pos[0] = this.componentVisibleOffsetX - size.width;
		}

		if (pos[1] + size.height < this.componentVisibleOffsetY) {
			pos[1] = this.componentVisibleOffsetY - size.height;
		}
		return pos;
	},

	onChange: function (model, properties) {
		if (properties.indexOf('imageName') > -1 || properties.indexOf('imagePath') > -1) {
			this.onImageChange();
		}
	},

	onImageFitChange: function () {
		var backgroundPosition = '',
			backgroundSize = '';

		if (this.model.get('imageFit')) {
			backgroundPosition = 'top left';
			backgroundSize = 'initial';
		}

		this.backgroundImageEl.setStyle({
			backgroundSize: backgroundSize,
			backgroundPosition: backgroundPosition
		});
	},

	onHeightChange: function () {
		var height = this.model.get('height'),
			isNeedReachor = Sb.Editor.Ui.editComponent == this;

		this.backgroundImageEl.setHeight(height);

		this.alignVerticalContent('logo');
		this.alignVerticalContent('title');
		this.alignVerticalContent('subtitle');

		if (isNeedReachor) {
			this.getEditPanel().reanchor();
		}

		this.setImageSmartPosition();
	},

	/**
	 * System banners "smart-resize" algorithm
	 * original logic from asachkov prototype is preserved
	 */
	setImageSmartPosition: function () {
		var data = this.model.data,
			finalArea = {},
			backgroundPositionX,
			backgroundPositionY,
			widgetSize,
			widgetAspectRatio,
			aspectRatioDiff,
			xScale,
			yScale,
			backgroundWidth,
			backgroundHeight,
			maxAspectRatio = 1000,
			maxAspectRatioAreaKey = null,
			minAspectRatio = -1000,
			minAspectRatioAreaKey = null,
			delta,
			aspectRatioFactor,
			minArea,
			maxArea;

		if (!data.imageName || !data.areas || data.areas.length === 0) {
			return;
		}

		widgetSize = this.el.getSize();

		var layoutData = Sb.Editor.model.design.data.layout,
			verticalContainer = this.model;
		while (verticalContainer && verticalContainer.get('type') != 'container_vertical') {
			verticalContainer = verticalContainer.ownerWidget;
		}
		var region = verticalContainer.get('region');
		if ((region == 'column1' && !layoutData.sidebarFirst) || (region == 'column2' && !layoutData.sidebarSecond)) {
			return;
		}

		widgetAspectRatio = widgetSize.width / widgetSize.height;

		Ext.each(data.areas, function (area, key) {
			Ext.iterate(area, function (key, value) {
				area[key] = parseInt(value, 10);
			});

			area.aspectRatio = (area.x2 - area.x1) / (area.y2 - area.y1);
			aspectRatioDiff = widgetAspectRatio - area.aspectRatio;

			if (aspectRatioDiff < maxAspectRatio && aspectRatioDiff > 0) {
				maxAspectRatio = aspectRatioDiff;
				maxAspectRatioAreaKey = key;
			}

			if (aspectRatioDiff > minAspectRatio && aspectRatioDiff < 0) {
				minAspectRatio = aspectRatioDiff;
				minAspectRatioAreaKey = key;
			}
		});

		if (maxAspectRatioAreaKey === null) {
			finalArea = data.areas[minAspectRatioAreaKey];

			delta = Math.round((finalArea.x2 - finalArea.x1) - ((finalArea.y2 - finalArea.y1) * widgetAspectRatio)) / 2;
			finalArea.x1 += delta;
			finalArea.x2 -= delta;
		} else if (minAspectRatioAreaKey === null) {
			finalArea = data.areas[maxAspectRatioAreaKey];

			delta = Math.round((finalArea.y2 - finalArea.y1) - ((finalArea.x2 - finalArea.x1) / widgetAspectRatio)) / 2;
			finalArea.y1 += delta;
			finalArea.y2 -= delta;
		} else {
			minArea = data.areas[minAspectRatioAreaKey];
			maxArea = data.areas[maxAspectRatioAreaKey];
			aspectRatioFactor = (widgetAspectRatio - maxArea.aspectRatio) / (minArea.aspectRatio - maxArea.aspectRatio);

			finalArea.x1 = maxArea.x1 + Math.round(aspectRatioFactor * (minArea.x1 - maxArea.x1));
			finalArea.x2 = maxArea.x2 + Math.round(aspectRatioFactor * (minArea.x2 - maxArea.x2));
			finalArea.y1 = maxArea.y1 + Math.round(aspectRatioFactor * (minArea.y1 - maxArea.y1));
			finalArea.y2 = finalArea.y1 + Math.round((finalArea.x2 - finalArea.x1) / widgetAspectRatio);
		}

		xScale = (finalArea.x2 - finalArea.x1) / widgetSize.width;
		yScale = (finalArea.y2 - finalArea.y1) / widgetSize.height;

		backgroundWidth = Math.round(data.imageSize[0] / xScale);
		backgroundHeight = Math.round(data.imageSize[1] / yScale);

		backgroundPositionX = Math.round(finalArea.x1 / xScale);
		backgroundPositionY = Math.round(finalArea.y1 / yScale);

		this.backgroundImageEl.setStyle({
			backgroundSize: [backgroundWidth + 'px', backgroundHeight + 'px'].join(' '),
			backgroundPosition: [-backgroundPositionX + 'px', -backgroundPositionY + 'px'].join(' ')
		});
	},

	doResize: function () {
		if (!this.isVisible()) {
			return;
		}

		this.alignHorizontalContent('logo');
		this.alignHorizontalContent('title');
		this.alignHorizontalContent('subtitle');
		this.setImageSmartPosition();
		this.model.saveIfChanged(this.memento);
	},

	/**
	 * Vertical align components like title when header resizes
	 *
	 * @param {String} componentName For example 'title'
	 **/
	alignVerticalContent: function (componentName) {
		var data			= this.model.data,
			headerHeight	= this.widgetContentEl.getHeight(),
			dataKey			= componentName + 'PositionTop',
			oldTop			= parseInt(data[dataKey], 10),
			bottom			= this.componentVisibleOffsetY + oldTop,
			isNeedAlign		= (bottom > headerHeight) && this.isVisible(),
			newTop			= Math.max(headerHeight - this.componentVisibleOffsetY, 0);

		if (isNeedAlign) {
			this.model.set(dataKey, newTop);
		}
	},

	/**
	 * Horizontal align components like title when header resizes
	 *
	 * @param {String} componentName For example 'title'
	 **/
	alignHorizontalContent: function (componentName) {
		var headerWidth = this.widgetContentEl.getWidth(),
			oldLeft = parseInt(this.model.data[componentName + 'PositionLeft'], 10),
			right = this.componentVisibleOffsetX + oldLeft,
			newData = {},
			newLeft;

		if (right > headerWidth) {
			newLeft = headerWidth - this.componentVisibleOffsetX;
			newLeft = (newLeft < 0) ? 0 : newLeft;
			newData[componentName + 'PositionLeft'] = newLeft;
			this.model.unserialize(newData);
		}
	},

	onImageChange: function () {
		// destroy image and create new for change image (because of 'onload' bug in ie)
		if (this.backgroundImageEl) {
			Ext.destroy(this.backgroundImageEl);
		}
		if (this.imageEl) {
			this.imageEl.remove();
		}

		if (Sb.utils.Flash.isFlash(this.model.get('imagePath'))) {
			this.updateFlash();
		} else {
			this.updateImage();
		}
	},

	updateImage: function () {
		var data = this.model.data,
			path, headersStore, format;

		if (!data.imagePath) {
			headersStore = Ext.StoreMgr.get('headers');
			format = headersStore.getAt(headersStore.find('name', data.imageName)).get('format');
			path = Sb.Config.headersUrl + '/' + data.imageName + '/header.' + format;
		} else {
			path = this.model.getResourcesUrl() + '/' + data.imagePath;

			//add timestamp for custom headers to avoid caching
			if (data.imageTime) {
				path += '?' + data.imageTime;
			}
		}

		this.imageEl = this.widgetContentEl.createChild({
			tag: 'img',
			cls: 'header-fake-image'
		});

		this.backgroundImageEl = this.widgetContentEl.createChild({
			tag: 'div',
			cls: 'header-image'
		});
		this.backgroundImageEl.setStyle({
			backgroundImage: 'url(' + path + ')',
			borderRadius: this.model.get('styles').borderRadius
		});
		this.updateShadow();

		this.imageEl.on('load', this.onImageLoad, this, {single: true});
		this.imageEl.set({src: path});
	},

	updateFlash: function () {
		var data = this.model.data,
			path = this.model.getResourcesUrl() + '/' + data.imagePath;

		//add timestamp for custom headers to avoid caching
		if (data.imageTime) {
			path += '?' + data.imageTime;
		}

		this.backgroundImageEl = this.widgetContentEl.createChild({
			tag: 'div',
			html: Sb.utils.Flash.createCode(path, this.model.get('flashWidth'), this.model.get('flashHeight'))
		});
		this.fireEvent('headerloaded', this);
	},

	onImageLoad: function () {
		if (!this.model.get('height')) {
			if (this.model.get('imageFit')) {
				this.model.set('height', this.imageEl.getHeight());
			} else {
				this.model.set(
					'height',
					Math.round(this.el.getWidth() * this.imageEl.getHeight() / this.imageEl.getWidth())
				);
			}
			if (Sb.Editor.model.design.presetType == 'generic') {
				this.model.save();
			}
		}

		this.onImageFitChange();
		this.onHeightChange();
		this.fireEvent('headerloaded', this);
	},

	refreshLogo: function () {
		this.logo.setVisible(1 != this.model.get('logoIsHidden'));
		if (this.logo.isVisible()) {
			this.logo.updateImage(this.model.getResourcesUrl() + '/' + Sb.Editor.model.get('logoPath'));
			this.logo.updateSize({
				width: this.model.get('logoWidth') || '',
				height: this.model.get('logoHeight') || ''
			});
			this.logo.setProportional(this.model.get('logoProportional'));
			this.onComponentPositionChange('logo');
		}
	},

	onLogoRemove: function () {
		this.model.unserialize({logoIsHidden: true});
		this.model.save();
	},

	onLogoCompleteEdit: function (logo) {
		this.enableClipping();
		var logoPath = logo.imageUrl.replace(new RegExp('^' + this.model.getResourcesUrl() + '/' + '(.*)'), '$1');

		if (logoPath != Sb.Editor.model.get('logoPath')) {
			Sb.Editor.model.set('logoPath', logoPath);
			Sb.Editor.model.save();
		}

		var memento = this.model.saveToMemento(),
			position = this.getConstrainLeftTop('logo'),
			imageSize = logo.getImageSize();

		this.model.unserialize({
			logoWidth: imageSize.width,
			logoHeight: imageSize.height,
			logoProportional: logo.isProportional,
			logoPositionLeft: position[0],
			logoPositionTop: position[1]
		});
		this.model.saveIfChanged(memento);
	},

	onTitleChange: function () {
		var data		= this.model.data,
			isVisible	= 1 != data.titleIsHidden;

		this.title.setData({
			content			: data.title,
			fontColor		: data.titleFontColor,
			fontBold		: data.titleFontBold,
			fontSize		: data.titleFontSize,
			fontItalic		: data.titleFontItalic,
			fontFamily		: data.titleFontFamily,
			fontUnderline	: data.titleFontUnderline,
			stroke			: data.titleStroke
		});
		this.title.setVisible(isVisible);
		this.onComponentPositionChange('title');
	},

	onTitleRemove: function () {
		this.model.unserialize({titleIsHidden: true});
		this.model.save();
	},

	onTitleCompleteEdit: function (title) {
		this.enableClipping();
		var position = this.getConstrainLeftTop('title');
		title.data.PositionLeft = position[0];
		title.data.PositionTop = position[1];
		this.onTextComponentChange('title', title.data);
	},

	onSubtitleChange: function () {
		var data = this.model.data;
		this.subtitle.setData({
			content: data.subtitle,
			fontColor: data.subtitleFontColor,
			fontBold: data.subtitleFontBold,
			fontSize: data.subtitleFontSize,
			fontItalic: data.subtitleFontItalic,
			fontFamily: data.subtitleFontFamily,
			fontUnderline: data.subtitleFontUnderline,
			stroke: data.subtitleStroke
		});
		this.subtitle.setVisible(data.subtitleIsHidden != 1);
		this.onComponentPositionChange('subtitle');
	},

	onSubtitleRemove: function () {
		this.model.unserialize({subtitleIsHidden: true});
		this.model.save();
	},

	onSubtitleCompleteEdit: function (subtitle) {
		this.enableClipping();
		var position = this.getConstrainLeftTop('subtitle');
		subtitle.data.PositionLeft = position[0];
		subtitle.data.PositionTop = position[1];
		this.onTextComponentChange('subtitle', subtitle.data);
	},

	onTextChange: function () {
		var isVisible = !this.model.get('textIsHidden');
		this.text.setVisible(isVisible);
		this.text.widgetEl.setStyle({
			width: Math.max(this.model.get('textWidth'), 50) + 'px',
			height: Math.max(this.model.get('textHeight'), 30) + 'px'
		});
		this.onComponentPositionChange('text');
	},

	onTextStartEdit: function () {
		this.disableClipping();

		this.textMemento = {
			textWidth: this.model.get('textWidth'),
			textHeight: this.model.get('textHeight'),
			textPositionTop: this.model.get('textPositionTop'),
			textPositionLeft: this.model.get('textPositionLeft')
		};

		this.text.resizer = new Ext.Resizable(this.text.getEl(), {
			header: this,
			widget: this.text,
			handles: 'all',
			minHeight: 30,
			minWidth: 50,
			maxHeight: 1000,
			maxWidth: 1000,
			pinned: true,
			dynamic: true,
			resizeElement: function () {
				var box = this.proxy.getBox(),
					newData = {},
					model = this.header.model,
					width = model.get('textWidth'),
					height = model.get('textHeight'),
					top = model.get('textPositionTop'),
					left = model.get('textPositionLeft'),
					horizontalPadding = this.widget.widgetEl.getPadding('lr'),
					verticalPadding = this.widget.widgetEl.getPadding('tb'),
					deltaX = box.width - width - horizontalPadding,
					deltaY = box.height - height - verticalPadding;

				if (this.activeHandle) {
					if (['west', 'southwest', 'northwest'].indexOf(this.activeHandle.position) > -1) {
						newData.textPositionLeft = left - deltaX;
					}
					if (['north', 'northeast', 'northwest'].indexOf(this.activeHandle.position) > -1) {
						newData.textPositionTop = top - deltaY;
					}
				}

				newData.textWidth = width + deltaX;
				newData.textHeight = height + deltaY;

				model.set(newData);

				return box;
			}
		});
	},

	onTextCompleteEdit: function () {
		this.enableClipping();
		this.text.removeClass('x-resizable-pinned');
		this.text.resizer.destroy();
		this.model.saveIfChanged(this.memento);
	},

	onTextCancelEdit: function () {
		this.model.set(this.textMemento);
		this.model.save();
	},

	onTextRemove: function () {
		this.model.unserialize({textIsHidden: true});
		this.model.save();
	},

	onComponentPositionChange: function (name) {
		this[name].el.setTop(this.model.get(name + 'PositionTop'));
		if (Sb.Editor.model.isRtl()) {
			this[name].el.setLeft('auto');
			this[name].el.setRight(this.model.get(name + 'PositionLeft'));
		} else {
			this[name].el.setLeft(this.model.get(name + 'PositionLeft'));
		}
	},

	onCancelEdit: function () {
		if (this.memento) {
			this.model.restoreFromMemento(this.memento);
			this.model.saveIfChanged(this.memento);
		}
	},

	onDestroy: function () {
		if (this.rendered) {
			this.logo.destroy();
			this.title.destroy();
			this.subtitle.destroy();
			this.text.destroy();
			if (this.imageEl) {
				this.imageEl.remove();
			}
		}
		this.callParent();
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.sb-color-dropdown')
			|| event.getTarget('.sb-widget-edit-panel-header')
			|| event.getTarget('.x-combo-list')
		) {
			return;
		}

		this.callParent(arguments);
	},

	onTextComponentChange: function (componentName, data) {
		data = data || {};
		var modelData = {};
		Ext.iterate(data, function (property, value) {
			if (property == 'content') {
				modelData[componentName] = value;
			} else {
				modelData[componentName + property.toUpperCaseFirst()] = value;
			}
		}, this);
		this.model.unserialize(modelData);
		this[componentName].getEditPanel().reanchor();

		this.model.saveIfChanged(this.memento);
	},

	enableClipping: function () {
		this.widgetContentEl.setStyle({overflow: 'hidden'});
	},

	disableClipping: function () {
		this.widgetContentEl.setStyle({overflow: 'visible'});
	},

	onStylesChange: function (styles) {
		this.callParent(arguments);

		if (styles.padding && this.getStylePanel().rendered && this.backgroundImageEl) {
			this.onHeightChange();
		}

		if (styles.borderRadius && this.backgroundImageEl) {
			this.backgroundImageEl.setStyle('borderRadius', styles.borderRadius);
		}

		this.updateShadow();
	},

	updateShadow: function () {
		var modelShadow = this.model.get('styles').boxShadow;

		if (!this.backgroundImageEl || !this.widgetEl) {
			return;
		}

		if (Sb.Utils.isInnerShadow(modelShadow)) {
			this.backgroundImageEl.setStyle('boxShadow', modelShadow);
			this.widgetEl.setStyle('boxShadow', '');
		} else {
			this.backgroundImageEl.setStyle('boxShadow', '');
		}
	}
});