/**
 * @class Sb.widget.Map
 * @extends Sb.widget.Widget
 * Class for map widget
 * @xtype map_widget
 */
Ext.define('Sb.widget.Map', {
	extend: 'Sb.widget.Widget',
	xtype: 'map_widget',
	translatable: true,

	cls: 'sb-widget-map',
	markers: [],

	onRender: function () {
		Sb.widget.Map.superclass.onRender.apply(this, arguments);

		Sb.GoogleJsApi.loadMaps(this.refresh, this);
		this.contentElement.mask();
		this.contentElement.removeClass('x-masked');
	},

	initEvents: function () {
		Sb.widget.Map.superclass.initEvents.call(this);

		this.mon(this.model, 'change', function (model, properties) {
			Ext.each(properties, function (property) {
				if (['place', 'height', 'markers'].indexOf(property) > -1) {
					this['on' + property.toUpperCaseFirst() + 'Change'](this.model.get(property));
					return false;
				}
			}, this);
		}, this);

		this.model.on('save', function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(
				Sb.Locale.get('messageChangesSaved')
			);
		});
	},

	onStartEdit: function () {
		Sb.widget.Map.superclass.onStartEdit.call(this);
		this.contentElement.unmask();
	},

	onCompleteEdit: function () {
		this.model.saveIfChanged(this.memento);

		this.markerWindow.close();
		this.contentElement.mask();
		this.contentElement.removeClass('x-masked');
	},

	onCancelEdit: function () {
		this.markerWindow.close();
		Sb.widget.Map.superclass.onCancelEdit.call(this);
		this.refresh();
		this.contentElement.mask();
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.x-menu-item-text')) {
			return;
		}
		Sb.widget.Map.superclass.onDocumentClick.call(this, event);
	},

	onContentElementClick: function (event) {
		event.stopPropagation();
		if (this.afterDD) {
			this.afterDD = false;
			return;
		}
		this.startEdit();
	},

	refresh: function () {
		this.initMap();
		this.initMapEvents();
	},

	initMap: function () {
		var place = this.model.get('place') || this.getDefaultLocation(),
			apiKey = this.model.get('apiKey') || '',
			body = Ext.getBody(),
			controls;


		this.markerMode = true;
		this.dragModeButton = body.createChild({
			cls: 'context-button drag-mode-button'
		});
		this.markerModeButton = body.createChild({
			cls: 'context-button marker-mode-button'
		});
		this.markerModeButton.addListener('click', this.onModeChange, this);
		this.dragModeButton.addListener('click', this.onModeChange, this);
		if (Ext.isIE) {
			this.dragModeButton.set({
				unselectable: "on"
			});
			this.markerModeButton.set({
				unselectable: "on"
			});
		} else {
			this.dragModeButton.unselectable();
			this.markerModeButton.unselectable();
		}

		if (this.mapEl) {
			this.mapEl.remove();
		}
		this.mapEl = this.widgetContentEl.createChild({
			id: 'map-' + this.model.data.uuid,
			style: {
				height: this.model.get('height') + 'px'
			}
		});

		this.map = new google.maps.Map(this.mapEl.dom, {
			mapTypeId: this.getMapTypeByMode(this.model.get('mode')),
			zoom: this.model.get('zoom'),
			streetViewControl: false,
			overviewMapControl: true,
			scrollwheel: true,
			scaleControl: true
		});

		this.onPlaceChange(place);
		this.onApiKeyChange(apiKey);

		if (!this.infoPanelEl) {
			this.infoPanelEl = body.createChild();
			this.markerWindow = new google.maps.InfoWindow({
				content: this.infoPanelEl.dom
			});
		}

		this.markers = this.model.get('markers').map(this.drawMarker, this);

		controls = this.map.controls[google.maps.ControlPosition.TOP_LEFT];
		controls.push(this.dragModeButton.dom);
		controls.push(this.markerModeButton.dom);
		this.onModeChange();
	},

	onMarkersChange: function () {
		this.refresh();
	},

	onCurrentLocaleChange: function (newLocale, oldLocale) {
		if (this.infoWindowPanel) {
			this.infoWindowPanel.ok.handler.call(this.infoWindowPanel.ok.scope, this.infoWindowPanel.ok, Ext.EventObject);
		}
		this.callParent([newLocale, oldLocale]);
	},

	initMapEvents: function () {
		var map = this.map,
			widget = this;
		google.maps.event.addListener(this.map, 'click', function(event) {
			if (!widget.markerMode) {
				return;
			}
			widget.onAddMarker({
				lat: event.latLng.lat(),
				lng: event.latLng.lng()
			});
		});

		google.maps.event.addListener(this.map, 'zoom_changed', function() {
			widget.model.set('zoom', map.getZoom());
		});

		google.maps.event.addListener(this.map, 'maptypeid_changed', function() {
			widget.model.set('mode', map.getMapTypeId());
		});

		google.maps.event.addListener(this.map, 'dragend', function() {
			var latlng = map.getCenter();
			widget.model.set('place', latlng.lat() + ',' + latlng.lng());

		});

		google.maps.event.addListener(this.map, 'rightclick', function(event) {
			var widgetXY = widget.mapEl.getXY(),
				lat = event.latLng.lat(),
				lng = event.latLng.lng();

				if (widget.markerMode) {
					widget.onAddMarker({
						lat: lat,
						lng: lng
					});
					return;
				}

				if (widget.mapMenu) {
					widget.mapMenu.destroy();
				}
				widget.mapMenu = new Ext.menu.Menu({
					cls: 'sb-widget-map-context-menu',
					items: [{
						text: Sb.Locale.get('mapMenuAddMarker'),
						handler: function () {
							widget.onAddMarker({
								lat: lat,
								lng: lng
							});
						}
					}, {
						text: Sb.Locale.get('mapMenuCenter'),
						handler: function () {
							widget.model.set('place', lat + ',' + lng);
						}
					}, {
						text: Sb.Locale.get('mapRemoveAllMarkers'),
						handler: function () {
							Ext.Msg.show({
								width: 400,
								closable : false,
								title: Sb.Locale.get('confirmRemoval'),
								msg: Sb.Locale.get('mapMarkerWindowRemoveConfirmAll'),
								buttons: Ext.Msg.YESNO,
								fn: function (btn) {
									if (btn == 'yes') {
										while (widget.markers.length > 0) {
											widget.onRemoveMarker(widget.markers[0]);
										}
									}
								}
							});
						}
					}]
				});
				widget.mapMenu.showAt([event.pixel.x + widgetXY[0], event.pixel.y + widgetXY[1]]);
		});
	},

	drawMarker: function (model) {
		var shadowXShift = 17,
			shadowYShift = 32,
			marker = new google.maps.Marker({
				map: this.map,
				position: new google.maps.LatLng(model.lat, model.lng),
				title: model.title,
				model: model,
				draggable: true,
				icon: Sb.Config.baseUrl + '/widgets/map/skins/default/images/marker-blue.png',
				shadow: {
					anchor: new google.maps.Point(shadowXShift, shadowYShift),
					url: Sb.Config.baseUrl + '/widgets/map/skins/default/images/shadow.png'
				}
		});

		google.maps.event.addListener(marker, 'click', Ext.createDelegate(function() {
			this.onMarkerEdit(marker);
		}, this));

		google.maps.event.addListener(marker, 'position_changed', function() {
			var position = marker.getPosition();
			marker.model.lat = position.lat();
			marker.model.lng = position.lng();
		});

		return marker;
	},

	onAddMarker: function (data) {
		var title = Sb.Locale.get('mapMarkerGenericTitle', this.markers.length + 1),
			description = '',
			marker,
			markerModel = {};

		new google.maps.Geocoder().geocode({
				latLng: new google.maps.LatLng(data.lat, data.lng)
			}, Ext.createDelegate(function(results, status) {
				if (status == 'OK' && results[0] && results[0].formatted_address) {
					description = results[0].formatted_address;
				}

				markerModel = {
					lat: data.lat,
					lng: data.lng,
					title: title,
					description: description,
					uuid: null
				};
				this.model.get('markers').push(markerModel);
				marker = this.drawMarker(markerModel);
				this.markers.push(marker);
				this.onMarkerEdit(marker);
			}, this)
		);
		this.onModeChange();
	},

	onRemoveMarker: function (marker) {
		marker.setMap(null);
		this.markers.remove(marker);
		this.model.get('markers').remove(marker.model);
	},

	onMarkerEdit: function (marker) {
		if (!this.infoWindowPanel) {
			this.createInfoWindowPanel(marker);
		} else {
			this.updateInfoWindowPanel(this.infoWindowPanel, marker);
		}
	},

	onModeChange: function () {
		var newModeButton, oldModeButton, draggable;
		this.markerMode = !this.markerMode;
		if (this.markerMode) {
			newModeButton = this.markerModeButton;
			oldModeButton = this.dragModeButton;
			draggable = false;
			this.addClass('map-marker-mode');
		} else {
			newModeButton = this.dragModeButton;
			oldModeButton = this.markerModeButton;
			draggable = true;
			this.removeClass('map-marker-mode');
		}
		newModeButton.addClass('active');
		oldModeButton.removeClass('active');
		this.map.set('draggable', draggable);
	},

	onPlaceChange: function (place) {
		var coordinates,
			fallbackLocation = new google.maps.LatLng(40.71, -74); // fallback location New York

		if (this.isPlaceInCoordinates(place)) {
			coordinates = place.split(',');
			this.map.panTo(new google.maps.LatLng(coordinates[0], coordinates[1]));
		} else if (place) {
			new google.maps.Geocoder().geocode({
					address: place
				}, Ext.createDelegate(function(results, status) {
					if (status == 'OK') {
						this.map.panTo(results[0].geometry.location);
					} else {
						this.map.panTo(fallbackLocation);
					}
				}, this)
			);
		} else {
			this.map.panTo(fallbackLocation);
		}
	},

	onApiKeyChange: function (apiKey) {
	},

	onHeightChange: function (height) {
		this.mapEl.setHeight(height);
		google.maps.event.trigger(this.map, 'resize');
	},

	/**
	 * Get map location by site owner credentials
	 * @returns {String} Full place address
	 */
	getDefaultLocation : function () {
		var locations = [],
			places = ['country', 'state', 'city', 'address'],
			ownerInfo = Sb.Editor.model.get('ownerInfo');

		Ext.each(places, function (place) {
			if (ownerInfo[place]) {
				locations.push(ownerInfo[place]);
			}
		});

		return locations.join(', ');
	},

	/**
	 * Convert map type from code to type id (for backward compatibility)
	 * @param {String} mode Google maps static url map type (one char)
	 * @returns {String} Google maps API map type id
	 */
	getMapTypeByMode : function (mode) {
		switch (mode) {
			case 'm': return google.maps.MapTypeId.ROADMAP;
			case 'k': return google.maps.MapTypeId.SATELLITE;
			case 'h': return google.maps.MapTypeId.HYBRID;
			case 'p': return google.maps.MapTypeId.TERRAIN;
			default: return mode;
		}
	},

	isPlaceInCoordinates: function (place) {
		return place.match(/^-?\d+\.\d+,-?\d+\.\d+$/);
	},

	updateInfoWindowPanel: function (panel, marker) {
		panel.marker = marker;
		panel.titleField.setValue(marker.model.title);
		panel.descriptionField.setValue(marker.model.description);
		this.markerWindow.close();
		this.markerWindow.setPosition(marker.getPosition());
		this.markerWindow.open(marker.map);
	},

	createInfoWindowPanel: function (marker) {
		var mapWidget = this;
		this.infoWindowPanel = new Ext.FormPanel({
			cls: 'info-window-panel',
			renderTo: this.infoPanelEl,
			labelAlign: 'top',
			border: false,
			marker: marker,
			defaults: {
				anchor: '100%'
			},
			listeners: {
				render: function (panel) {
					mapWidget.updateInfoWindowPanel(panel, marker);
				}
			},
			items: [{
				xtype: 'textfield',
				fieldLabel: Sb.Locale.get('mapMarkerWindowTitleFieldLabel'),
				ref: 'titleField'
			}, {
				xtype: 'textarea',
				fieldLabel: Sb.Locale.get('mapMarkerWindowDescriptionFieldLabel'),
				ref: 'descriptionField'
			}, {
				cls: 'info-window-button-panel',
				border: false,
				items: [{
					xtype: 'button',
					cls: 'info-window-button tool-left',
					text: Sb.Locale.get('mapMarkerWindowRemove'),
					handler: Ext.createDelegate(function (btn, e) {
						e.stopEvent();
						Ext.Msg.show({
							width: 400,
							closable : false,
							title: Ext.util.Format.htmlEncode(Sb.Locale.get('confirmRemoval')),
							msg: Ext.util.Format.htmlEncode(Sb.Locale.get('mapMarkerWindowRemoveConfirm')),
							buttons: Ext.Msg.YESNO,
							scope: this,
							fn: function (btn) {
								if (btn == 'yes') {
									this.onRemoveMarker(this.infoWindowPanel.marker);
									this.markerWindow.close();
								}
							}
						});
					}, this)
				}, {
					xtype: 'container',
					cls: 'tool-right',
					defaults: {
						cls: 'info-window-button'
					},
					items: [{
						xtype: 'button',
						cls: 'info-window-button middle',
						ref: '../../ok',
						text: Sb.Locale.get('ok'),
						handler: Ext.createDelegate(function (btn, e) {
							var title = this.infoWindowPanel.titleField.getValue();
							e.stopEvent();
							this.infoWindowPanel.marker.setTitle(title);
							this.infoWindowPanel.marker.model.title = title;
							this.infoWindowPanel.marker.model.description =
								this.infoWindowPanel.descriptionField.getValue();

							this.markerWindow.close();
						}, this)
					}, {
						xtype: 'button',
						cls: 'info-window-button',
						text: Sb.Locale.get('cancelButtonTitle'),
						handler: Ext.createDelegate(function (btn, e) {
							e.stopEvent();
							this.markerWindow.close();
						}, this)
					}]
				}]
			}]
		});
	}
});