/**
 * @class Sb.widget.Text
 * @extends Sb.widget.Widget
 * Class for Text widget
 * @constructor
 * @param {Object} config The config object
 * @xtype text_widget
 */
Ext.define('Sb.widget.Text', {
	extend: 'Sb.widget.Widget',
	xtype: 'text_widget',
	translatable: true,

	cls: 'sb-widget-text',

	/**
	 * Content specially for editor.
	 *  - with default value,
	 *  - updated relative URLs
	 *  - and links like page://h1q88xiezwx
	 */
	contentForEditor: '',

	/**
	 * @type tinymce.Editor
	 * @property instance
	 */
	instance: null,

	initEvents: function () {
		Sb.widget.Text.superclass.initEvents.call(this);

		this.mon(Sb.Editor.model.rootPage, 'change', this.refreshContent, this);
		this.mon(this.model, 'save', function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(Sb.Locale.get('messageChangesSaved'));
		});

		this.mon(this.contentElement, 'contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
			type	: 'widget',
			widget	: this
		});
		this.mon(this.model, 'propertychange', function (key) {
			if (key === 'content') {
				this.refreshText();
			}
		}, this);
		this.mon(this, 'remove', this.removeAnchors);
	},

	refreshContent: function () {
		// :INFO: asinko@ 20120217: Sb.Editor.model.rootPage fire change event,
		// one of subscribers(BlogWidget) kills this subscriber(TextWidget),
		// but handler in this subscriber still called. See bug #97809
		if (this.isDestroyed) {
			return;
		}
		Sb.widget.Text.superclass.refreshContent.call(this);
	},

	refreshText: function () {
		this.widgetContentEl.update(this.prepareData().content);
		this.setupContent();
		this.instance.refit();
		this.refreshAnchors();
		jQuery.addImagesAnimation('#' + this.widgetContentEl.id);
	},

	onCurrentLocaleChange: function (newLocale, oldLocale) {
		this.updateModelFromInstance();
		this.callParent([newLocale, oldLocale]);
	},

	afterRenderContent: function () {
		this.refreshInternalSiteLinks();
		this.refreshAnchors();
		this.loadFonts();
		this.addImagesAnimation();
	},

	addImagesAnimation: function () {
		var id = this.getId();
		Sb.ResourceManager.require([
			Sb.Config.siteResourcesUrl + '/components/' + Sb.Config.jsFrameworkFile
		], function () {
			Sb.ResourceManager.require([
				Sb.Utils.antiCache(
					Sb.Config.siteResourcesUrl + '/modules/text/text.js',
					Sb.Config.isDebug
				),
				Sb.Config.siteResourcesUrl + '/modules/text/text.css'
			], function () {
				jQuery.addImagesAnimation('#' + id);
			});
		});
	},

	prepareData: function () {
		var valid = Sb.Config.imageValidExtensions.join('|'),
			exp = new RegExp('(src="?)(' + Sb.Config.path.attachmentsFolder + '/' +
				Sb.Config.path.imageFolder + '/[^/]+\\.(' + valid + ')"?)', 'img'),
			templatesUrl = Sb.Config.templatesUrl.replace(Sb.Config.baseUrl, ''),
			templateResourcesRegex = new RegExp('(src="?)(' + templatesUrl + ')', 'img');

		this.cleanInternalListeners();
		this.contentForEditor = Ext.util.Format.defaultValue(
			this.model.get('content'), '<p>' + Sb.Locale.get('textWidgetEmptyText') + '</p>'
		);
		this.contentForEditor = this.contentForEditor.replace(
			templateResourcesRegex, '$1' + Sb.Config.baseUrl + '$2'
		);
		this.contentForEditor = this.contentForEditor.replace(exp, '$1' + this.model.getResourcesUrl() + '/$2');
		if (Ext.isChrome || Ext.isSafari) {
			//avoid caching (see bugs #96183, 99015, 131613)
			var attachmentImageRegexp = new RegExp(
				'(<img[^>]*\\s+src=[\"\'])([^\'\"?]*)[?]*([^\'\"]*)([\'\"][^>]*>)', 'igm'
			);
			var timestamp = new Date().getTime();
			this.contentForEditor = this.contentForEditor.replace(attachmentImageRegexp, '$1$2?' + timestamp + '$4');
		}

		//:INFO: mmiropolskiy@: remove internal identifiers, see bug #139573
		exp = /id=["'](wrapper|template|layout|watermark|column|header|footer|content|widget|ext-|sb-)[^"']*["']/igm;
		this.contentForEditor = this.contentForEditor.replace(exp, '');

		var content = this.contentForEditor;
		var placeholders = Ext.apply({
			siteName: Sb.Editor.model.get('companyName')
		}, Sb.Editor.model.get('ownerInfo'));
		Ext.iterate(placeholders, function (name, value) {
			value = value || Sb.Locale.get(name + 'DefaultPlaceholder');
			// Workaround for unsupported placeholders in attribute values
			content = content.replace(
				new RegExp('"([^"]*)%%' + name + '%%([^"]*)"', 'g'),
				'"$1' + value + '$2"'
			);
		}, this);

		Ext.iterate(placeholders, function (name, value) {
			value = value || Sb.Locale.get(name + 'DefaultPlaceholder');
			content = content.replace(
				new RegExp('%%' + name + '%%', 'g'),
				'<span data-sb-placeholder="' + name + '">' + value + '</span>'
			);
		}, this);
		return {content: content};
	},

	refreshAnchors: function () {
		Sb.Editor.siteView.currentPage.refreshAnchorsByWidget(this.model, this.model.collectAnchors());
	},

	removeAnchors: function () {
		Sb.Editor.siteView.currentPage.removeAnchorsByWidget(this.model);
	},

	loadFonts: function () {
		var re = /font-family:\s*(.*?);/ig,
			match;

		while ((match = re.exec(this.contentForEditor)) != null) {
			Sb.FontManager.loadFont(match[1]);
		}
	},

	/**
	 * Refresh internal site links by adding onclick and published-like URLs
	 */
	refreshInternalSiteLinks: function () {
		var rootPage = Sb.Editor.model.rootPage,
			pages = rootPage.getChildren('mixedcollection'),
			rePage = new RegExp('^page://([^/]+)');
		pages.add(rootPage.get('identity'), rootPage);

		this.getContentTarget().select('a').each(function (flyLink) {
			var matches = flyLink.getAttribute('href').match(rePage);
			if (null == matches) {
				return;
			}
			var url = matches[1].split('#'),
				page = url[0] ? pages.map[url[0]] : undefined,
				anchor = url[1] ? url[1] : undefined;
			this.wrapInternalLink(flyLink, page, anchor);
		}, this);
	},

	/**
	 * Wrap internal link, changing click behaviour and also hacking href to looks like on published side
	 *
	 * @param {Ext.Element} flyLink
	 * @param {Sb.model.SitePage} page Site page model node
	 * @param {String} anchor Site page anchor
	 */
	wrapInternalLink: function (flyLink, page, anchor) {
		if (undefined === page) {
			flyLink.set({href: Sb.Editor.model.getNotFoundUrl()});
			return;
		}
		flyLink.on('click', function (event, el, options) {
			if (!options.page.isDestroyed) {
				event.stopEvent();
				Sb.Editor.siteView.setCurrentPage(options.page, function () {
					if (undefined !== anchor) {
						window.location.hash = anchor;
					}
				});
			}
		}, this, {'page' : page});
		var link = Ext.get(flyLink.getAttribute('id')),
			anchorPart = anchor ? '#' + anchor : '';
		link.set({href: page.getFullUrl() + anchorPart});
		this.addInternalListener(page, 'urlchange', function () {
			link.set({href: page.getFullUrl() + anchorPart});
		});
	},

	getEditPanel: function () {
		return Sb.WidgetTypeManager.get('text').getEditPanel({hidden: true});
	},

	onStartEdit: function () {
		this.callParent();
		this.initEditor();
	},

	updateModelFromInstance: function () {
		var templatesUrl = Sb.Config.templatesUrl.replace(Sb.Config.baseUrl, ''),
			content, attachmentImageRegexp;

		if (!this.instance.initialized) {
			return;
		}

		content = this.instance.getContent();
		var sizeInBytes = encodeURI(content).split(/%..|./).length - 1; // To properly calculate UTF-8 symbols
		if (sizeInBytes > Sb.Config.maxTextWidgetSize) {
			Ext.MessageBox.show({
				title: Sb.Locale.get('warningMessageBoxTitle'),
				msg: Sb.Locale.get('textWidgetTooLongContent', Sb.Config.maxTextWidgetSize),
				buttons: Ext.MessageBox.OK,
				buttonText: {
					ok: Sb.Locale.get('ok')
				},
				closable: false
			});
			this.model.set('content', content);
			return;
		}

		attachmentImageRegexp = new RegExp(
			'(<img[^>]*\\s+src=").*(' + this.model.getResourcesUrl() + '/)([^"]*"[^>]*>)', 'igm'
		);
		content = content.replace(attachmentImageRegexp, '$1$3');
		content = content.replace(
			new RegExp('(<img[^>]*\\s+src=")(' + Sb.Config.templatesUrl + ')([^"]*"[^>]*>)', 'igm'),
			'$1' + templatesUrl + '$3'
		);

		this.model.set('content', content);
	},

	onCompleteEdit: function () {
		if (this.model.get('visibilityMode') !== this.memento.visibilityMode) {
			this.updateVisibilityState();
		}
		this.updateModelFromInstance();
		this.model.saveIfChanged(this.memento);
		this.refreshContent();
		Sb.Editor.model.updateUploadedImages();
	},

	onCancelEdit: function () {
		this.refreshContent();
		Sb.widget.Text.superclass.onCancelEdit.call(this);
	},

	onHideEdit: function () {
		this.removeEditor();
		Sb.widget.Text.superclass.onHideEdit.call(this);
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.mceMenu') || event.getTarget('.sb-color-dropdown') || event.getTarget('#mceModalBlocker')
			|| Ext.fly(event.target).up('.mceModal') || event.getTarget('.mceWrapper')
			|| (event.getTarget('.sb-dialog')) || Ext.isObject(this.instance.ExpandedSelectbox)) {
			return;
		}
		Sb.widget.Text.superclass.onDocumentClick.call(this, event);
	},

	/**
	 * Determines, when passed node is valid by having right name and attribute
	 * @param node Raw DOM node
	 */
	isValidNode: function (node, name, attribute) {
		if (!(node && node.nodeName == name)) {
			return false;
		}
		return node.hasAttribute(attribute);
	},

	/**
	 * Determines, when passed node is valid link with href attribute
	 * @param node Raw DOM node
	 */
	isValidLink: function (node) {
		return this.isValidNode(node, 'A', 'href') && node.getAttribute('href').substring(0, 7) !== 'mailto:';
	},

	/**
	 * Determines, when passed node is valid email link
	 * @param node
	 * @returns {boolean}
	 */
	isValidEmailLink: function (node) {
		return this.isValidNode(node, 'A', 'href') && node.getAttribute('href').substring(0, 7) === 'mailto:';
	},

	/**
	 * Determines, when passed node is valid anchor with name attribute
	 * @param node Raw DOM node
	 */
	isValidAnchor: function (node) {
		return this.isValidNode(node, 'A', 'name');
	},

	initEditor: function () {
		var height			= this.el.getSize().height,
			currentWidget	= this;

		this.tinyFakeContainer = this.getContentTarget().parent().createChild({
			id: Ext.id(),
			tag: 'div'
		});

		this.instance = new tinymce.Editor(this.tinyFakeContainer.id, {
			mode		: 'none',
			height		: height,
			width		: '100%',
			plugins		: 'inlinepopups,media,tablewithalign,paste',
			body_class	: 'pageContentText widget-text' + (Sb.Editor.model.isRtl() ? ' rtl' : ''),
			theme		: 'sbt',
			language	: Sb.Config.locale.split('_')[0],
			media_strict: false,
			gecko_spellcheck: true,

			theme_sbt_resizing_min_height			: 20,
			theme_sbt_toolbar_location				: 'external',
			theme_sbt_toolbar_containerId			: this.getEditPanel().body.id,
			theme_sbt_toolbar_align					: 'left',
			theme_sbt_statusbar_location			: 'none',
			theme_sbt_resizing						: false,
			theme_sbt_buttons1: 'sbformatselect, bold, italic, underline, strikethrough,' +
				'justifyleft, justifycenter, justifyright, justifyfull, numlist, bullist, hr, undo, ' +
				'redo, sbflash, sbimage',
			theme_sbt_buttons2: 'sbfontselect, sbfontsizeselect, forecolor, backcolor, outdent, indent, table,' +
				'charmap, code, anchor, unlink, link, sbemaillink',
			theme_sbt_buttons3: 'row_props, cell_props, row_before, row_after, delete_row, col_before, col_after, ' +
				'delete_col, split_cells, merge_cells',

			relative_urls							: false,
			convert_urls							: false,

			formats: {
				bold	: {inline : 'span', styles : {fontWeight: '%value'}},
				italic	: {inline : 'span', styles : {fontStyle : '%value'}}
			},

			setupcontent_callback: Ext.createDelegate(this.setupContent, this),

			iframe_load_callback: Ext.createDelegate(function () {
				if (Ext.isIE11) {
					tinymce.isIE = false;
				}
				this.getContentTarget().addClass('sb-height-helper');
				Ext.get(this.instance.editorContainer).show();

				Ext.defer(function () {
					if (!tinymce.activeEditor) {
						return;
					}
					this.instance.refit();

					var editorBody = this.instance.getBody(),
						range = this.instance.dom.createRng(),
						firstNode;

					if (editorBody) {
						firstNode = editorBody.firstChild;
						range.setStart(firstNode, 0);
						range.setEnd(firstNode, 0);
						this.instance.selection.setRng(range);

						editorBody.focus();
					}

					this.instance.getWin().focus();
				}, 100, this);
			}, this),

			setup: function (ed) {
				ed.addButton('sbformatselect', {
					'class': 'mce_formatselect',
					cmd: 'empty'
				});
				ed.addButton('sbflash', {
					'class': 'mce_flash',
					cmd: 'mceFlashAction'
				});
				ed.addButton('sbimage', {
					'class': 'mce_image',
					cmd: 'mceImageAction'
				});
				ed.addButton('sbfontselect', {
					'class': 'mce_fontselect',
					cmd: 'empty'
				});
				ed.addButton('sbfontsizeselect', {
					'class': 'mce_fontsize',
					cmd: 'empty'
				});
				ed.addButton('sbemaillink', {
					'class': 'mce_emaillink',
					cmd: 'empty'
				});
			}
		});

		this.instance.editPanel = this.getEditPanel();

		this.instance.refit = function () {
			var doc = this.getDoc(),
				docHeight,
				frame = Ext.get(this.id + '_ifr');

			if (!frame || !doc) {
				return;
			}

			//Fixes the issue of the table leaving empty space below iframe
			var table = frame.dom.parentNode.parentNode.parentNode.parentNode;
			table.style.height = 'auto';

			var updateHeight = function (ed) {
				currentWidget.update(ed.getContent());
				docHeight = currentWidget.getContentTarget().dom.scrollHeight;

				if (docHeight != frame.getHeight()) {
					// workaround for bug #136215
					if (Ext.isIE) {
						var overCls		= currentWidget.getStateCls('over'),
							focusCls	= currentWidget.getStateCls('focus');

						currentWidget.removeClass([overCls, focusCls]);
						setTimeout(function() {
							currentWidget.addClass([overCls, focusCls]);
						}, 0);
					}
					frame.setHeight(Math.max(docHeight, 30));
					ed.editPanel.reanchor(false);
				}
			};

			updateHeight(this);
			if (Ext.isIE) {
				// workaround for bug #117808
				updateHeight(this);
			}
		};

		this.instance.onPreInit.add(function (ed) {
			var styles = this.model.getComputedStyles();
			this.syncCss(ed);
			ed.controlManager.setDisabled('media', 1);
			this.enableImageProcessing(ed);

			ed.controlManager.get('backcolor').value = styles.backgroundColor;
			ed.controlManager.get('forecolor').value = styles.textColor;

			for (var i = 1; i <= 4; i++) {
				for (var j = 0; j <= 4; j++) {
					var color = 'color' + i + (j > 0 ? '-' + j : '');
					ed.formatter.register(
						'palette-' + color,
						{inline: 'span', classes: 'palette-' + color, wrap_links: true}
					);
					ed.formatter.register(
						'palette-background-' + color,
						{inline: 'span', classes: 'palette-background-' + color, wrap_links: true}
					);
				}
			}
		}.createDelegate(this));

		this.instance.onInit.add(function (ed) {
			ed.onChange.add(ed.refit);

			ed.editPanel.toolbar3 = Ext.get(ed.controlManager.prefix + 'toolbar3');
			ed.editPanel.toolbar3.setVisibilityMode(Ext.Element.DISPLAY);

			ed.theme.stateControls.remove('bold');
			ed.theme.stateControls.remove('italic');
			ed.editorCommands.addCommands({
				'Bold' : function () {
					var fontWeight = Ext.fly(ed.selection.getNode()).getStyle('font-weight');
					var newValue = (fontWeight == 'bold' || fontWeight == 700) ? 'normal' : 'bold';
					ed.formatter.toggle('bold', {value : newValue});
				},
				'Italic' : function () {
					var newValue = Ext.fly(ed.selection.getNode()).getStyle('font-style') == 'italic' ? 'normal' : 'italic';
					ed.formatter.toggle('italic', {value : newValue});
				}
			});

			ed.addShortcut('ctrl+l', 'left', 'justifyleft');
			Ext.getDom(ed.editorId + '_' + 'justifyleft').title += ' (Ctrl+L)';

			ed.addShortcut('ctrl+e', 'center', 'justifycenter');
			Ext.getDom(ed.editorId + '_' + 'justifycenter').title += ' (Ctrl+E)';

			ed.addShortcut('ctrl+r', 'right', 'justifyright');
			Ext.getDom(ed.editorId + '_' + 'justifyright').title += ' (Ctrl+R)';

			ed.addShortcut('ctrl+j', 'justify', 'justifyfull');
			Ext.getDom(ed.editorId + '_' + 'justifyfull').title += ' (Ctrl+J)';

			this.initSbImage(ed);

			this.initSbFlash(ed);

			this.initSbLink(ed);

			this.initSbEmailLink(ed);

			ed.controlManager.get('anchor').settings.onclick = function () {
				(new Sb.editor.AnchorEditDialog()).show(ed);
			};
			this.instance.onDblClick.add(function (ed, evt) {
				var n = ed.selection.getNode();
				if (currentWidget.isValidAnchor(n)) {
					(new Sb.editor.AnchorEditDialog()).show(ed);
				}
			});

			this.instance.onKeyDown.add(this.removeAnchorCorrectly);

			if (Ext.isSafari) {
				tinymce.dom.Event.add(ed.getBody(), 'dragstart', function (e) {
					if (currentWidget.isImgNode(e.target)) {
						e.target.ondragend = function (evt) {
							var od = evt.target.ownerDocument;
							od.body.innerHTML = od.body.innerHTML.replace('<img src="' + evt.target.src + '">', evt.target.outerHTML);
						};
					}
				});
			}

			ed.controlManager.get('backcolor').settings.onselect = ed.controlManager.get('forecolor').settings.onselect = function (v) {
				for (var i = 1; i <= 4; i++) {
					for (var j = 0; j <= 4; j++) {
						ed.formatter.remove('palette-' + (this.cmd == 'HiliteColor' ? 'background-' : '') +
							'color' + i + (j > 0 ? '-' + j : ''));
					}
				}

				var removeCmdFormat = function(cmd) {
					ed.formatter.toggle(cmd.toLowerCase(), {value: '#000000'});
					ed.formatter.toggle(cmd.toLowerCase(), {value: '#000000'});
				};

				if ('transparent' == v) {
					removeCmdFormat(this.cmd);
					return;
				}
				if (Ext.ux.util.Color.isValid(v)) {
					ed.formatter.apply(this.cmd.toLowerCase(), {value: v});
				} else {
					removeCmdFormat(this.cmd);
					ed.formatter.apply('palette-' + (this.cmd == 'HiliteColor' ? 'background-' : '') + v);
				}
			};

			ed.controlManager.get('backcolor').settings.onclick = ed.controlManager.get('forecolor').settings.onclick = function (v) {
				if (tinymce.isIE) {
					bm = ed.selection.getBookmark(1);
				}
				this.onselect(v);
			};

			var alignments = {
				alignleft: {'float' : 'left', marginRight : '20px'},
				alignright: {'float' : 'right', marginLeft : '20px'}
			};
			Ext.iterate(alignments, function (alignName, styles) {
				var align = ed.formatter.get(alignName);
				align[1].styles = styles;
				ed.formatter.register(alignName, align);
			});

			if (this.getShowEditPanelState()) {
				this.getEditPanel().show(Sb.Editor.Ui.editComponent);
			} else {
				this.propertiesEl.removeClass(this.baseCls + '-control-item-disabled');
				this.propertiesEl.child('span').update(Sb.Locale.get('propertiesWidgetShow'));
			}

			this.createFontSizeSelector(ed);
			this.createFontSelector(ed);
			this.createFormatSelector(ed);
		}.createDelegate(this));

		this.instance.onGetContent.add(function (ed) {
			document.getElementById(ed.id + '_ifr').setAttribute("allowTransparency", "true");
		});
		this.instance.onNodeChange.add(function (ed, cm, n, co) {
			if (Ext.isObject(ed.ExpandedSelectbox)) {
				ed.ExpandedSelectbox.collapse();
			}
			ed.lastSelectionBookmark = ed.selection.getBookmark(1);

			if (undefined !== Ext.fly(n).child('body', true)) {
				return;
			}

			var fontWeight = Ext.fly(n).getStyle('font-weight');
			cm.get('bold').setActive(fontWeight == 'bold' || fontWeight == 700);
			cm.get('italic').setActive(Ext.fly(n).getStyle('font-style') == 'italic');
		});

		this.instance.onDblClick.add(function (ed, evt) {
			var domEl = ed.selection.getNode();
			if (currentWidget.isImgNode(domEl)) {
				Sb.Utils.getImageOriginalSize(domEl.getAttribute('src'), function (originalSize) {
					ed.imageDlg = new Sb.editor.ImageEditDialog({
						domEl				: domEl,
						afterHeightChange	: ed.refit.createDelegate(ed),
						modelData			: {
							originalSize	: originalSize
						}
					});
					ed.imageDlg.show();
				}.createDelegate(this));
			}
		});

		this.instance.onChange.add(function (ed) {
			Ext.each(ed.dom.select('a'), function (item) {
				Ext.fly(item).addClass('link');
			});
		});
		this.instance.onKeyUp.add(this.instance.refit);
		this.instance.onPaste.add(this.instance.refit);

		this.instance.onKeyDown.add(Sb.widget.Text.onKeyDown);

		this.instance.render();
	},

	setupContent: function () {
		var content = this.model.get('content').length === 0
			? ''
			: this.contentForEditor;
		this.instance.setContent(content);
		this.instance.undoManager.clear();
	},

	/**
	 * @param {tinymce.Editor} ed
	 */
	enableImageProcessing: function (ed) {
		var me = this;

		ed.parser.addNodeFilter('span', function(nodes) {
			Ext.each(nodes, function (node) {
				if ((node.attr('class') || '').indexOf('image-block') !== -1) {
					me.imageBlockToPlaceholder(node);
				}
			});
		});

		ed.parser.addNodeFilter('img', function (nodes) {
			Ext.each(nodes, function (node) {
				var cls = node.attr('class') || '';
				if (cls.indexOf('mce') === -1) {
					cls = cls + ' mceImage';
					if (cls.indexOf('image-block') === -1) {
						cls += 'image-block';
					}
					node.attr('class', cls);
				}
			});
		});

		ed.serializer.addNodeFilter('img', function(nodes) {
			Ext.each(nodes, function (node) {
				if ((node.attr('class') || '').indexOf('mceImage') !== -1) {
					me.placeholderToImageBlock(node);
				}
			});
		});
	},

	imageBlockToPlaceholder: function (node) {
		var img = node.getAll('img')[0],
			id;

		if (!img) {
			return;
		}

		id = img.attr('id');
		img = img.clone();

		img.attr({
			id: id,
			'class': node.attr('class') + ' mceImage',
			style: (node.attr('style') || '').replace(/width:\s*\d+px;/g, '').trim()
		});

		node.replace(img);
	},

	placeholderToImageBlock: function (node) {
		var block, img, caption, text;

		block = new tinymce.html.Node('span', 1);
		block.attr({
			'class': node.attr('class').replace('mceImage', '').trim(),
			style: node.attr('style') + ' width:' + node.attr('width') + 'px;'
		});

		img = new tinymce.html.Node('img', 1);
		block.append(img);

		img.attr({
			id: node.attr('id'),
			src: node.attr('src'),
			width: node.attr('width'),
			height: node.attr('height'),
			alt: node.attr('alt'),
			title: node.attr('title')
		});

		if (img.attr('alt')
			&& (node.attr('class').indexOf(Sb.auxilaryModel.Image.captionUnderImage) !== -1
				|| node.attr('class').indexOf(Sb.auxilaryModel.Image.captionOverImage) !== -1
			)
		) {
			caption = new tinymce.html.Node('span', 1);
			block.append(caption);
			text = new tinymce.html.Node('#text', 3);
			text.value = img.attr('alt');
			caption.append(text);
		}

		node.replace(block);
	},

	/**
	 * Tries to remove html anchor correctly, as in some cases they are not removed and duplicates
	 *
	 * @param {tinymce.Editor} ed
	 * @param {Event} e
	 */
	removeAnchorCorrectly: function (ed, e) {
		var isDelete	= e.keyCode == Ext.EventObject.DELETE,
			isBackspace	= e.keyCode == Ext.EventObject.BACKSPACE,
			s			= ed.selection;
		if (!s.collapsed && (isDelete || isBackspace)) {
			var n			= s.getNode(),
				r			= s.getRng(),
				isAnchor	= function (candidateNode) {
					return candidateNode && candidateNode.nodeName == 'A' && candidateNode.innerHTML === '';
				};

			if (!isAnchor(n) && r.startContainer) {
				n = r.startContainer;
				if (!isAnchor(n)) {
					if (isDelete && r.startOffset == r.startContainer.length) {
						n = r.endContainer.nextSibling;
						if (n && !isAnchor(n)) {
							n = r.endContainer.nextSibling.nextSibling;
						}
					} else if (isBackspace && r.startOffset === 0) {
						n = r.startContainer.previousSibling;
						if (n && !isAnchor(n)) {
							n = r.startContainer.previousSibling.previousSibling;
						}
					}
				}
			}
			if (isAnchor(n)) {
				tinymce.dom.Event.cancel(e);
				tinymce.DOM.remove(n);
			}
		}
	},

	removeEditor: function () {
		this.tinyFakeContainer.remove();
		this.instance.remove();
		this.getContentTarget().removeClass('sb-height-helper');
	},

	/**
	 * @param {tinymce.Editor} ed
	 */
	createFontSizeSelector: function (ed) {
		var p = Ext.fly(ed.controlManager.get('sbfontsizeselect').id).parent();
		Ext.DomHelper.overwrite(p, '');
		var lastFontSizeValue		= null,		// Last font size value for current selection
			isFontSizeFromSource	= false,	// Flag for case when we set font size from source
			isNodeChangeFromCode	= false;	// Flag for case when we change node by our actions (not user)
		new Sb.form.FontSizeComboBox({
			id: 'mceFontSize',
			renderTo: p,
			width: 78,
			listeners: {
				scope: this,
				valid: function (combo) {
					if (isFontSizeFromSource || null == ed.selection) {
						return;
					}
					var value = combo.getRawValue();
					if (lastFontSizeValue != value) {
						if (Ext.isIE) {
							ed.selection.moveToBookmark(ed.lastSelectionBookmark);
							combo.focusAfterSelectEvent = false;
						}
						isNodeChangeFromCode = true;
						ed.execCommand('FontSize', false, value + 'px');
						isNodeChangeFromCode = false;
						lastFontSizeValue = value;
					}
					combo.focus();
				},
				afterrender: function (combo) {
					ed.onNodeChange.add(function (ed, cm, n, co, ob) {
						if (isNodeChangeFromCode || undefined !== Ext.fly(n).child('body', true)) {
							return;
						}
						lastFontSizeValue = null;
						var fontSizeValue = Ext.fly(n).getStyle('font-size').replace(/px$/, '');
						if (combo.getRawValue() != fontSizeValue) {
							isFontSizeFromSource = true;
							combo.setValue(fontSizeValue);
							isFontSizeFromSource = false;
						}
					});
				}
			}
		});
	},

	/**
	 * @param {tinymce.Editor} ed
	 */
	createFontSelector: function (ed) {
		var p = Ext.fly(ed.controlManager.get('sbfontselect').id).parent();
		Ext.DomHelper.overwrite(p, '');
		new Sb.form.FontFamilyComboBox({
			id: 'mceFontFamily',
			renderTo: p,
			width: 112,
			listeners: {
				scope: this,
				beforeselect: function (combo, record) {
					if (combo.getValue() == record.get('value')) {
						combo.collapse();
						return false;
					}
				},
				select: function (combo, record) {
					var fontFamily = record.get('fontFamilyCss'),
						fontUrl = Sb.FontManager.getFontUrl(fontFamily.split(',')[0].replace(/('|")/g, ''));

					if (Ext.isIE) {
						ed.selection.moveToBookmark(ed.lastSelectionBookmark);
						combo.focusAfterSelectEvent = false;
					}
					tinyMCE.execCommand('FontName', false, fontFamily);

					if (fontUrl) {
						ed.dom.loadCSS(fontUrl);
					}
				},
				expand: function (combo) {
					ed.ExpandedSelectbox = combo;
				},
				collapse: function (combo) {
					ed.ExpandedSelectbox = null;
				},
				afterrender: function (combo) {
					ed.onNodeChange.add(function (ed, cm, n, co, ob) {
						if (undefined !== Ext.fly(n).child('body', true)) {
							return;
						}

						var ff = '';
						Ext.each(ob.parents, function (item) {
							if ((ff = Ext.fly(item).getStyle('font-family'))) {
								return false;
							}
						});
						ff = ff.split(",")[0].replace(/('|")/g, '');
						combo.setValue(ff || Sb.Editor.model.design.data.font.textFont);
					});
				}
			}
		});
	},

	/**
	 * @param {tinymce.Editor} ed
	 */
	createFormatSelector: function (ed) {
		var p = Ext.fly(ed.controlManager.get('sbformatselect').id).parent(),
			styles = this.model.getComputedStyles();
		Ext.DomHelper.overwrite(p, '');
		new Ext.form.ComboBox({
			store: new Ext.data.ArrayStore({
				fields: ['tag', 'display', 'color'],
				data: [['p', Sb.Locale.get('bodyText'), Sb.Editor.model.design.getColor(styles.textColor)],
					['h1', Sb.Locale.get('heading1'), Sb.Editor.model.design.getColor(styles.h1Color)],
					['h2', Sb.Locale.get('heading2'), Sb.Editor.model.design.getColor(styles.h2Color)]
				]
			}),
			triggerAction: 'all',
			mode: 'local',
			valueField: 'tag',
			displayField: 'display',
			renderTo: p,
			width: 112,
			tpl: new Ext.XTemplate(
				'<tpl for="."><div class="x-combo-list-item pageContentText"><{tag} style="background-color: ' +
				styles.backgroundColor + '; color: {color};">{display}</{tag}>' +
				'</div></tpl>'
			),
			listeners: {
				scope: this,
				expand: function (combo) {
					combo.list.addClass('mce-sb-formatselect');
					ed.ExpandedSelectbox = combo;
				},
				collapse: function (combo) {
					ed.ExpandedSelectbox = null;
				},
				select: function (combo, record, index) {
					if (Ext.isIE) {
						ed.selection.moveToBookmark(ed.lastSelectionBookmark);
					}
					tinyMCE.execCommand('FormatBlock', false, record.get('tag'));
				},
				afterrender: function (combo) {
					ed.onNodeChange.add(function (ed, cm, n, co, ob) {
						var i, len;
						for (i = 0, len = ob.parents.length; i < len; i++) {
							if (ed.dom.isBlock(ob.parents[i])) {
								if (/^(P|H1|H2)$/.test(ob.parents[i].nodeName)) {
									combo.setValue(ob.parents[i].nodeName.toLowerCase());
									return;
								}
								break;
							}
						}
						combo.setValue(Sb.Locale.get('customParagraph'));
					});
				}
			}
		});
	},

	/**
	 * Initialize SB insert image button
	 * @param {tinymce.Editor} ed
	 */
	initSbImage: function (ed) {
		var p = Ext.fly(ed.controlManager.get('sbimage').id).parent(),
			me = this;
		Ext.DomHelper.overwrite(p, '');
		p.addClass('mce-sb-toolbar-button');
		var buttonImg = new Ext.ux.form.FileUploadField({
			id: 'mce-sb-image',
			regex: Sb.Utils.getImageValidExtensionsRegex(),
			buttonOnly: true,
			name: 'image',
			buttonText: Sb.Locale.get('mceEditImage'),
			listeners: {
				scope: this,
				fileselected: this.onImageUpload,
				afterrender: function (btn) {
					btn.fileInput.set({title: Sb.Locale.get('mceEditImage')});
					ed.onNodeChange.add(function (ed, cm, n, co, ob) {
						var isImageNode = me.isImgNode(n);
						btn.fileInput[isImageNode ? 'hide' : 'show']();
						if (isImageNode) {
							buttonImg.button.setHandler(function () {
								Sb.Utils.getImageOriginalSize(n.getAttribute('src'), function (originalSize) {
									ed.imageDlg = new Sb.editor.ImageEditDialog({
										domEl				: n,
										afterHeightChange	: ed.refit.createDelegate(ed),
										modelData			: {
											originalSize	: originalSize
										}
									});
									ed.imageDlg.show();
								}.createDelegate(this));
							}, this);
						}
						btn.button[isImageNode ? 'addClass' : 'removeClass']('x-btn-over');
					});
				}
			}
		});
		new Ext.form.FormPanel({
			cls			: 'mce-sb-image',
			renderTo	: p,
			fileUpload	: true,
			border		: false,
			hideLabels	: true,
			items		: buttonImg
		});

		buttonImg.button.setIconClass('mce-sb-image-icon');

		buttonImg.button.on('mouseout', function (button, evt) {
			if (me.isImgNode(ed.selection.getNode())) {
				buttonImg.button.addClass('x-btn-over');
			}
		});
	},

	isImgNode: function (n) {
		return n && n.nodeName == 'IMG' && (n.className.indexOf('mceItemFlash') == -1)
			&& (n.className.indexOf('mceItemMedia') == -1);
	},

	isFlashNode: function (n) {
		return n && n.nodeName == 'IMG' && (n.className.indexOf('mceItemFlash') != -1);
	},

	/**
	 * Initialize SB insert flash button
	 * @param {tinymce.Editor} ed
	 */
	initSbFlash: function (ed) {
		var p = Ext.fly(ed.controlManager.get('sbflash').id).parent(),
			me = this;
		Ext.DomHelper.overwrite(p, '');
		p.addClass('mce-sb-toolbar-button');
		var buttonFlash = new Ext.ux.form.FileUploadField({
			id: 'mce-sb-flash',
			regex: new RegExp('\\.(' + Sb.Config.flashValidExtensions.join('|') + ')$', 'i'),
			buttonOnly: true,
			name: 'flash',
			buttonText: '',
			listeners: {
				scope: this,
				fileselected: this.onFlashUpload,
				afterrender: function (btn) {
					btn.fileInput.set({title: Sb.Locale.get('mceInsertFlash')});
					ed.onNodeChange.add(function (ed, cm, n, co, ob) {
						var isFlashNode = me.isFlashNode(n);
						btn.fileInput[isFlashNode ? 'hide' : 'show']();
						btn.button[isFlashNode ? 'addClass' : 'removeClass']('x-btn-over');
					});
				}
			}
		});
		new Ext.form.FormPanel({
			cls			: 'mce-sb-flash',
			renderTo	: p,
			fileUpload	: true,
			border		: false,
			hideLabels	: true,
			items		: buttonFlash
		});

		buttonFlash.button.on('click', function () {
			var selection = ed.selection.getNode();
			if (!me.isFlashNode(selection)) {
				return;
			}
			ed.imageDlg = new Sb.editor.FlashEditDialog({
				domEl: selection,
				afterHeightChange: ed.refit
			});
			ed.imageDlg.show();

		}, this);
		buttonFlash.button.setIconClass('mce-sb-flash-icon');
	},

	/**
	 * Flash change handler, that fires up when user select flash in upload control
	 */
	onFlashUpload: function (field, fieldValue) {
		this.submitFile(
			field,
			Sb.Config.baseUrl + '/Upload/Flash/',
			'afterUploadFlash',
			Sb.Locale.get('uploadingFlash'),
			'flashEditDialogChangeWarningInvalidType'
		);
	},

	afterUploadFlash: function (result) {
		var flashUrl = this.model.getResourcesUrl() + '/' + Sb.Config.path.attachmentsFolder + '/' +
				Sb.Config.path.imageFolder + '/' + encodeURIComponent(result.sourceFileName),
			uniqueId = Ext.id(null, 'mce-'),
			flash = Sb.utils.Flash.createCode(
				flashUrl,
				result.width,
				result.height, {
					id: uniqueId,
					withMask: false,
					style: 'float: ' + (Sb.Editor.model.isRtl() ? 'right' : 'left') + ';'
				}
			);

		if (Ext.isIE) {
			this.instance.selection.moveToBookmark(this.instance.lastSelectionBookmark);
		}
		this.instance.execCommand('mceInsertContent', false, flash);

		this.instance.imageDlg = new Sb.editor.FlashEditDialog({
			domEl: this.instance.dom.get(uniqueId),
			afterHeightChange: this.instance.refit.createDelegate(this.instance),
			isUploaded: true
		});
		this.instance.imageDlg.show();
	},

	/**
	 * Image change handler, that fires up when user select image in upload control
	 */
	onImageUpload: function (field, fieldValue) {
		this.submitFile(
			field,
			Sb.Config.baseUrl + '/Upload/Image/',
			'afterUploadImage',
			Sb.Locale.get('uploadingImage'),
			'imageEditDialogChangeWarningInvalidType'
		);
	},

	submitFile: function (field, url, afterUploadAction, waitMsg, invalidTypeError) {
		var frm = field.ownerCt.getForm();

		if (!frm.isValid()) {
			Sb.Editor.Ui.mainWindow.restore();
			Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get(invalidTypeError));
			return;
		}

		frm.submit({
			url: url,
			waitMsg: waitMsg,
			scope: this,
			success: function (fp, action) {
				this[afterUploadAction](action.result);
				field.resetOnUpload();
			},
			failure: function (fp, action) {
				this.afterUploadFileFailure(action.result.message);
			}
		});
	},

	afterUploadFileFailure: function (message) {
		Sb.Editor.Ui.mainWindow.restore();
		Sb.Editor.Ui.mainWindow.statusBar.setMessageError(Sb.Locale.get(message));
	},

	afterUploadImage: function (result) {
		var imagePath = this.model.getResourcesUrl() + '/' + Sb.Config.path.attachmentsFolder + '/' +
			Sb.Config.path.imageFolder + '/' + encodeURIComponent(result.sourceFileName);

		var uniqueId = Ext.id(null, 'mce-'),
			block = '<span class="image-block" ' +
				'style="float: ' + (Sb.Editor.model.isRtl() ? 'right' : 'left') + ';">' +
				'<img src="' + imagePath + '" id="' + uniqueId + '" ' +
				'width="'  + result.width  + '"' +
				'height="' + result.height + '" /></span>';

		if (Ext.isIE) {
			this.instance.selection.moveToBookmark(this.instance.lastSelectionBookmark);
		}
		this.instance.execCommand('mceInsertContent', false, block);

		Ext.get(this.instance.dom.get(uniqueId)).on('load', function () {
			Sb.Utils.getImageOriginalSize(imagePath, function (originalSize) {
				this.instance.imageDlg = new Sb.editor.ImageEditDialog({
				domEl				: this.instance.dom.get(uniqueId),
				afterHeightChange	: this.instance.refit.createDelegate(this.instance),
				isUploaded			: true,
				modelData			: {
					originalSize	: originalSize
				}
			});
			this.instance.refit();
			this.instance.imageDlg.show();
			}.createDelegate(this));
		}, this, {single: true});
	},

	/**
	 * Initialize insert link button
	 * @param ed
	 */
	initSbLink: function (ed) {
		var textWidget = this,
			p = Ext.fly(ed.controlManager.get('link').id).parent();

		Ext.DomHelper.overwrite(p, '');
		p.addClass('mce-sb-toolbar-button');
		new Ext.Button({
			id: 'id-mce-sb-link',
			cls: 'mce-sb-link',
			iconCls: 'mce-sb-link-icon',
			text: Sb.Locale.get('mceInsertLink'),
			tooltip: Sb.Locale.get('linkButtonTooltip'),
			tooltipType: 'title',
			renderTo: p,
			disabled: true,
			listeners: {
				scope: this,
				click: function () {
					(new Sb.editor.LinkEditDialog()).show(ed);
				},
				mouseout: function (btn, evt){
					var n = ed.selection.getNode();
					if (textWidget.isValidLink(n)) {
						btn.addClass('x-btn-over');
					}
				},
				afterrender: function (btn) {
					ed.onNodeChange.add(function (ed, cm, n, co, ob) {
						var isLink	= textWidget.isValidLink(n),
							isEdit	= cm.get('link').active && !textWidget.isValidEmailLink(n);
						if (co && !isLink) {
							btn.disable();
							btn.setText(Sb.Locale.get('mceInsertLink'));
						}
						if (co && isLink) {
							btn.enable();
							btn.setText(Sb.Locale.get('mceEditLink'));
						}
						if (ed.selection.getContent()) {
							btn.enable();
							btn.setText(Sb.Locale.get(isEdit ? 'mceEditLink' : 'mceInsertLink'));
						}
						btn[!btn.disabled && (isLink || isEdit) ? 'addClass' : 'removeClass']('x-btn-over');
					});
				}
			}
		});
	},

	/**
	 * Initialize insert email link button
	 * @param {tinymce.Editor} ed
	 */
	initSbEmailLink: function (ed) {
		var textWidget = this,
			p = Ext.fly(ed.controlManager.get('sbemaillink').id).parent();

		Ext.DomHelper.overwrite(p, '');
		p.addClass('mce-sb-toolbar-button');
		new Ext.Button({
			id: 'id-mce-sb-email-link',
			cls: 'mce-sb-email-link',
			iconCls: 'mce-sb-email-link-icon',
			tooltip: Sb.Locale.get('emailLinkButtonTooltip'),
			tooltipType: 'title',
			renderTo: p,
			listeners: {
				scope: this,
				click: function () {
					(new Sb.editor.EmailLinkEditDialog()).show(ed);
				},
				mouseout: function (btn, evt){
					var n = ed.selection.getNode();
					if (textWidget.isValidEmailLink(n)) {
						btn.addClass('x-btn-over');
					}
				},
				afterrender: function (btn) {
					ed.onNodeChange.add(function (ed, cm, n, co, ob) {
						var isLink	= textWidget.isValidEmailLink(n);
						if (co && !isLink) {
							btn.disable();
						}
						if ((co && isLink) || ed.selection.getContent()) {
							btn.enable();
						}
						btn[isLink ? 'addClass' : 'removeClass']('x-btn-over');
					});
				}
			}
		});
	},

	syncCss: function (ed) {
		var styles, cssPrefix;

		cssPrefix = Sb.Config.templatesUrl + '/' + Sb.Editor.model.data.template.code;
		ed.dom.loadCSS(cssPrefix + '/css/style.css');
		if (Sb.Editor.model.isRtl()) {
			ed.dom.loadCSS(cssPrefix + '/css/style-rtl.css');
		}

		function addStyle(cssText) {
			var head = ed.dom.select('head')[0],
				widgetStylesNode = ed.dom.create('style', {type: 'text/css'});

			if (widgetStylesNode.styleSheet) { // IE
				head.appendChild(widgetStylesNode);
				widgetStylesNode.styleSheet.cssText = cssText;
			} else { // w3c
				widgetStylesNode.appendChild(ed.getDoc().createTextNode(cssText));
				head.appendChild(widgetStylesNode);
			}
		}

		styles = this.model.getComputedStyles();
		addStyle(Sb.StyleManager.getStyleSheet('font-set'));
		addStyle(Sb.SiteView.fontColorCssTemplate.apply({
			className: 'body',
			textColor: Sb.Editor.model.design.getColor(styles.textColor),
			linkColor: Sb.Editor.model.design.getColor(styles.linkColor),
			h1Color: Sb.Editor.model.design.getColor(styles.h1Color),
			h2Color: Sb.Editor.model.design.getColor(styles.h2Color)
		}));
		addStyle(Sb.StyleManager.getStyleSheet('fonts'));
		addStyle(Sb.StyleManager.getStyleSheet('colorscheme'));
	},

	onStylesChange: function (styles) {
		this.callParent(arguments);

		if (styles.borderRadius && this.widgetContentEl) {
			this.widgetContentEl.setStyle('borderRadius', styles.borderRadius);
		}
	}
});

Sb.widget.Text.onKeyDown = function (inst, e) {
	Sb.app.renewSessionHandler();
	var code = e.keyCode || e.which;
	if (code == 9) { //"Tab" pressed
		inst.execCommand('mceInsertContent', false, '&nbsp;&nbsp;&nbsp;&nbsp;');
		if (e.preventDefault) {
			e.preventDefault();
		}
		return false;
	}
};

Ext.define('Sb.widget.TextEditPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	defaultPosition: 't',

	initComponent: function () {
		Ext.apply(this, {
			cls: 'sb-widget-edit-panel-text',
			bodyCssClass: 'defaultSkin'
		});

		Sb.widget.TextEditPanel.superclass.initComponent.apply(this, arguments);
	},

	setToolbarVisibility: function (toolbarName, isVisible) {
		var toolbar = this[toolbarName];
		if (!isVisible) {
			toolbar.slideOut('t', {
				easing: 'easeOut',
				duration: 0.3,
				concurrent: false,
				stopFx: true
			});
		} else if (!toolbar.isSlideInActive && (!toolbar.isVisible() || toolbar.hasActiveFx())) {
			toolbar.isSlideInActive = true;
			toolbar.slideIn('t', {
				easing: 'easeIn',
				duration: 0.3,
				concurrent: false,
				stopFx: true,
				scope: toolbar,
				callback: function () {
					this.isSlideInActive = false;
				}
			});
		}
	}
});

tinymce.Editor.prototype.hide = function () {
	var t = this, d = t.getDoc();

	// Fixed bug where IE has a blinking cursor left from the editor
	if (tinymce.isIE && d) {
		d.execCommand('SelectAll');
	}

	// We must save before we hide so Safari doesn't crash
	// :INFO: echernodarov 20100421: we are not needed in "save" call at "hide" method
	//t.save();
	tinymce.DOM.hide(t.getContainer());
	tinymce.DOM.setStyle(t.id, 'display', t.orgDisplay);
};

tinymce.ui.ColorSplitButton.prototype.displayColor = function(c) {
	var t = this;

	tinymce.DOM.setStyle(t.id + '_preview', 'backgroundColor', Sb.Editor.model.design.getColor(c));

	t.value = c;
};

tinymce.ui.ColorSplitButton.prototype.onColorMenuHide = function (menu) {
	tinymce.activeEditor.selection.moveToBookmark(this.selectionBookmark);
	if (!menu.isCancelled) {
		this.setColor(menu.value);
	}
	tinymce.activeEditor.getWin().focus();
};

tinymce.ui.ColorSplitButton.prototype.showMenu = function () {
	this.selectionBookmark = tinymce.activeEditor.selection.getBookmark(1);
	if (!this.menu) {
		this.menu = new Sb.menu.ColorMenu({
			allowBlank: true,
			allowColorPalette: true,
			isShowColorField: true
		});
		this.menu.on('hide', this.onColorMenuHide, this);
	}
	if (this.menu.hidden) {
		this.menu.show(this.id, "tl-bl?");
	}
	this.menu.setValue(this.value);
};

tinymce.ui.ColorSplitButton.prototype.hideMenu = function () {
	if (this.menu && !this.menu.hidden) {
		this.menu.hide();
	}
};

tinymce.ui.ColorSplitButton.prototype.destroy = function () {
	this.parent();

	tinymce.dom.Event.clear(this.id + '_menu');
	tinymce.dom.Event.clear(this.id + '_more');
	tinymce.DOM.remove(this.id + '_menu');

	if (this.menu && !this.menu.isDestroyed) {
		this.menu.destroy();
	}
};
