/**
 * @class Sb.widgetType.Base
 */
Ext.define('Sb.widgetType.Base', {
	extend: 'Ext.util.Observable',

	/**
	 * Name of the type
	 *
	 * @type String
	 */
	name: '',

	/**
	 * Does widgets of this type will work with widgetTitle?
	 *
	 * @type Boolean
	 */
	hasWidgetTitle: false,

	/**
	 * Source of CSS template that will be compiled
	 *
	 * @type Array
	 * @property cssTemplateSource
	 */
	cssTemplateSource: null,

	/**
	 * Source of color scheme CSS template that will be compiled
	 *
	 * @type Array
	 * @property colorSchemeCssTemplateSource
	 */
	colorSchemeCssTemplateSource: null,

	/**
	 * Source of font color CSS template that will be compiled
	 *
	 * @type Array
	 * @property fontColorCssTemplateSource
	 */
	fontColorCssTemplateSource: null,

	/**
	 * The reason why widget was disabled
	 *
	 * @type String
	 * @property disableReason
	 */
	disableReason: '',

	/**
	 * @type {Boolean}
	 */
	isUpsell: false,

	allowedColors: 'text,link,h1,h2',

	constructor: function (name) {
		if (name) {
			this.name = name;
		}

		/**
		 * Components, that is registered for each type
		 *
		 * @type Ext.util.MixedCollection
		 * @property components
		 */
		this.components = new Ext.util.MixedCollection();

		this.components.on('add', this.onComponentsAdd, this);

		/**
		 * Cache of widgets by uuid
		 *
		 * @type Ext.util.MixedCollection
		 * @property cache
		 */
		this.cache = new Ext.util.MixedCollection(false,
			function (widget) {
				return widget.data.uuid;
			}
		);

		this.addEvents('isupsellchange');
		// need for zero limits
		this.checkUpsellChange();

		this.cache.on('add', this.onCacheChange, this);
		this.cache.on('remove', this.onCacheChange, this);
	},

	/**
	 * Initialize type (clear cache, etc.)
	 */
	init: function () {
		this.cache.clear();
	},

	/**
	 * Fires when number of shared widgets in site changed
	 */
	onCacheChange: function () {
		this.toggleButton();
		this.checkUpsellChange();
	},

	checkUpsellChange: function () {
		if (Sb.Limits.isUpsellWidgets(Sb.Editor.model, this.getPhpType(), this.cache) !== this.isUpsell) {
			this.isUpsell = !this.isUpsell;
			this.fireEvent('isupsellchange', this.isUpsell);
		}

		if (Sb.Editor.Ui.mainWindow) {
			Sb.Editor.Ui.mainWindow.updatePublishButton();
		}
	},

	/**
	 * Automatic conversion to string for simple usage cases
	 * @return {String}
	 */
	toString: function () {
		return this.name;
	},

	/**
	 * Gets type for creating child
	 * @return {String}
	 */
	getPhpType: function () {
		return this.name;
	},

	/**
	 * Get key for collections
	 * @return {String}
	 */
	getKey: function () {
		return this.name;
	},

	/**
	 * Get title for presentation elements
	 * @return {String}
	 */
	/**
	 * Return translation for localization key
	 * @param {String} key localization key
	 * @return {String} translation
	 */
	getTranslation: function (key) {
		arguments[0] = 'Widgets/' + this.name.toUpperCaseFirst() + '/' + key;
		return Sb.Locale.get.apply(Sb.Locale, arguments);
	},

	/**
	 * Toggle all registered components to state
	 * @param {Boolean} state
	 */
	toggle: function (state) {
		this.components.each(function (cmp) {
			cmp[state ? 'enable' : 'disable']();
		});
	},

	/**
	 * Toggle all registered components to state according current availability
	 */
	toggleButton: function () {
		this.toggle(this.isAvailable());
	},

	/**
	 * Enables all registered components
	 */
	enable: function () {
		this.toggle(true);
	},

	/**
	 * Disable all registered components
	 */
	disable: function () {
		this.toggle(false);
	},

	/**
	 * Determine whether we can drop widget to container
	 *
	 * @param {Sb.model.Widget} container
	 */
	isDroppableTo: function (container) {
		return true;
	},

	/**
	 * Determine whether we can drop widget to column
	 */
	isDroppableToColumn: function () {
		return true;
	},

	/**
	 * Determine whether we can create new instances
	 */
	isAvailable: function () {
		return true;
	},

	/**
	 * Fires when component register
	 */
	onComponentsAdd: Ext.emptyFn,

	/**
	 * @param {Object} [config] The configuration options to create new instance of edit panel
	 */
	getEditPanel: function (config) {
		var className = this.name.toUpperCaseFirst()+ 'EditPanel';
		if (this.editPanel === undefined) {
			this.editPanel = Sb.widget[className] ? new Sb.widget[className](config) : null;
		}
		return this.editPanel;
	},

	/**
	 * Gets CSS template of shared widget by key.
	 * @return {Ext.XTemplate}
	 */
	getCssTemplateByKey: function (key) {
		var templateKey = (key + 'CssTemplate').toLowerCaseFirst(),
			sourceKey	= (key + 'CssTemplateSource').toLowerCaseFirst();

		if (null == this[templateKey]) {
			this[templateKey] = new Ext.XTemplate(this[sourceKey]);
			this[templateKey].compile();
		}
		return this[templateKey];
	},

	/**
	 * Gets CSS template of shared widget.
	 * @return {Ext.XTemplate}
	 */
	getCssTemplate: function () {
		return this.getCssTemplateByKey('');
	},

	/**
	 * Gets color scheme CSS template of shared widget.
	 * @return {Ext.XTemplate}
	 */
	getColorSchemeCssTemplate: function () {
		return this.getCssTemplateByKey('colorScheme');
	},

	/**
	 * Gets font color CSS template of shared widget.
	 * @return {Ext.XTemplate}
	 */
	getFontColorCssTemplate: function () {
		return this.getCssTemplateByKey('fontColor');
	}
});
