/*!
 * @class Ext.ux.util.Color
 * Utility class for color manipulating
 * @singleton
 */
Ext.define('Ext.ux.util.Color', {
	singleton: true,

	/**
	 * Convert color from RGB triplet [0..1, 0..1, 0..1] to HTML syntax #123456
	 * @param {Array} rgb
	 * @return {String}
	 */
	pack: function (rgb) {
		var r, g, b, color;
		r = Math.round(rgb[0] * 255);
		g = Math.round(rgb[1] * 255);
		b = Math.round(rgb[2] * 255);
		color = '#' + (r < 16 ? '0' : '') + r.toString(16) +
			(g < 16 ? '0' : '') + g.toString(16) +
			(b < 16 ? '0' : '') + b.toString(16);
		return color.toUpperCase();
	},

	/**
	 * Convert color from HTML syntax #123456 to RGB triplet [0..1, 0..1, 0..1]
	 * @param {String} color
	 * @return {Array}
	 */
	unpack: function (color) {
		if (color.length == 7) {
			return [parseInt('0x' + color.substring(1, 3), 16) / 255,
				parseInt('0x' + color.substring(3, 5), 16) / 255,
				parseInt('0x' + color.substring(5, 7), 16) / 255];
		}
		if (color.length == 4) {
			return [parseInt('0x' + color.substring(1, 2), 16) / 15,
				parseInt('0x' + color.substring(2, 3), 16) / 15,
				parseInt('0x' + color.substring(3, 4), 16) / 15];
		}
	},

	/**
	 * Convert color from HSL triplet [0..1, 0..1, 0..1] to RGB triplet [0..1, 0..1, 0..1]
	 * @param {Array} hsl
	 * @return {Array}
	 */
	hsl2rgb: function (hsl) {
		var m1, m2, h, s, l;
		h = hsl[0];
		s = hsl[1];
		l = hsl[2];
		m2 = (l <= 0.5) ? l * (s + 1) : l + s - l * s;
		m1 = l * 2 - m2;
		return [Ext.ux.util.Color.hue2rgb(m1, m2, h + 0.33333),
			Ext.ux.util.Color.hue2rgb(m1, m2, h),
			Ext.ux.util.Color.hue2rgb(m1, m2, h - 0.33333)];
	},

	/**
	 * Convert color from HUE to RGB triplet [0..1, 0..1, 0..1]
	 * @param {Number} m1
	 * @param {Number} m2
	 * @param {Number} h
	 * @return {Number}
	 */
	hue2rgb: function (m1, m2, h) {
		h = (h < 0) ? h + 1 : ((h > 1) ? h - 1 : h);
		if (h * 6 < 1) {
			return m1 + (m2 - m1) * h * 6;
		}
		if (h * 2 < 1) {
			return m2;
		}
		if (h * 3 < 2) {
			return m1 + (m2 - m1) * (0.66666 - h) * 6;
		}
		return m1;
	},

	/**
	 * Convert color from RGB triplet [0..1, 0..1, 0..1] to HSL triplet [0..1, 0..1, 0..1]
	 * @param {Array} rgb
	 * @return {Array}
	 */
	rgb2hsl: function (rgb) {
		var min, max, delta, h, s, l, r, g, b;
		r = rgb[0];
		g = rgb[1];
		b = rgb[2];
		min = Math.min(r, Math.min(g, b));
		max = Math.max(r, Math.max(g, b));
		delta = max - min;
		l = (min + max) / 2;
		s = 0;
		if (l > 0 && l < 1) {
			s = delta / (l < 0.5 ? (2 * l) : (2 - 2 * l));
		}
		h = 0;
		if (delta > 0) {
			if (max == r && max != g) {
				h += (g - b) / delta;
			}
			if (max == g && max != b) {
				h += (2 + (b - r) / delta);
			}
			if (max == b && max != r) {
				h += (4 + (r - g) / delta);
			}
			h /= 6;
		}
		return [h, s, l];
	},

	/**
	 * Checks color is light
	 * @return {Boolean}
	 */
	isLight: function (color) {
		var rgb = Ext.ux.util.Color.unpack(color),
			hsl = Ext.ux.util.Color.rgb2hsl(rgb);
		return (hsl[0] < 0.55 && hsl[2] >= 0.5) || (hsl[0] >= 0.55 && hsl[2] >= 0.75);
	},

	/**
	 * Checks color is dark
	 * @return {Boolean}
	 */
	isDark: function (color) {
		return !Ext.ux.util.Color.isLight(color);
	},

	/**
	 * Get variant of color with the adjusted saturation and lightness
	 * @param {String} color Color in HTML syntax #123456
	 * @param {Number} dS Saturation delta -1..1
	 * @param {Number} dL Lightness delta -1..1
	 * @return {String}
	 */
	getVariant: function (color, dS, dL) {
		var hsl = Ext.ux.util.Color.rgb2hsl(Ext.ux.util.Color.unpack(color));
		hsl[1] = dS <= 0 ? hsl[1] * (dS + 1) : hsl[1] + (1 - hsl[1]) * dS;
		hsl[2] = dL <= 0 ? hsl[2] * (dL + 1) : hsl[2] + (1 - hsl[2]) * dL;

		return Ext.ux.util.Color.pack(Ext.ux.util.Color.hsl2rgb(hsl));
	},

	/**
	 * Check is valid color
	 * @param {String} color
	 * @returns {Boolean}
	 */
	isValid: function (color) {
		return (/^#([0-9A-F]{3}|[0-9A-F]{6})$/i).test(color);
	},

	/**
	 * Convert color from HTML syntax #123456 and opacity (0-100) to RGBA [0..255, 0..255, 0..255, 0..1]
	 * @param {String} color
	 * @param {Number} opacity 0-100
	 * @return {Array}
	 */
	hex2rgba: function (color, opacity) {
		var rgba = [];

		if (opacity === undefined) {
			opacity = 100;
		}

		if (color.length == 7) {
			rgba[0] = parseInt('0x' + color.substring(1, 3), 16);
			rgba[1] = parseInt('0x' + color.substring(3, 5), 16);
			rgba[2] = parseInt('0x' + color.substring(5, 7), 16);
		}
		if (color.length == 4) {
			rgba[0] = parseInt('0x' + color.substring(1, 2), 16) * 17;
			rgba[1] = parseInt('0x' + color.substring(2, 3), 16) * 17;
			rgba[2] = parseInt('0x' + color.substring(3, 4), 16) * 17;
		}

		rgba[3] = (opacity / 100).toFixed(2);

		return rgba;
	},

	/**
	 * Apply foreground to background color
	 * @param {Array} background RGBA [0..255, 0..255, 0..255, 0..1]
	 * @param {Array} foreground RGBA [0..255, 0..255, 0..255, 0..1]
	 * @return {Array}
	 */
	apply: function (background, foreground) {
		var alpha, backgroundAlpha, rgba = [];

		alpha = 1 - (1 - foreground[3]) * (1 - background[3]);
		backgroundAlpha = (1 - foreground[3]) * background[3] * background[0];
		rgba[0] = parseInt((foreground[3] * foreground[0] + backgroundAlpha) / alpha, 10);
		rgba[1] = parseInt((foreground[3] * foreground[1] + backgroundAlpha) / alpha, 10);
		rgba[2] = parseInt((foreground[3] * foreground[2] + backgroundAlpha) / alpha, 10);
		rgba[3] = alpha.toFixed(2);

		return rgba;
	}
});