/*!
 * @class Ext.ux.ColorPicker
 * @extends Ext.BoxComponent
 */
Ext.define('Ext.ux.ColorPicker', {
	extend: 'Ext.BoxComponent',
	xtype: 'colorpicker',

	/**
	 * @cfg {Number} value The color to initialize the color picker.
	 */

	/**
	 * @cfg {Number} delayPeriod Number of milliseconds, that colorpicker change delayed.
	 */

	// private properties
	_radius	: 84,
	_square	: 100,
	_width	: 194,

	constructor: function (config) {
		config = Ext.apply({
			delayPeriod: 0,
			value: '#000000'
		}, config);

		Ext.ux.ColorPicker.superclass.constructor.call(this, config);
	},


	// private override
	initComponent: function () {

		Ext.ux.ColorPicker.superclass.initComponent.apply(this, arguments);

		this.addEvents('change', 'select', 'mouseup');

		this.setRawValue(this.value);
		delete this.value;
	},

	// private override
	onRender: function () {
		this.autoEl = {
			cls: 'x-colorpicker',
			cn: [{
				cls: 'x-colorpicker-h'
			}, {
				cls: 'x-colorpicker-sl'
			}, {
				cls: 'x-colorpicker-thumb x-colorpicker-thumb-h'
			}, {
				cls: 'x-colorpicker-thumb x-colorpicker-thumb-sl'
			}]
		};
		Ext.ux.ColorPicker.superclass.onRender.apply(this, arguments);
		this.hEl = this.el.first();
		this.slEl = this.hEl.next();
		this.hThumb = this.slEl.next();
		this.slThumb = this.hThumb.next();
		this.initEvents();
	},

	// private override
	initEvents: function () {
		this.mon(this.el, 'mousedown', this.onMouseDown, this);
	},

	/**
	 * Mousedown handler
	 */
	onMouseDown: function (e) {
		e.stopEvent();

		if (this.disabled) {
			return;
		}

		this.focus();

		if (!this._dragging) {
			Ext.getDoc().on('mousemove', this.onMouseMove, this);
			Ext.getDoc().on('mouseup', this.onMouseUp, this);
			this._dragging = true;
		}

		// Check which area is being dragged
		var pos = this.el.translatePoints(e.getXY());
		pos.left -= this._width / 2;
		pos.top -= this._width / 2;

		this._isCircleDrag = Math.max(Math.abs(pos.left), Math.abs(pos.top)) * 2 > this._square;

		this.onMouseMove(e);
	},

	/**
	 * Mousemove handler
	 */
	onMouseMove: function (e) {
		// Get coordinates relative to color picker center
		var pos = this.el.translatePoints(e.getXY());
		pos.left -= this._width / 2;
		pos.top -= this._width / 2;

		// Set new HSL parameters
		if (this._isCircleDrag) {
			var hue = Math.atan2(pos.left, -pos.top) / 6.28;
			if (hue < 0) {
				hue += 1;
			}
			this.setHSL([hue, this.hsl[1], this.hsl[2]]);
		} else {
			var sat = Math.max(0, Math.min(1, -(pos.left / this._square) + 0.5));
			var lum = Math.max(0, Math.min(1, -(pos.top / this._square) + 0.5));
			this.setHSL([this.hsl[0], sat, lum]);
		}

		this.delayChange();
	},

	/**
	 * Last delayed change time in milliseconds
	 *
	 * @type Number
	 * @property lastChangeTime
	 */
	lastChangeTime: (new Date()).getTime(),

	/**
	 * Delay change event for specified number of milliseconds
	 */
	delayChange: function () {
		if (this.delayPeriod > 0) {
			var now			= (new Date()).getTime();
			var difference	= now - this.lastChangeTime;
			if (difference > this.delayPeriod) {
				this.lastChangeTime = now;
			} else {
				return;
			}
		}
		this.fireEvent('change', this, this.color);
	},

	/**
	 * Mouseup handler
	 */
	onMouseUp: function (e) {
		Ext.getDoc().un('mousemove', this.onMouseMove, this);
		Ext.getDoc().un('mouseup', this.onMouseUp, this);
		this._dragging = false;
		if (this.delayPeriod > 0) {
			this.fireEvent('change', this, this.color);
		}
		this.fireEvent('select', this, this.color);
		this.fireEvent('mouseup', this, this.color);
	},

	// private override
	afterRender: function () {
		Ext.ux.ColorPicker.superclass.afterRender.apply(this, arguments);
		this.updateDisplay();
	},

	/**
	 * Is value actually changed?
	 *
	 * @type Boolean
	 * @property isChanged
	 */
	isChanged: true,

	/**
	 * Change color with HTML syntax #123456
	 *
	 * @param {String} color The color to set the color picker
	 */
	setRawValue: function (color) {
		var unpack		= Ext.ux.util.Color.unpack(color);
		if (this.color != color && unpack) {
			this.color	= color.toUpperCase();
			this.rgb	= unpack;
			this.hsl	= Ext.ux.util.Color.rgb2hsl(this.rgb);

			if (this.rendered) {
				this.updateDisplay();
			}
			this.isChanged = true;
		}
		return this;
	},

	/**
	 * Changes color with HTML syntax #123456 and fires change event
	 * @param {String} color The color to set the color picker
	 */
	setValue: function (color) {
		this.setRawValue(color);
		if (this.isChanged) {
			this.fireEvent('change', this, this.color);
		}
	},

	/**
	 * Returns color with HTML syntax #123456
	 * @return {String}
	 */
	getValue: function (color) {
		return this.color;
	},

	/**
	 * Change color with HSL triplet [0..1, 0..1, 0..1]
	 * @param {Array} hsl
	 */
	setHSL: function (hsl) {
		this.hsl = hsl;
		this.rgb = Ext.ux.util.Color.hsl2rgb(hsl);
		this.color = Ext.ux.util.Color.pack(this.rgb);

		if (this.rendered) {
			this.updateDisplay();
		}

		return this;
	},

	/**
	 * Update the markers and styles
	 */
	updateDisplay: function () {
		// Markers
		var angle = this.hsl[0] * 6.28;
		this.hThumb.setStyle({
			left: Math.round(Math.sin(angle) * this._radius + this._width / 2) + 'px',
			top: Math.round(-Math.cos(angle) * this._radius + this._width / 2) + 'px'
		});

		this.slThumb.setStyle({
			left: Math.round(this._square * (0.5 - this.hsl[1]) + this._width / 2) + 'px',
			top: Math.round(this._square * (0.5 - this.hsl[2]) + this._width / 2) + 'px'
		});

		this.slEl.setStyle('backgroundColor', Ext.ux.util.Color.pack(Ext.ux.util.Color.hsl2rgb([this.hsl[0], 1, 0.5])));
	}
});